function getOrthogonalVectors(t, i, e) {
  var n, o;
  Math.abs(t.z) > 0.7071067811865476
    ? ((n = t.y * t.y + t.z * t.z),
      (o = 1 / Math.sqrt(n)),
      (i.x = 0),
      (i.y = -t.z * o),
      (i.z = t.y * o),
      (e.x = n * o),
      (e.y = -t.x * i.z),
      (e.z = t.x * i.y))
    : ((n = t.x * t.x + t.y * t.y),
      (o = 1 / Math.sqrt(n)),
      (i.x = -t.y * o),
      (i.y = t.x * o),
      (i.z = 0),
      (e.x = -t.z * i.y),
      (e.y = t.z * i.x),
      (e.z = n * o));
}
var PointToPointConstraint = pc.createScript("pointToPointConstraint");
PointToPointConstraint.attributes.add("pivotA", {
  title: "Pivot",
  description: "Position of the constraint in the local space of this entity.",
  type: "vec3",
  default: [0, 0, 0],
}),
  PointToPointConstraint.attributes.add("entityB", {
    title: "Connected Entity",
    description: "Optional second entity",
    type: "entity",
  }),
  PointToPointConstraint.attributes.add("pivotB", {
    title: "Connected Pivot",
    description:
      "Position of the constraint in the local space of entity B (if specified).",
    type: "vec3",
    default: [0, 0, 0],
  }),
  PointToPointConstraint.attributes.add("breakingThreshold", {
    title: "Break Threshold",
    description:
      "Maximum breaking impulse threshold required to break the constraint.",
    type: "number",
    default: 34e37,
  }),
  PointToPointConstraint.attributes.add("enableCollision", {
    title: "Enable Collision",
    description: "Enable collision between linked rigid bodies.",
    type: "boolean",
    default: !0,
  }),
  PointToPointConstraint.attributes.add("debugRender", {
    title: "Debug Render",
    description: "Enable to render a representation of the constraint.",
    type: "boolean",
    default: !1,
  }),
  PointToPointConstraint.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  (PointToPointConstraint.prototype.initialize = function () {
    this.createConstraint(),
      this.on("attr", function (t, i, e) {
        "pivotA" === t || "entityB" === t || "pivotB" === t
          ? this.createConstraint()
          : "breakingThreshold" === t &&
            (this.constraint.setBreakingImpulseThreshold(
              this.breakingThreshold
            ),
            this.activate());
      }),
      this.on("enable", function () {
        this.createConstraint();
      }),
      this.on("disable", function () {
        this.destroyConstraint();
      }),
      this.on("destroy", function () {
        this.destroyConstraint();
      });
  }),
  (PointToPointConstraint.prototype.createConstraint = function () {
    this.constraint && this.destroyConstraint();
    var t = this.entity.rigidbody.body,
      i = new Ammo.btVector3(this.pivotA.x, this.pivotA.y, this.pivotA.z);
    if (this.entityB && this.entityB.rigidbody) {
      var e = this.entityB.rigidbody.body,
        n = new Ammo.btVector3(this.pivotB.x, this.pivotB.y, this.pivotB.z);
      this.constraint = new Ammo.btPoint2PointConstraint(t, e, i, n);
    } else this.constraint = new Ammo.btPoint2PointConstraint(t, i);
    this.app.systems.rigidbody.dynamicsWorld.addConstraint(
      this.constraint,
      !this.enableCollision
    ),
      this.activate();
  }),
  (PointToPointConstraint.prototype.destroyConstraint = function () {
    this.constraint &&
      (this.app.systems.rigidbody.dynamicsWorld.removeConstraint(
        this.constraint
      ),
      Ammo.destroy(this.constraint),
      (this.constraint = null));
  }),
  (PointToPointConstraint.prototype.activate = function () {
    this.entity.rigidbody.activate(),
      this.entityB && this.entityB.rigidbody.activate();
  }),
  (PointToPointConstraint.prototype.update = function (t) {
    if (this.debugRender) {
      var i = new pc.Vec3();
      this.entity.getWorldTransform().transformPoint(this.pivotA, i),
        this.app.renderLine(this.entity.getPosition(), i, this.debugColor),
        this.entityB &&
          this.app.renderLine(this.entityB.getPosition(), i, this.debugColor);
    }
  });
var HingeConstraint = pc.createScript("hingeConstraint");
HingeConstraint.attributes.add("entityA", {
  title: "Entity",
  description: "Connected entity.",
  type: "entity",
}),
  HingeConstraint.attributes.add("pivotA", {
    title: "Pivot",
    description:
      "Position of the constraint in the local space of this entity.",
    type: "vec3",
    default: [0, 0, 0],
  }),
  HingeConstraint.attributes.add("axisA", {
    title: "Axis",
    description:
      "Axis of rotation of the constraint in the local space this entity.",
    type: "vec3",
    default: [0, 1, 0],
  }),
  HingeConstraint.attributes.add("entityB", {
    title: "Connected Entity",
    description: "Optional second connected entity.",
    type: "entity",
  }),
  HingeConstraint.attributes.add("pivotB", {
    title: "Connected Pivot",
    description:
      "Position of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 0, 0],
  }),
  HingeConstraint.attributes.add("axisB", {
    title: "Connected Axis",
    description:
      "Axis of rotation of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 1, 0],
  }),
  HingeConstraint.attributes.add("limits", {
    title: "Limits",
    description:
      "Low and high angular limits for the constraint in degrees. By default, low is greater than high meaning no limits.",
    type: "vec2",
    default: [1, -1],
  }),
  HingeConstraint.attributes.add("softness", {
    title: "Softness",
    description:
      'Softness of the constraint. Recommend 0.8 to 1. Describes the percentage of limits where movement is free. Beyond this softness percentage, the limit is gradually enforced until the "hard" (1.0) limit is reached.',
    type: "number",
    min: 0,
    max: 1,
    default: 0.9,
  }),
  HingeConstraint.attributes.add("biasFactor", {
    title: "Bias Factor",
    description:
      "Bias factor of the constraint. Recommend 0.3 +/- approximately 0.3. Strength with which constraint resists zeroth order (angular, not angular velocity) limit violation.",
    type: "number",
    min: 0,
    max: 1,
    default: 0.3,
  }),
  HingeConstraint.attributes.add("relaxationFactor", {
    title: "Relaxation Factor",
    description:
      "Relaxation factor of the constraint. Recommend to keep this near 1. The lower the value, the less the constraint will fight velocities which violate the angular limits.",
    type: "number",
    min: 0,
    max: 1,
    default: 1,
  }),
  HingeConstraint.attributes.add("enableMotor", {
    title: "Use Motor",
    description:
      "Enable a motor to power the automatic rotation around the hinge axis.",
    type: "boolean",
    default: !1,
  }),
  HingeConstraint.attributes.add("motorTargetVelocity", {
    title: "Target Velocity",
    description: "Target motor angular velocity.",
    type: "number",
    default: 0,
  }),
  HingeConstraint.attributes.add("maxMotorImpulse", {
    title: "Max Motor Impulse",
    description: "Maximum motor impulse.",
    type: "number",
    default: 0,
  }),
  HingeConstraint.attributes.add("breakingThreshold", {
    title: "Break Threshold",
    description:
      "Maximum breaking impulse threshold required to break the constraint.",
    type: "number",
    default: 34e37,
  }),
  HingeConstraint.attributes.add("enableCollision", {
    title: "Enable Collision",
    description: "Enable collision between linked rigid bodies.",
    type: "boolean",
    default: !0,
  }),
  HingeConstraint.attributes.add("debugRender", {
    title: "Debug Render",
    description: "Enable to render a representation of the constraint.",
    type: "boolean",
    default: !1,
  }),
  HingeConstraint.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  (HingeConstraint.prototype.initialize = function () {
    this.createConstraint(),
      this.on("attr", (t, i, e) => {
        if (
          "pivotA" === t ||
          "axisA" === t ||
          "entityB" === t ||
          "pivotB" === t ||
          "axisB" === t
        )
          this.createConstraint();
        else if (
          "limits" === t ||
          "softness" === t ||
          "biasFactor" === t ||
          "relaxationFactor" === t
        ) {
          var n = (this.limits.x * Math.PI) / 180,
            o = (this.limits.y * Math.PI) / 180;
          this.constraint.setLimit(
            n,
            o,
            this.softness,
            this.biasFactor,
            this.relaxationFactor
          );
        } else
          "enableMotor" === t ||
          "motorTargetVelocity" === t ||
          "maxMotorImpulse" === t
            ? (this.constraint.enableAngularMotor(
                this.enableMotor,
                (this.motorTargetVelocity * Math.PI) / 180,
                this.maxMotorImpulse
              ),
              this.activate())
            : "breakingThreshold" === t &&
              (this.constraint.setBreakingImpulseThreshold(
                this.breakingThreshold
              ),
              this.activate());
      }),
      this.on("enable", function () {
        this.createConstraint();
      }),
      this.on("disable", function () {
        this.destroyConstraint();
      }),
      this.on("destroy", function () {
        this.destroyConstraint();
      });
  }),
  (HingeConstraint.prototype.createConstraint = function () {
    this.constraint && this.destroyConstraint();
    var t = new pc.Vec3(),
      i = new pc.Vec3(),
      e = new pc.Quat(),
      n = new pc.Mat4();
    this.entityA ? (this.entityA = this.entityA) : (this.entityA = this.entity);
    var o = this.entityA.rigidbody.body,
      s = new Ammo.btVector3(this.pivotA.x, this.pivotA.y, this.pivotA.z);
    getOrthogonalVectors(this.axisA, t, i),
      n.set([
        t.x,
        t.y,
        t.z,
        0,
        i.x,
        i.y,
        i.z,
        0,
        this.axisA.x,
        this.axisA.y,
        this.axisA.z,
        0,
        0,
        0,
        0,
        1,
      ]),
      e.setFromMat4(n);
    var r = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
      a = new Ammo.btTransform(r, s);
    if (this.entityB && this.entityB.rigidbody) {
      var d = this.entityB.rigidbody.body,
        h = new Ammo.btVector3(this.pivotB.x, this.pivotB.y, this.pivotB.z);
      getOrthogonalVectors(this.axisB, t, i),
        n.set([
          t.x,
          t.y,
          t.z,
          0,
          i.x,
          i.y,
          i.z,
          0,
          this.axisB.x,
          this.axisB.y,
          this.axisB.z,
          0,
          0,
          0,
          0,
          1,
        ]),
        e.setFromMat4(n);
      var c = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
        l = new Ammo.btTransform(c, h);
      (this.constraint = new Ammo.btHingeConstraint(o, d, a, l, !1)),
        Ammo.destroy(l),
        Ammo.destroy(c),
        Ammo.destroy(h);
    } else this.constraint = new Ammo.btHingeConstraint(o, a, !1);
    var p = (this.limits.x * Math.PI) / 180,
      y = (this.limits.y * Math.PI) / 180;
    this.constraint.setLimit(
      p,
      y,
      this.softness,
      this.biasFactor,
      this.relaxationFactor
    ),
      this.constraint.setBreakingImpulseThreshold(this.breakingThreshold),
      this.constraint.enableAngularMotor(
        this.enableMotor,
        (this.motorTargetVelocity * Math.PI) / 180,
        this.maxMotorImpulse
      ),
      Ammo.destroy(a),
      Ammo.destroy(r),
      Ammo.destroy(s),
      this.app.systems.rigidbody.dynamicsWorld.addConstraint(
        this.constraint,
        !this.enableCollision
      ),
      this.activate();
  }),
  (HingeConstraint.prototype.destroyConstraint = function () {
    this.constraint &&
      (this.app.systems.rigidbody.dynamicsWorld.removeConstraint(
        this.constraint
      ),
      Ammo.destroy(this.constraint),
      (this.constraint = null));
  }),
  (HingeConstraint.prototype.activate = function () {
    this.entityA.rigidbody.activate(),
      this.entityB && this.entityB.rigidbody.activate();
  }),
  (HingeConstraint.prototype.update = function (t) {
    if (this.debugRender) {
      var i = new pc.Vec3(),
        e = new pc.Vec3(),
        n = new pc.Vec3(),
        o = new pc.Vec3(),
        s = this.entityA.getWorldTransform();
      s.transformPoint(this.pivotA, i),
        s.transformVector(this.axisA, e),
        e.normalize().scale(0.5),
        n.add2(i, e),
        o.sub2(i, e),
        this.app.renderLine(this.entityA.getPosition(), i, this.debugColor),
        this.app.renderLine(n, o, this.debugColor),
        this.entityB &&
          this.app.renderLine(this.entityB.getPosition(), i, this.debugColor);
    }
  });
var ConeTwistConstraint = pc.createScript("coneTwistConstraint");
ConeTwistConstraint.attributes.add("pivotA", {
  title: "Pivot",
  description: "Position of the constraint in the local space of this entity.",
  type: "vec3",
  default: [0, 0, 0],
}),
  ConeTwistConstraint.attributes.add("axisA", {
    title: "Axis",
    description:
      "Axis of rotation of the constraint in the local space this entity.",
    type: "vec3",
    default: [0, 1, 0],
  }),
  ConeTwistConstraint.attributes.add("entityB", {
    title: "Connected Entity",
    description: "Optional second connected entity.",
    type: "entity",
  }),
  ConeTwistConstraint.attributes.add("pivotB", {
    title: "Connected Pivot",
    description:
      "Position of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 0, 0],
  }),
  ConeTwistConstraint.attributes.add("axisB", {
    title: "Connected Axis",
    description:
      "Axis of rotation of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 1, 0],
  }),
  ConeTwistConstraint.attributes.add("swingSpan1", {
    title: "Swing 1 Limit",
    description:
      "The Swing 1 Limit limits the rotation around the swing axis. Swing 1 Limit limits an axis of rotation orthogonal to the twist axis and Swing Limit 2. The limit angle is symmetric. Therefore, a value of 20 will limit the rotation between -20 and 20 degrees.",
    type: "number",
    default: 1e30,
  }),
  ConeTwistConstraint.attributes.add("swingSpan2", {
    title: "Swing 2 Limit",
    description:
      "The Swing 2 Limit limits the rotation around the swing axis. Swing 2 Limit limits an axis of rotation orthogonal to the twist axis and Swing Limit 1. The limit angle is symmetric. Therefore, a value of 20 will limit the rotation between -20 and 20 degrees.",
    type: "number",
    default: 1e30,
  }),
  ConeTwistConstraint.attributes.add("twistSpan", {
    title: "Twist Limit",
    description:
      "The Twist Limit limits the rotation around the twist axis. The limit angle is symmetric. Therefore, a value of 20 will limit the twist rotation between -20 and 20 degrees.",
    type: "number",
    default: 1e30,
  }),
  ConeTwistConstraint.attributes.add("breakingThreshold", {
    title: "Break Threshold",
    description:
      "Maximum breaking impulse threshold required to break the constraint.",
    type: "number",
    default: 1e30,
  }),
  ConeTwistConstraint.attributes.add("enableCollision", {
    title: "Enable Collision",
    description: "Enable collision between linked rigid bodies.",
    type: "boolean",
    default: !0,
  }),
  ConeTwistConstraint.attributes.add("debugRender", {
    title: "Debug Render",
    description: "Enable to render a representation of the constraint.",
    type: "boolean",
    default: !1,
  }),
  ConeTwistConstraint.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  (ConeTwistConstraint.prototype.initialize = function () {
    this.createConstraint(),
      this.on("attr", function (t, i, e) {
        "pivotA" === t ||
        "axisA" === t ||
        "entityB" === t ||
        "pivotB" === t ||
        "axisB" === t
          ? this.createConstraint()
          : "swingSpan1" === t
          ? (this.constraint.setLimit(5, (this.swingSpan1 * Math.PI) / 180),
            this.activate())
          : "swingSpan2" === t
          ? (this.constraint.setLimit(4, (this.swingSpan2 * Math.PI) / 180),
            this.activate())
          : "twistSpan" === t
          ? (this.constraint.setLimit(3, (this.twistSpan * Math.PI) / 180),
            this.activate())
          : "breakingThreshold" === t &&
            (this.constraint.setBreakingImpulseThreshold(
              this.breakingThreshold
            ),
            this.activate());
      }),
      this.on("enable", function () {
        this.createConstraint();
      }),
      this.on("disable", function () {
        this.destroyConstraint();
      }),
      this.on("destroy", function () {
        this.destroyConstraint();
      });
  }),
  (ConeTwistConstraint.prototype.createConstraint = function () {
    this.constraint && this.destroyConstraint();
    var t = new pc.Vec3(),
      i = new pc.Vec3(),
      e = new pc.Quat(),
      n = new pc.Mat4(),
      o = this.entity.rigidbody.body,
      s = new Ammo.btVector3(this.pivotA.x, this.pivotA.y, this.pivotA.z);
    getOrthogonalVectors(this.axisA, t, i),
      n.set([
        t.x,
        t.y,
        t.z,
        0,
        i.x,
        i.y,
        i.z,
        0,
        this.axisA.x,
        this.axisA.y,
        this.axisA.z,
        0,
        0,
        0,
        0,
        1,
      ]),
      e.setFromMat4(n);
    var r = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
      a = new Ammo.btTransform(r, s);
    if (this.entityB && this.entityB.rigidbody) {
      var d = this.entityB.rigidbody.body,
        h = new Ammo.btVector3(this.pivotB.x, this.pivotB.y, this.pivotB.z);
      getOrthogonalVectors(this.axisB, t, i),
        n.set([
          t.x,
          t.y,
          t.z,
          0,
          i.x,
          i.y,
          i.z,
          0,
          this.axisB.x,
          this.axisB.y,
          this.axisB.z,
          0,
          0,
          0,
          0,
          1,
        ]),
        e.setFromMat4(n);
      var c = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
        l = new Ammo.btTransform(c, h);
      (this.constraint = new Ammo.btConeTwistConstraint(o, d, a, l)),
        Ammo.destroy(l),
        Ammo.destroy(c),
        Ammo.destroy(h);
    } else this.constraint = new Ammo.btConeTwistConstraint(o, a);
    this.constraint.setLimit(3, (this.twistSpan * Math.PI) / 180),
      this.constraint.setLimit(4, (this.swingSpan2 * Math.PI) / 180),
      this.constraint.setLimit(5, (this.swingSpan1 * Math.PI) / 180),
      this.constraint.setBreakingImpulseThreshold(this.breakingThreshold),
      Ammo.destroy(a),
      Ammo.destroy(r),
      Ammo.destroy(s),
      this.app.systems.rigidbody.dynamicsWorld.addConstraint(
        this.constraint,
        !this.enableCollision
      ),
      this.activate();
  }),
  (ConeTwistConstraint.prototype.destroyConstraint = function () {
    this.constraint &&
      (this.app.systems.rigidbody.dynamicsWorld.removeConstraint(
        this.constraint
      ),
      Ammo.destroy(this.constraint),
      (this.constraint = null));
  }),
  (ConeTwistConstraint.prototype.activate = function () {
    this.entity.rigidbody.activate(),
      this.entityB && this.entityB.rigidbody.activate();
  }),
  (ConeTwistConstraint.prototype.update = function (t) {
    this.debugRender &&
      this.app.renderLine(
        this.entity.getPosition(),
        pc.Vec3.ZERO,
        this.debugColor
      );
  });
var SliderConstraint = pc.createScript("sliderConstraint");
SliderConstraint.attributes.add("pivotA", {
  title: "Pivot",
  description: "Position of the constraint in the local space of this entity.",
  type: "vec3",
  default: [0, 0, 0],
}),
  SliderConstraint.attributes.add("axisA", {
    title: "Axis",
    description:
      "Axis of rotation of the constraint in the local space this entity.",
    type: "vec3",
    default: [0, 1, 0],
  }),
  SliderConstraint.attributes.add("entityB", {
    title: "Connected Entity",
    description: "Optional second connected entity.",
    type: "entity",
  }),
  SliderConstraint.attributes.add("pivotB", {
    title: "Connected Pivot",
    description:
      "Position of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 0, 0],
  }),
  SliderConstraint.attributes.add("axisB", {
    title: "Connected Axis",
    description:
      "Axis of rotation of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 1, 0],
  }),
  SliderConstraint.attributes.add("linearLimits", {
    title: "Linear Limits",
    description: "Linear limits of the constraint.",
    type: "vec2",
    default: [1, -1],
  }),
  SliderConstraint.attributes.add("angularLimits", {
    title: "Angular Limits",
    description: "Angular limits of the constraint.",
    type: "vec2",
    default: [0, 0],
  }),
  SliderConstraint.attributes.add("breakingThreshold", {
    title: "Break Threshold",
    description:
      "Maximum breaking impulse threshold required to break the constraint.",
    type: "number",
    default: 1e30,
  }),
  SliderConstraint.attributes.add("enableCollision", {
    title: "Enable Collision",
    description: "Enable collision between linked rigid bodies.",
    type: "boolean",
    default: !0,
  }),
  SliderConstraint.attributes.add("debugRender", {
    title: "Debug Render",
    description: "Enable to render a representation of the constraint.",
    type: "boolean",
    default: !1,
  }),
  SliderConstraint.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  (SliderConstraint.prototype.initialize = function () {
    this.createConstraint(),
      this.on("attr", function (t, i, e) {
        "pivotA" === t ||
        "axisA" === t ||
        "entityB" === t ||
        "pivotB" === t ||
        "axisB" === t
          ? this.createConstraint()
          : "linearLimits" === t
          ? (this.constraint.setLowerLinLimit(this.linearLimits.x),
            this.constraint.setUpperLinLimit(this.linearLimits.y),
            this.activate())
          : "angularLimits" === t
          ? (this.constraint.setLowerAngLimit(this.angularLimits.x),
            this.constraint.setUpperAngLimit(this.angularLimits.y),
            this.activate())
          : "breakingThreshold" === t &&
            (this.constraint.setBreakingImpulseThreshold(
              this.breakingThreshold
            ),
            this.activate());
      }),
      this.on("enable", function () {
        this.createConstraint();
      }),
      this.on("disable", function () {
        this.destroyConstraint();
      }),
      this.on("destroy", function () {
        this.destroyConstraint();
      });
  }),
  (SliderConstraint.prototype.createConstraint = function () {
    this.constraint && this.destroyConstraint();
    var t = new pc.Vec3(),
      i = new pc.Vec3(),
      e = new pc.Quat(),
      n = new pc.Mat4(),
      o = this.entity.rigidbody.body,
      s = new Ammo.btVector3(this.pivotA.x, this.pivotA.y, this.pivotA.z);
    getOrthogonalVectors(this.axisA, t, i),
      n.set([
        this.axisA.x,
        this.axisA.y,
        this.axisA.z,
        0,
        t.x,
        t.y,
        t.z,
        0,
        i.x,
        i.y,
        i.z,
        0,
        0,
        0,
        0,
        1,
      ]),
      e.setFromMat4(n);
    var r = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
      a = new Ammo.btTransform(r, s);
    if ((a.setOrigin(s), this.entityB && this.entityB.rigidbody)) {
      var d = this.entityB.rigidbody.body,
        h = new Ammo.btVector3(this.pivotB.x, this.pivotB.y, this.pivotB.z);
      getOrthogonalVectors(this.axisB, t, i),
        n.set([
          t.x,
          t.y,
          t.z,
          0,
          i.x,
          i.y,
          i.z,
          0,
          this.axisB.x,
          this.axisB.y,
          this.axisB.z,
          0,
          0,
          0,
          0,
          1,
        ]),
        e.setFromMat4(n);
      var c = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
        l = new Ammo.btTransform(c, h);
      (this.constraint = new Ammo.btSliderConstraint(o, d, a, l)),
        Ammo.destroy(l),
        Ammo.destroy(c),
        Ammo.destroy(h);
    } else this.constraint = new Ammo.btSliderConstraint(o, a);
    this.constraint.setLowerLinLimit(this.linearLimits.x),
      this.constraint.setUpperLinLimit(this.linearLimits.y),
      this.constraint.setLowerAngLimit(this.angularLimits.x),
      this.constraint.setUpperAngLimit(this.angularLimits.y),
      this.constraint.setBreakingImpulseThreshold(this.breakingThreshold),
      Ammo.destroy(a),
      Ammo.destroy(r),
      Ammo.destroy(s),
      this.app.systems.rigidbody.dynamicsWorld.addConstraint(
        this.constraint,
        !this.enableCollision
      ),
      this.activate();
  }),
  (SliderConstraint.prototype.destroyConstraint = function () {
    this.constraint &&
      (this.app.systems.rigidbody.dynamicsWorld.removeConstraint(
        this.constraint
      ),
      Ammo.destroy(this.constraint),
      (this.constraint = null));
  }),
  (SliderConstraint.prototype.activate = function () {
    this.entity.rigidbody.activate(),
      this.entityB && this.entityB.rigidbody.activate();
  }),
  (SliderConstraint.prototype.update = function (t) {
    this.debugRender;
  });
var Generic6DofConstraint = pc.createScript("generic6DofConstraint");
Generic6DofConstraint.attributes.add("pivotA", {
  title: "Pivot",
  description: "Position of the constraint in the local space of this entity.",
  type: "vec3",
  default: [0, 0, 0],
}),
  Generic6DofConstraint.attributes.add("axisA", {
    title: "Axis",
    description:
      "Axis of rotation of the constraint in the local space this entity.",
    type: "vec3",
    default: [0, 1, 0],
  }),
  Generic6DofConstraint.attributes.add("entityB", {
    title: "Connected Entity",
    description: "Optional second connected entity.",
    type: "entity",
  }),
  Generic6DofConstraint.attributes.add("pivotB", {
    title: "Connected Pivot",
    description:
      "Position of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 0, 0],
  }),
  Generic6DofConstraint.attributes.add("axisB", {
    title: "Connected Axis",
    description:
      "Axis of rotation of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 1, 0],
  }),
  Generic6DofConstraint.attributes.add("linearLimits", {
    title: "Linear Limits",
    description: "Linear limits of the constraint.",
    type: "vec2",
    default: [1, -1],
  }),
  Generic6DofConstraint.attributes.add("angularLimits", {
    title: "Angular Limits",
    description: "Angular limits of the constraint.",
    type: "vec2",
    default: [0, 0],
  }),
  Generic6DofConstraint.attributes.add("breakingThreshold", {
    title: "Break Threshold",
    description:
      "Maximum breaking impulse threshold required to break the constraint.",
    type: "number",
    default: 1e30,
  }),
  Generic6DofConstraint.attributes.add("enableCollision", {
    title: "Enable Collision",
    description: "Enable collision between linked rigid bodies.",
    type: "boolean",
    default: !0,
  }),
  Generic6DofConstraint.attributes.add("debugRender", {
    title: "Debug Render",
    description: "Enable to render a representation of the constraint.",
    type: "boolean",
    default: !1,
  }),
  Generic6DofConstraint.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  (Generic6DofConstraint.prototype.initialize = function () {
    this.createConstraint(),
      this.on("attr", function (t, i, e) {
        "pivotA" === t ||
        "axisA" === t ||
        "entityB" === t ||
        "pivotB" === t ||
        "axisB" === t
          ? this.createConstraint()
          : "linearLimits" === t
          ? (this.constraint.setLowerLinLimit(this.linearLimits.x),
            this.constraint.setUpperLinLimit(this.linearLimits.y),
            this.activate())
          : "angularLimits" === t
          ? (this.constraint.setLowerAngLimit(this.angularLimits.x),
            this.constraint.setUpperAngLimit(this.angularLimits.y),
            this.activate())
          : "breakingThreshold" === t &&
            (this.constraint.setBreakingImpulseThreshold(
              this.breakingThreshold
            ),
            this.activate());
      }),
      this.on("enable", function () {
        this.createConstraint();
      }),
      this.on("disable", function () {
        this.destroyConstraint();
      }),
      this.on("destroy", function () {
        this.destroyConstraint();
      });
  }),
  (Generic6DofConstraint.prototype.createConstraint = function () {
    this.constraint && this.destroyConstraint();
    var t = new pc.Vec3(),
      i = new pc.Vec3(),
      e = new pc.Quat(),
      n = new pc.Mat4(),
      o = this.entity.rigidbody.body,
      s = new Ammo.btVector3(this.pivotA.x, this.pivotA.y, this.pivotA.z);
    getOrthogonalVectors(this.axisA, t, i),
      n.set([
        this.axisA.x,
        this.axisA.y,
        this.axisA.z,
        0,
        t.x,
        t.y,
        t.z,
        0,
        i.x,
        i.y,
        i.z,
        0,
        0,
        0,
        0,
        1,
      ]),
      e.setFromMat4(n);
    var r = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
      a = new Ammo.btTransform(r, s);
    if ((a.setOrigin(s), this.entityB && this.entityB.rigidbody)) {
      var d = this.entityB.rigidbody.body,
        h = new Ammo.btVector3(this.pivotB.x, this.pivotB.y, this.pivotB.z);
      getOrthogonalVectors(this.axisB, t, i),
        n.set([
          t.x,
          t.y,
          t.z,
          0,
          i.x,
          i.y,
          i.z,
          0,
          this.axisB.x,
          this.axisB.y,
          this.axisB.z,
          0,
          0,
          0,
          0,
          1,
        ]),
        e.setFromMat4(n);
      var c = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
        l = new Ammo.btTransform(c, h);
      (this.constraint = new Ammo.btSliderConstraint(o, d, a, l)),
        Ammo.destroy(l),
        Ammo.destroy(c),
        Ammo.destroy(h);
    } else this.constraint = new Ammo.btSliderConstraint(o, a);
    this.constraint.setLowerLinLimit(this.linearLimits.x),
      this.constraint.setUpperLinLimit(this.linearLimits.y),
      this.constraint.setLowerAngLimit(this.angularLimits.x),
      this.constraint.setUpperAngLimit(this.angularLimits.y),
      this.constraint.setBreakingImpulseThreshold(this.breakingThreshold),
      Ammo.destroy(a),
      Ammo.destroy(r),
      Ammo.destroy(s),
      this.app.systems.rigidbody.dynamicsWorld.addConstraint(
        this.constraint,
        !this.enableCollision
      ),
      this.activate();
  }),
  (Generic6DofConstraint.prototype.destroyConstraint = function () {
    this.constraint &&
      (this.app.systems.rigidbody.dynamicsWorld.removeConstraint(
        this.constraint
      ),
      Ammo.destroy(this.constraint),
      (this.constraint = null));
  }),
  (Generic6DofConstraint.prototype.activate = function () {
    this.entity.rigidbody.activate(),
      this.entityB && this.entityB.rigidbody.activate();
  }),
  (Generic6DofConstraint.prototype.update = function (t) {
    this.debugRender;
  });
var SpringJoint = pc.createScript("springJoint");
SpringJoint.attributes.add("entityA", {
  title: "entityA",
  description: "Optional second connected entity.",
  type: "entity",
}),
  SpringJoint.attributes.add("entityB", {
    title: "entityB",
    description: "Optional second connected entity.",
    type: "entity",
  }),
  SpringJoint.attributes.add("pivotA", {
    title: "pivotA",
    description:
      "Position of the constraint in the local space of this entity.",
    type: "vec3",
    default: [0, 0, 0],
  }),
  SpringJoint.attributes.add("pivotB", {
    title: "pivotB",
    description:
      "Position of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 0, 0],
  }),
  SpringJoint.attributes.add("axisB", {
    title: "Upper Limit",
    description:
      "Axis of rotation of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 1, 0],
  }),
  SpringJoint.attributes.add("axisA", {
    title: "Lower Limit",
    description:
      "Axis of rotation of the constraint in the local space this entity.",
    type: "vec3",
    default: [0, 1, 0],
  }),
  SpringJoint.attributes.add("stifness", {
    title: "stifness",
    description: "Linear limits of the constraint.",
    type: "number",
    default: 10,
  }),
  SpringJoint.attributes.add("damping", {
    title: "damping",
    description: "Angular limits of the constraint.",
    type: "number",
    default: 0.1,
  }),
  SpringJoint.attributes.add("enableCollision", {
    title: "Enable Collision",
    description: "Enable collision between linked rigid bodies.",
    type: "boolean",
    default: !0,
  }),
  SpringJoint.attributes.add("debugRender", {
    title: "Debug Render",
    description: "Enable to render a representation of the constraint.",
    type: "boolean",
    default: !1,
  }),
  SpringJoint.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  (SpringJoint.prototype.initialize = function () {
    (this.initialStifness = this.stifness),
      this.createConstraint(),
      this.on("attr", function (t, i, e) {
        ("pivotA" !== t &&
          "axisA" !== t &&
          "entityB" !== t &&
          "pivotB" !== t &&
          "axisB" !== t) ||
          this.createConstraint();
      }),
      this.on("enable", function () {
        this.createConstraint();
      }),
      this.on("disable", function () {
        this.destroyConstraint();
      }),
      this.on("destroy", function () {
        this.destroyConstraint();
      });
  }),
  (SpringJoint.prototype.createConstraint = function () {
    this.constraint && this.destroyConstraint();
    let t = new Ammo.btVector3(this.axisA.x, this.axisA.y, this.axisA.z),
      i =
        (new Ammo.btVector3(this.axisB.x, this.axisB.y, this.axisB.z),
        new Ammo.btVector3(this.pivotA.x, this.pivotA.y, this.pivotA.z)),
      e = new Ammo.btVector3(this.pivotB.x, this.pivotB.y, this.pivotB.z),
      n = new Ammo.btQuaternion(0, 0, 0, 1),
      o = new Ammo.btTransform(n, i),
      s = new Ammo.btTransform(n, e);
    (this.constraint = new Ammo.btGeneric6DofSpringConstraint(
      this.entityA.rigidbody.body,
      this.entityB.rigidbody.body,
      o,
      s,
      !0
    )),
      this.constraint.setLinearLowerLimit(t),
      this.constraint.setLinearUpperLimit(new Ammo.btVector3(0, 0, 0)),
      this.constraint.enableSpring(1, !0),
      this.constraint.setStiffness(1, this.stifness),
      this.constraint.setDamping(1, this.damping),
      this.constraint.enableSpring(0, !0),
      this.constraint.setStiffness(0, this.stifness),
      this.constraint.setDamping(0, this.damping),
      this.app.systems.rigidbody.dynamicsWorld.addConstraint(
        this.constraint,
        !this.enableCollision
      );
  }),
  (SpringJoint.prototype.destroyConstraint = function () {
    this.constraint &&
      (this.app.systems.rigidbody.dynamicsWorld.removeConstraint(
        this.constraint
      ),
      Ammo.destroy(this.constraint),
      (this.constraint = null));
  }),
  (SpringJoint.prototype.activate = function () {
    this.entity.rigidbody.activate(),
      this.entityB && this.entityB.rigidbody.activate();
  }),
  (SpringJoint.prototype.update = function (t) {
    if (this.debugRender) {
      var i = new pc.Vec3();
      this.entityA.getWorldTransform().transformPoint(this.pivotA, i);
      var e = new pc.Vec3();
      this.entityB.getWorldTransform().transformPoint(this.pivotB, e),
        this.app.renderLine(i, e, this.debugColor),
        this.entityB;
    }
  });
var SpringJoint = pc.createScript("springJoint");
SpringJoint.attributes.add("entityA", {
  title: "entityA",
  description: "Optional second connected entity.",
  type: "entity",
}),
  SpringJoint.attributes.add("pivotA", {
    title: "pivotA",
    description:
      "Position of the constraint in the local space of this entity.",
    type: "vec3",
    default: [0, 0, 0],
  }),
  SpringJoint.attributes.add("axisA", {
    title: "axisA",
    description:
      "Axis of rotation of the constraint in the local space this entity.",
    type: "vec3",
    default: [0, 1, 0],
  }),
  SpringJoint.attributes.add("entityB", {
    title: "entityB",
    description: "Optional second connected entity.",
    type: "entity",
  }),
  SpringJoint.attributes.add("pivotB", {
    title: "pivotB",
    description:
      "Position of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 0, 0],
  }),
  SpringJoint.attributes.add("axisB", {
    title: "axisB",
    description:
      "Axis of rotation of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 1, 0],
  }),
  SpringJoint.attributes.add("stifness", {
    title: "stifness",
    description: "Linear limits of the constraint.",
    type: "number",
    default: 10,
  }),
  SpringJoint.attributes.add("damping", {
    title: "damping",
    description: "Angular limits of the constraint.",
    type: "number",
    default: 0.1,
  }),
  SpringJoint.attributes.add("enableCollision", {
    title: "Enable Collision",
    description: "Enable collision between linked rigid bodies.",
    type: "boolean",
    default: !0,
  }),
  SpringJoint.attributes.add("debugRender", {
    title: "Debug Render",
    description: "Enable to render a representation of the constraint.",
    type: "boolean",
    default: !1,
  }),
  SpringJoint.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  (SpringJoint.prototype.initialize = function () {
    this.createConstraint(),
      this.on("attr", function (t, i, n) {}),
      this.on("enable", function () {
        this.createConstraint();
      }),
      this.on("disable", function () {
        this.destroyConstraint();
      }),
      this.on("destroy", function () {
        this.destroyConstraint();
      });
  }),
  (SpringJoint.prototype.createConstraint = function () {
    this.constraint && this.destroyConstraint();
    let t = new Ammo.btVector3(this.axisA.x, this.axisA.y, this.axisA.z),
      i =
        (new Ammo.btVector3(this.axisB.x, this.axisB.y, this.axisB.z),
        new Ammo.btVector3(this.pivotA.x, this.pivotA.y, this.pivotA.z)),
      n = new Ammo.btVector3(this.pivotB.x, this.pivotB.y, this.pivotB.z),
      e = new Ammo.btQuaternion(0, 0, 0, 1),
      o = new Ammo.btTransform(e, i),
      s = new Ammo.btTransform(e, n);
    (this.constraint = new Ammo.btGeneric6DofSpringConstraint(
      this.entityA.rigidbody.body,
      this.entityB.rigidbody.body,
      o,
      s,
      !0
    )),
      this.constraint.setLinearLowerLimit(t),
      this.constraint.setLinearUpperLimit(new Ammo.btVector3(0, 0, 0)),
      this.constraint.enableSpring(1, !0),
      this.constraint.setStiffness(1, this.stifness),
      this.constraint.setDamping(1, this.damping),
      this.app.systems.rigidbody.dynamicsWorld.addConstraint(
        this.constraint,
        !this.enableCollision
      );
  }),
  (SpringJoint.prototype.destroyConstraint = function () {
    this.constraint &&
      (this.app.systems.rigidbody.dynamicsWorld.removeConstraint(
        this.constraint
      ),
      Ammo.destroy(this.constraint),
      (this.constraint = null));
  }),
  (SpringJoint.prototype.activate = function () {
    this.entity.rigidbody.activate(),
      this.entityB && this.entityB.rigidbody.activate();
  }),
  (SpringJoint.prototype.update = function (t) {
    if (this.debugRender) {
      var i = new pc.Vec3();
      this.entityA.getWorldTransform().transformPoint(this.pivotA, i);
      var n = new pc.Vec3();
      this.entityB.getWorldTransform().transformPoint(this.pivotB, n),
        this.app.renderLine(i, n, this.debugColor),
        this.entityB;
    }
  });
var PlayerController = pc.createScript("playerController");
PlayerController.attributes.add("head", { type: "entity" }),
  PlayerController.attributes.add("tail", { type: "entity" }),
  PlayerController.attributes.add("keyboardSensitivity", {
    type: "number",
    default: 2e3,
  }),
  PlayerController.attributes.add("segmentMass", {
    type: "number",
    default: 1,
  }),
  PlayerController.attributes.add("coolDownTime", { type: "number" }),
  PlayerController.attributes.add("sausageSegments", {
    type: "entity",
    array: !0,
  }),
  PlayerController.attributes.add("dashMeter", { type: "entity" }),
  PlayerController.attributes.add("eyes", { type: "entity", array: !0 }),
  PlayerController.attributes.add("force", { type: "number", default: 9 }),
  PlayerController.attributes.add("forceThresholds", {
    title: "Force Thresholds",
    type: "vec2",
    default: [0.5, 2.5],
  }),
  PlayerController.attributes.add("springMultiplier", {
    type: "number",
    default: 1,
  }),
  PlayerController.attributes.add("springs", { type: "entity", array: !0 }),
  PlayerController.attributes.add("landParticles", {
    type: "asset",
    assetType: "template",
  }),
  PlayerController.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  PlayerController.attributes.add("activated", {
    type: "boolean",
    default: !0,
  }),
  PlayerController.attributes.add("debug", { type: "boolean", default: !0 }),
  (PlayerController.prototype.initialize = function () {
    (PlayerController.instance = this),
      (this.headOrTail = this.sausageSegments[0]),
      (this.screenToWorldRatio =
        (2 * GameManager.instance.camera.camera.orthoHeight) /
        window.innerHeight),
      (this.startPos = new pc.Vec2()),
      (this.attached = !1),
      (this.loaded = !1),
      (this.segment = this.head),
      (this.stuckSegment = this.segment),
      (this.offset = new pc.Vec2()),
      (this.coolDown = !1),
      (this.firstMouseUp = !0);
    var t = this.app.touch;
    (this.controls = 0),
      t
        ? (t.on(pc.EVENT_TOUCHSTART, this.onMouseDown, this),
          t.on(pc.EVENT_TOUCHMOVE, this.onMouseMove, this),
          t.on(pc.EVENT_TOUCHEND, this.onMouseUp, this))
        : (this.app.mouse.on(pc.EVENT_MOUSEMOVE, this.onMouseMove, this),
          this.app.mouse.on(pc.EVENT_MOUSEDOWN, this.onMouseDown, this),
          this.app.mouse.on(pc.EVENT_MOUSEUP, this.onMouseUp, this),
          this.app.keyboard.on(pc.EVENT_KEYDOWN, this.pressedKey, this),
          this.app.keyboard.on(pc.EVENT_KEYUP, this.releasedKey, this)),
      this.on(
        "destroy",
        function () {
          (PlayerController.instance = void 0),
            this.app.touch &&
              (this.app.touch.off(pc.EVENT_TOUCHSTART, this.onMouseDown, this),
              this.app.touch.off(pc.EVENT_TOUCHEND, this.onMouseUp, this),
              this.app.touch.off(pc.EVENT_TOUCHMOVE, this.onMouseMove, this)),
            this.app.mouse.off(pc.EVENT_MOUSEDOWN, this.onMouseDown, this),
            this.app.mouse.off(pc.EVENT_MOUSEUP, this.onMouseUp, this),
            this.app.mouse.off(pc.EVENT_MOUSEMOVE, this.onMouseMove, this),
            this.app.keyboard.off(pc.EVENT_KEYDOWN, this.pressedKey, this),
            this.app.keyboard.off(pc.EVENT_KEYUP, this.releasedKey, this);
        },
        this
      ),
      this.sausageSegments.forEach((t) => {
        t.collision.on(
          "collisionstart",
          (e) => {
            this.onCollisionEnter(e, t);
          },
          this
        );
      }),
      setTimeout(() => {
        this.springs.forEach((t) => {
          (t.script.springJoint.stifness =
            t.script.springJoint.initialStifness * this.springMultiplier),
            (t.script.springJoint.damping =
              t.script.springJoint.initialStifness * this.springMultiplier),
            t.script.springJoint.createConstraint();
        });
      }, 100),
      this.on("attr", function (t, e, s) {
        "springMultiplier" === t && this.changeSpringStifness(e);
      });
  }),
  (PlayerController.prototype.changeSpringStifness = function (t) {
    this.springs.forEach((e) => {
      (e.script.springJoint.stifness =
        e.script.springJoint.initialStifness * this.springMultiplier * t),
        (e.script.springJoint.damping =
          e.script.springJoint.initialStifness * this.springMultiplier * t),
        e.script.springJoint.createConstraint();
    });
  }),
  (PlayerController.prototype.hingeLimits = function (t) {
    this.sausageSegments.forEach((e, s) => {
      s > 0 &&
        (e.script.hingeConstraint.limits = t
          ? new pc.Vec2(-7, 7)
          : new pc.Vec2(-45, 45));
    });
  }),
  (PlayerController.prototype.onMouseDown = function (t) {
    if (0 == this.controls || 2 == this.controls) {
      let e, s;
      (this.controls = 2),
        this.activated &&
          (t.hasOwnProperty("touches")
            ? ((e = t.touches[0].touch.clientX),
              (s = t.touches[0].touch.clientY))
            : ((e = t.x), (s = t.y)),
          (!this.loaded && !this.debug) ||
            GameManager.instance.finished ||
            this.paused ||
            ((this.mouseDown = !0),
            this.dashMeter.setLocalScale(0, 0, 0),
            (this.dashMeter.enabled = !0),
            (this.app.timeScale = 0.1),
            (this.startPos = new pc.Vec2(e, s))));
    }
    t.event.preventDefault();
  });
const clamp = (t, e, s) => Math.min(Math.max(t, e), s);
(PlayerController.prototype.onMouseMove = function (t) {
  let e, s;
  t.hasOwnProperty("touches")
    ? ((e = t.touches[0].touch.clientX), (s = t.touches[0].touch.clientY))
    : (!t.buttons[0] && this.mouseDown && (this.mouseDown = !1),
      (e = t.x),
      (s = t.y)),
    this.activated &&
      !this.paused &&
      this.mouseDown &&
      ((0 != this.controls && 2 != this.controls) ||
        ((this.controls = 2),
        (this.mouseMoved = !0),
        (this.mousePos = new pc.Vec2(e, s)),
        (this.delta = this.mousePos
          .sub(this.startPos)
          .mulScalar(this.screenToWorldRatio)),
        (this.offset = new pc.Vec2()
          .copy(this.delta)
          .normalize()
          .mulScalar(
            clamp(
              this.delta.length(),
              1.25 * this.forceThresholds.x,
              1.25 * this.forceThresholds.y
            )
          )),
        this.dashMeter.setLocalScale(
          this.offset.length() / 5,
          this.offset.length() / 5,
          0
        ))),
    t.event.preventDefault();
}),
  (PlayerController.prototype.onMouseUp = function (t) {
    (0 != this.controls && 2 != this.controls) ||
      ((this.controls = 2),
      (this.loaded || this.debug) &&
        !this.paused &&
        this.mouseDown &&
        (null != this.delta &&
          this.delta != pc.Vec3.ZERO &&
          this.delta.length() >= 1 &&
          (this.unstick(), this.dash(this.processForce(this.delta))),
        (this.app.timeScale = 1),
        (this.mouseDown = !1),
        (this.dashMeter.enabled = !1),
        (this.mouseMoved = !1)),
      this.unPause &&
        ((this.paused = !1), (this.unPause = !1), (this.mouseDown = !1)),
      (this.controls = 0)),
      t && t.event.preventDefault();
  }),
  (PlayerController.prototype.pressedKey = function (t) {
    !this.activated ||
      (0 != this.controls && 1 != this.controls) ||
      this.paused ||
      ("ArrowLeft" != t.event.code &&
        "KeyA" != t.event.code &&
        "ArrowRight" != t.event.code &&
        "KeyD" != t.event.code) ||
      ((this.controls = 1),
      ("ArrowLeft" != t.event.code && "KeyA" != t.event.code) ||
        (this.rightKeyDown = !0),
      ("ArrowRight" != t.event.code && "KeyD" != t.event.code) ||
        (this.leftKeyDown = !0),
      (this.leftKeyDown || this.rightKeyDown) &&
        !this.aiming &&
        this.loaded &&
        this.attached &&
        ((this.degree = 0), (this.delta = new pc.Vec2()), (this.aiming = !0)));
  }),
  (PlayerController.prototype.releasedKey = function (t) {
    this.activated &&
      1 == this.controls &&
      (("ArrowLeft" != t.event.code &&
        "KeyA" != t.event.code &&
        "ArrowRight" != t.event.code &&
        "KeyD" != t.event.code) ||
        ((this.loaded || this.debug) && this.paused,
        this.unPause &&
          ((this.app.timeScale = 1),
          (this.paused = !1),
          (this.unPause = !1),
          (this.spaceDown = !1),
          (this.aiming = !1),
          (this.rightKeyDown = !1),
          (this.leftKeyDown = !1)),
        ("ArrowLeft" != t.event.code && "KeyA" != t.event.code) ||
          (this.rightKeyDown = !1),
        ("ArrowRight" != t.event.code && "KeyD" != t.event.code) ||
          (this.leftKeyDown = !1),
        this.leftKeyDown ||
          this.rightKeyDown ||
          (!this.loaded && !this.debug) ||
          GameManager.instance.finished ||
          !this.activated ||
          this.paused ||
          !this.attached ||
          setTimeout(() => {
            this.leftKeyDown ||
              this.rightKeyDown ||
              (!this.loaded && !this.debug) ||
              GameManager.instance.finished ||
              !this.activated ||
              this.paused ||
              !this.attached ||
              (this.unstick(),
              (this.aiming = !1),
              (this.controls = 0),
              (this.app.timeScale = 1),
              (this.spaceDown = !1),
              (this.dashMeter.enabled = !1),
              this.dash(this.processForce(this.delta)));
          }, 100),
        t && t.event.preventDefault())),
      t.event.preventDefault();
  }),
  (PlayerController.prototype.processForce = function (t) {
    let e = t.clone().normalize(),
      s = clamp(t.length(), 0.5, 2.5);
    return e.mulScalar(s * this.force);
  }),
  (PlayerController.prototype.dash = function (t) {
    this.firstMouseUp &&
      (PauseButton.instance.show(),
      SkinsButton.instance.hideButton(),
      PokiManager.instance.removeBanner(),
      (this.firstMouseUp = !1));
    let e = new Ammo.btVector3();
    this.sausageSegments.forEach((t, s) => {
      t.rigidbody.body.setActivationState(1),
        t.rigidbody.body.activate(),
        e.setValue(0, 0, 0),
        t.rigidbody.body.setLinearVelocity(e),
        e.setValue(0, 0, 0),
        this.sausageSegments[2].rigidbody.body.setAngularVelocity(e);
    }),
      this.eyes.forEach((t, s) => {
        this.eye &&
          (e.setValue(0, 0, 0), t.rigidbody.body.setLinearVelocity(e));
      }),
      (this.coolDown = !0),
      setTimeout(() => {
        this.coolDown = !1;
      }, this.coolDownTime),
      this.headOrTail.rigidbody.body.applyImpulse(
        new Ammo.btVector3(t.x, -t.y, 0)
      );
  }),
  (PlayerController.prototype.unstick = function (t) {
    if (this.loaded && this.activated && this.attached && !this.paused) {
      if (this.stuckSegment.children[0]) {
        this.stuckSegment.children[0].script.hinge.enabled = !1;
      }
      this.hingeLimits(!0),
        this.changeSpringStifness(8),
        (this.loaded = !1),
        (this.attached = !1);
    }
  }),
  (PlayerController.prototype.stick = function (t, e) {
    (this.attached = !0), (this.loaded = !0);
    let s = t.children[0].script.hinge;
    (this.stuckSegment = t), (this.controls = 0);
    let i = 0;
    this.sausageSegments.forEach((t) => {
      let e = this.stuckSegment.getPosition().sub(t.getPosition()).length();
      e <= i || ((i = e), (this.headOrTail = t));
    }),
      this.hingeLimits(!1),
      this.changeSpringStifness(4),
      (s.entityB = e.other),
      (s.pivotA = e.contacts[0].localPoint),
      (s.pivotB = e.contacts[0].localPointOther),
      s.createConstraint(),
      (s.enabled = !0);
    let o = this.landParticles.resource.instantiate();
    t.addChild(o),
      o.particlesystem.play(),
      o.setLocalPosition(0, 0, 0),
      CameraManager.instance.flashOutline(0.5);
  }),
  (PlayerController.prototype.onCollisionEnter = function (t, e) {
    this.attached ||
      this.coolDown ||
      t.other.tags._list.includes("NotSticky") ||
      ((this.oldEntity = t),
      t.other.tags._list.includes("Segment") ||
        (this.stick(e, t), (this.mouseDown = !1)));
  }),
  (PlayerController.prototype.death = function () {
    PokiManager.instance.interstitial(() => {
      (this.activated = !1),
        (GameManager.instance.respawned = !0),
        GameManager.instance.startLevel(!0);
    });
  }),
  (PlayerController.prototype.update = function (t) {
    if (
      ((this.app.keyboard.isPressed(pc.KEY_RIGHT) ||
        this.app.keyboard.isPressed(pc.KEY_D)) &&
        (this.degree += this.keyboardSensitivity * t),
      this.app.keyboard.wasPressed(pc.KEY_RIGHT) &&
        (this.degree -= this.keyboardSensitivity * t),
      (this.app.keyboard.isPressed(pc.KEY_LEFT) ||
        this.app.keyboard.isPressed(pc.KEY_Q) ||
        this.app.keyboard.isPressed(pc.KEY_A)) &&
        (this.degree -= this.keyboardSensitivity * t),
      // console.log(this.degree),
      this.aiming)
    ) {
      this.dashMeter.setLocalScale(0, 0, 0),
        (this.dashMeter.enabled = !0),
        (this.app.timeScale = 0.1);
      this.forceThresholds.x;
      var e = (this.degree * Math.PI) / 180;
      (this.delta.x = Math.cos(e)),
        (this.delta.y = Math.sin(e)),
        this.delta.mulScalar(100),
        (this.offset = new pc.Vec2()
          .copy(this.delta)
          .normalize()
          .mulScalar(
            clamp(
              this.delta.length(),
              1.25 * this.forceThresholds.x,
              1.25 * this.forceThresholds.y
            )
          )),
        this.dashMeter.setLocalScale(
          this.offset.length() / 5,
          this.offset.length() / 5,
          0
        );
    }
    this.head.getPosition().y < -20 &&
      !GameManager.instance.finished &&
      this.activated &&
      this.death(),
      this.dashMeter.enabled &&
        (this.dashMeter.setPosition(this.headOrTail.getPosition()),
        this.dashMeter.setRotation(
          new pc.Quat().setFromEulerAngles(
            0,
            0,
            (180 *
              Math.atan2(
                -this.offset.normalize().y,
                this.offset.normalize().x
              )) /
              Math.PI
          )
        )),
      // console.log(this.controls);
      0;
  });
var CameraManager = pc.createScript("cameraManager");
CameraManager.attributes.add("target", { type: "entity" }),
  CameraManager.attributes.add("speed", { type: "number" }),
  CameraManager.attributes.add("offset", { type: "number" }),
  CameraManager.attributes.add("yOffset", { type: "number" }),
  CameraManager.attributes.add("xOffset", { type: "number" }),
  CameraManager.attributes.add("verticalBounds", { type: "number" }),
  CameraManager.attributes.add("flashEntity", { type: "entity" }),
  CameraManager.attributes.add("outline", { type: "entity" }),
  CameraManager.attributes.add("state", { type: "string" }),
  CameraManager.attributes.add("xLimits", { type: "vec2", default: [0, 0] }),
  CameraManager.attributes.add("yLimits", { type: "vec2", default: [0, 0] }),
  (CameraManager.prototype.initialize = function () {
    (CameraManager.instance = this),
      (this.orthoHeight = this.entity.camera.orthoHeight),
      (this.sausageCamera = this.entity.findByName("Sausage Camera")),
      (this.shadowCamera = this.entity.findByName("Shadow Camera")),
      (this.shopOffsetX = 0),
      (this.shopOffsetY = 0),
      (this.baseShopOffsetY = 5);
  }),
  (CameraManager.prototype.flash = function (t) {
    let e = { x: 1 },
      a = this.flashEntity.tween(e);
    a.to({ x: 0 }, t / 2, pc.Linear, 0, 1)
      .on("update", () => {
        a.manager.setTs(1 / this.app.timeScale),
          (this.flashEntity.sprite.opacity = e.x);
      })
      .start();
  }),
  (CameraManager.prototype.flashOutline = function (t) {
    let e = { x: 1 },
      a = this.outline.tween(e);
    a.to({ x: 0 }, t / 2, pc.Linear, 0, 1)
      .on("update", () => {
        a.manager.setTs(1 / this.app.timeScale),
          (this.outline.element.opacity = e.x);
      })
      .start();
  }),
  (CameraManager.prototype.update = function (t) {
    this.orthoHeight != this.entity.camera.orthoHeight &&
      ((this.orthoHeight = this.entity.camera.orthoHeight),
      (this.sausageCamera.camera.orthoHeight = this.orthoHeight),
      (this.shadowCamera.camera.orthoHeight = this.orthoHeight)),
      GameManager.instance.level &&
        (GameManager.instance.playerController &&
        GameManager.instance.levelStarted
          ? ((this.target =
              GameManager.instance.playerController.sausageSegments[2]),
            (this.position = this.entity.getPosition()),
            this.entity.setPosition(
              pc.math.lerp(
                this.position.x,
                this.target.getPosition().x + this.xOffset,
                t * this.offset
              ),
              this.yOffset,
              this.position.z
            ))
          : "Main Menu" == GameManager.instance.level.name
          ? ((this.position = this.entity.getPosition()),
            this.entity.setPosition(
              pc.math.lerp(this.position.x, 0, t * this.offset * 10),
              this.yOffset,
              this.position.z
            ))
          : ("Shop" != GameManager.instance.level.name &&
              "shop" != this.state) ||
            ((this.target = ShopManager.instance.selectedPage),
            (this.position = this.entity.getPosition()),
            this.entity.setPosition(
              pc.math.lerp(
                this.position.x,
                clamp(this.target.getPosition().x + this.shopOffsetX, -40, 10),
                t * this.offset * 2
              ),
              pc.math.lerp(
                this.position.y,
                this.target.getPosition().y +
                  this.shopOffsetY +
                  this.baseShopOffsetY,
                t * this.offset * 2
              ),
              this.position.z
            )));
  });
var PhysicsLayer = pc.createScript("physicsLayer");
PhysicsLayer.attributes.add("groupA", {
  type: "boolean",
  default: !1,
  title: "Group A",
}),
  PhysicsLayer.attributes.add("groupB", {
    type: "boolean",
    default: !1,
    title: "Group B",
  }),
  PhysicsLayer.attributes.add("groupC", {
    type: "boolean",
    default: !1,
    title: "Group C",
  }),
  PhysicsLayer.attributes.add("groupD", {
    type: "boolean",
    default: !1,
    title: "Group D",
  }),
  PhysicsLayer.attributes.add("maskAll", {
    type: "boolean",
    default: !0,
    title: "Mask All",
  }),
  PhysicsLayer.attributes.add("maskA", {
    type: "boolean",
    default: !1,
    title: "Mask A",
  }),
  PhysicsLayer.attributes.add("maskB", {
    type: "boolean",
    default: !1,
    title: "Mask B",
  }),
  PhysicsLayer.attributes.add("maskC", {
    type: "boolean",
    default: !1,
    title: "Mask C",
  }),
  PhysicsLayer.attributes.add("maskD", {
    type: "boolean",
    default: !1,
    title: "Mask D",
  }),
  (PhysicsLayer.prototype.initialize = function () {
    var t = this.entity.rigidbody;
    this.groupA && (t.group |= pc.BODYGROUP_USER_1),
      this.groupB && (t.group |= pc.BODYGROUP_USER_2),
      this.groupC && (t.group |= pc.BODYGROUP_USER_3),
      this.groupD && (t.group |= pc.BODYGROUP_USER_4),
      (t.mask = pc.BODYGROUP_TRIGGER),
      this.maskAll && (t.mask |= pc.BODYMASK_ALL),
      this.maskA && (t.mask |= pc.BODYGROUP_USER_1),
      this.maskB && (t.mask |= pc.BODYGROUP_USER_2),
      this.maskC && (t.mask |= pc.BODYGROUP_USER_3),
      this.maskD && (t.mask |= pc.BODYGROUP_USER_4);
  });
var TriggerLayer = pc.createScript("triggerLayer");
TriggerLayer.attributes.add("groupA", {
  type: "boolean",
  default: !1,
  title: "Group A",
}),
  TriggerLayer.attributes.add("groupB", {
    type: "boolean",
    default: !1,
    title: "Group B",
  }),
  TriggerLayer.attributes.add("groupC", {
    type: "boolean",
    default: !1,
    title: "Group C",
  }),
  TriggerLayer.attributes.add("groupD", {
    type: "boolean",
    default: !1,
    title: "Group D",
  }),
  (TriggerLayer.prototype.initialize = function () {
    var r = this.entity.trigger.body,
      t = 4;
    this.groupA && (t |= pc.BODYGROUP_USER_1),
      this.groupB && (t |= pc.BODYGROUP_USER_2),
      this.groupC && (t |= pc.BODYGROUP_USER_3),
      this.groupD && (t |= pc.BODYGROUP_USER_4),
      r.setCollisionFlags(t);
  });
var Hinge = pc.createScript("hinge");
Hinge.attributes.add("entityA", {
  title: "Entity",
  description: "Connected entity.",
  type: "entity",
}),
  Hinge.attributes.add("pivotA", {
    title: "Pivot",
    description:
      "Position of the constraint in the local space of this entity.",
    type: "vec3",
    default: [0, 0, 0],
  }),
  Hinge.attributes.add("axisA", {
    title: "Axis",
    description:
      "Axis of rotation of the constraint in the local space this entity.",
    type: "vec3",
    default: [0, 1, 0],
  }),
  Hinge.attributes.add("entityB", {
    title: "Connected Entity",
    description: "Optional second connected entity.",
    type: "entity",
  }),
  Hinge.attributes.add("pivotB", {
    title: "Connected Pivot",
    description:
      "Position of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 0, 0],
  }),
  Hinge.attributes.add("axisB", {
    title: "Connected Axis",
    description:
      "Axis of rotation of the constraint in the local space of the connected entity (if specified).",
    type: "vec3",
    default: [0, 1, 0],
  }),
  Hinge.attributes.add("limits", {
    title: "Limits",
    description:
      "Low and high angular limits for the constraint in degrees. By default, low is greater than high meaning no limits.",
    type: "vec2",
    default: [1, -1],
  }),
  Hinge.attributes.add("softness", {
    title: "Softness",
    description:
      'Softness of the constraint. Recommend 0.8 to 1. Describes the percentage of limits where movement is free. Beyond this softness percentage, the limit is gradually enforced until the "hard" (1.0) limit is reached.',
    type: "number",
    min: 0,
    max: 1,
    default: 0.9,
  }),
  Hinge.attributes.add("biasFactor", {
    title: "Bias Factor",
    description:
      "Bias factor of the constraint. Recommend 0.3 +/- approximately 0.3. Strength with which constraint resists zeroth order (angular, not angular velocity) limit violation.",
    type: "number",
    min: 0,
    max: 1,
    default: 0.3,
  }),
  Hinge.attributes.add("relaxationFactor", {
    title: "Relaxation Factor",
    description:
      "Relaxation factor of the constraint. Recommend to keep this near 1. The lower the value, the less the constraint will fight velocities which violate the angular limits.",
    type: "number",
    min: 0,
    max: 1,
    default: 1,
  }),
  Hinge.attributes.add("enableMotor", {
    title: "Use Motor",
    description:
      "Enable a motor to power the automatic rotation around the hinge axis.",
    type: "boolean",
    default: !1,
  }),
  Hinge.attributes.add("motorTargetVelocity", {
    title: "Target Velocity",
    description: "Target motor angular velocity.",
    type: "number",
    default: 0,
  }),
  Hinge.attributes.add("maxMotorImpulse", {
    title: "Max Motor Impulse",
    description: "Maximum motor impulse.",
    type: "number",
    default: 0,
  }),
  Hinge.attributes.add("breakingThreshold", {
    title: "Break Threshold",
    description:
      "Maximum breaking impulse threshold required to break the constraint.",
    type: "number",
    default: 34e37,
  }),
  Hinge.attributes.add("enableCollision", {
    title: "Enable Collision",
    description: "Enable collision between linked rigid bodies.",
    type: "boolean",
    default: !0,
  }),
  Hinge.attributes.add("debugRender", {
    title: "Debug Render",
    description: "Enable to render a representation of the constraint.",
    type: "boolean",
    default: !1,
  }),
  Hinge.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  (Hinge.prototype.initialize = function () {
    this.createConstraint(),
      this.on("attr", (t, i, e) => {
        if (
          "pivotA" === t ||
          "axisA" === t ||
          "entityB" === t ||
          "pivotB" === t ||
          "axisB" === t
        )
          this.createConstraint();
        else if (
          "limits" === t ||
          "softness" === t ||
          "biasFactor" === t ||
          "relaxationFactor" === t
        ) {
          var o = (this.limits.x * Math.PI) / 180,
            n = (this.limits.y * Math.PI) / 180;
          this.constraint.setLimit(
            o,
            n,
            this.softness,
            this.biasFactor,
            this.relaxationFactor
          );
        } else
          "enableMotor" === t ||
          "motorTargetVelocity" === t ||
          "maxMotorImpulse" === t
            ? (this.constraint.enableAngularMotor(
                this.enableMotor,
                (this.motorTargetVelocity * Math.PI) / 180,
                this.maxMotorImpulse
              ),
              this.activate())
            : "breakingThreshold" === t &&
              (this.constraint.setBreakingImpulseThreshold(
                this.breakingThreshold
              ),
              this.activate());
      }),
      this.on("enable", function () {
        this.createConstraint();
      }),
      this.on("disable", function () {
        this.destroyConstraint();
      }),
      this.on("destroy", function () {
        this.destroyConstraint();
      });
  }),
  (Hinge.prototype.createConstraint = function () {
    this.constraint && this.destroyConstraint();
    var t = new pc.Vec3(),
      i = new pc.Vec3(),
      e = new pc.Quat(),
      o = new pc.Mat4();
    this.entityA ? (this.entityA = this.entityA) : (this.entityA = this.entity);
    var n = this.entityA.rigidbody.body,
      s = new Ammo.btVector3(this.pivotA.x, this.pivotA.y, this.pivotA.z);
    getOrthogonalVectors(this.axisA, t, i),
      o.set([
        t.x,
        t.y,
        t.z,
        0,
        i.x,
        i.y,
        i.z,
        0,
        this.axisA.x,
        this.axisA.y,
        this.axisA.z,
        0,
        0,
        0,
        0,
        1,
      ]),
      e.setFromMat4(o);
    var a = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
      r = new Ammo.btTransform(a, s);
    if (this.entityB && this.entityB.rigidbody) {
      var h = this.entityB.rigidbody.body,
        d = new Ammo.btVector3(this.pivotB.x, this.pivotB.y, this.pivotB.z);
      getOrthogonalVectors(this.axisB, t, i),
        o.set([
          t.x,
          t.y,
          t.z,
          0,
          i.x,
          i.y,
          i.z,
          0,
          this.axisB.x,
          this.axisB.y,
          this.axisB.z,
          0,
          0,
          0,
          0,
          1,
        ]),
        e.setFromMat4(o);
      var l = new Ammo.btQuaternion(e.x, e.y, e.z, e.w),
        c = new Ammo.btTransform(l, d);
      (this.constraint = new Ammo.btHingeConstraint(n, h, r, c, !1)),
        Ammo.destroy(c),
        Ammo.destroy(l),
        Ammo.destroy(d);
    } else this.constraint = new Ammo.btHingeConstraint(n, r, !1);
    var m = (this.limits.x * Math.PI) / 180,
      p = (this.limits.y * Math.PI) / 180;
    this.constraint.setLimit(
      m,
      p,
      this.softness,
      this.biasFactor,
      this.relaxationFactor
    ),
      this.constraint.setBreakingImpulseThreshold(this.breakingThreshold),
      this.constraint.enableAngularMotor(
        this.enableMotor,
        (this.motorTargetVelocity * Math.PI) / 180,
        this.maxMotorImpulse
      ),
      Ammo.destroy(r),
      Ammo.destroy(a),
      Ammo.destroy(s),
      this.app.systems.rigidbody.dynamicsWorld.addConstraint(
        this.constraint,
        !this.enableCollision
      ),
      this.activate();
  }),
  (Hinge.prototype.destroyConstraint = function () {
    this.constraint &&
      (this.app.systems.rigidbody.dynamicsWorld.removeConstraint(
        this.constraint
      ),
      Ammo.destroy(this.constraint),
      (this.constraint = null));
  }),
  (Hinge.prototype.activate = function () {
    this.entityA.rigidbody.activate(),
      this.entityB && this.entityB.rigidbody.activate();
  }),
  (Hinge.prototype.update = function (t) {
    if (this.debugRender) {
      var i = new pc.Vec3(),
        e = new pc.Vec3(),
        o = new pc.Vec3(),
        n = new pc.Vec3(),
        s = this.entityA.getWorldTransform();
      s.transformPoint(this.pivotA, i),
        s.transformVector(this.axisA, e),
        e.normalize().scale(0.5),
        o.add2(i, e),
        n.sub2(i, e),
        this.app.renderLine(this.entityA.getPosition(), i, this.debugColor),
        this.app.renderLine(o, n, this.debugColor),
        this.entityB &&
          this.app.renderLine(this.entityB.getPosition(), i, this.debugColor);
    }
  }); // kinematic.js

var Shadow = pc.createScript("shadow");
Shadow.attributes.add("offset", {
  title: "Offset",
  type: "vec3",
  default: [0.25, -0.25, 0],
}),
  Shadow.attributes.add("color", {
    title: "Color",
    type: "rgb",
    default: [0, 0, 0],
  }),
  (Shadow.prototype.initialize = function () {
    let t = this.app.assets.get(this.entity.sprite.spriteAsset);
    (this.shadow = new pc.Entity()),
      this.shadow.addComponent("sprite", {
        type: pc.SPRITETYPE_SIMPLE,
        spriteAsset: t,
      }),
      (this.shadow.name = this.entity.name + " Shadow"),
      this.shadow.setPosition(this.entity.getPosition()),
      this.shadow.setRotation(this.entity.getRotation()),
      this.shadow.setLocalScale(this.entity.getLocalScale()),
      (this.shadow.sprite.width = this.entity.sprite.width),
      (this.shadow.sprite.height = this.entity.sprite.height),
      (this.shadow.sprite.color = this.color),
      (this.shadow.sprite.layers = [1004]),
      (this.shadow.sprite.opacity = 0.25);
    var i = this.shadow.sprite._material.clone();
    (i.blendType = pc.BLEND_SUBTRACTIVE),
      (i.blendType.alphaWrite = !1),
      i.update(),
      (this.shadow.sprite._meshInstance.material = i),
      this.app.root.addChild(this.shadow);
  }),
  (Shadow.prototype.update = function (t) {
    this.shadow.setRotation(this.entity.getRotation()),
      this.shadow.setPosition(
        new pc.Vec3().copy(this.entity.getPosition()).add(this.offset)
      );
  });
var ShadowRenderTexture = pc.createScript("shadowRenderTexture");
ShadowRenderTexture.attributes.add("image", { type: "entity" }),
  ShadowRenderTexture.attributes.add("model", { type: "entity" }),
  (ShadowRenderTexture.prototype.initialize = function () {
    var e = new pc.Texture(this.app.graphicsDevice, {
        width: 1024,
        height: 1024,
        format: pc.PIXELFORMAT_R8_G8_B8,
      }),
      t = new pc.RenderTarget({ colorBuffer: e, depth: !0 });
    console.log(this.model);
    var r = this.model.render.meshInstances[0].material;
    (r.diffuseMap = t.colorBuffer), r.update(), console.log(this.image);
  }),
  (ShadowRenderTexture.prototype.update = function (e) {});
var RenderLayerTotexture = pc.createScript("renderLayerTotexture");
RenderLayerTotexture.attributes.add("layerName", { type: "string" }),
  (RenderLayerTotexture.prototype.initialize = function () {
    var e = new pc.Texture(this.app.graphicsDevice, {
      width: 512,
      height: 512,
      format: pc.PIXELFORMAT_R8_G8_B8,
      autoMipmap: !0,
    });
    (e.minFilter = pc.FILTER_LINEAR), (e.magFilter = pc.FILTER_LINEAR);
    var r = new pc.RenderTarget(this.app.graphicsDevice, e, { depth: !0 });
    this.app.scene.layers.getLayerByName(this.layerName).renderTarget = r;
  });
var ApplyTexture = pc.createScript("applyTexture");
ApplyTexture.attributes.add("layerName", { type: "string" }),
  (ApplyTexture.prototype.postInitialize = function () {
    this.app.scene.layers.getLayerByName(this.layerName);
    for (var e = this.entity.model.meshInstances, t = 0; t < e.length; ++t) {
      var r = e[t];
      (r.material.emissiveMap = renderTarget.colorBuffer), r.material.update();
    }
  }),
  (ApplyTexture.prototype.update = function (e) {});
var RotateCylinder = pc.createScript("rotateCylinder");
(RotateCylinder.prototype.initialize = function () {
  (this.x = pc.math.random(0, 90)),
    (this.y = pc.math.random(0, 90)),
    (this.z = pc.math.random(0, 90));
}),
  (RotateCylinder.prototype.update = function (t) {
    this.entity.rotate(this.x * t, this.y * t, this.z * t);
  });
var RenderToTexture = pc.createScript("renderToTexture");
RenderToTexture.attributes.add("applyTextureToEntity", {
  type: "entity",
  title: "Entity to Apply Texture to",
}),
  (RenderToTexture.prototype.initialize = function () {
    var e = new pc.Texture(this.app.graphicsDevice, {
        width: 512,
        height: 512,
        format: pc.PIXELFORMAT_R8_G8_B8,
      }),
      t = new pc.RenderTarget(this.app.graphicsDevice, e, { depth: !0 });
    this.entity.camera.renderTarget = t;
    for (
      var r = this.applyTextureToEntity.model.meshInstances, i = 0;
      i < r.length;
      ++i
    ) {
      var a = r[i];
      (a.material.emissiveMap = t.colorBuffer), a.material.update();
    }
  }),
  (RenderToTexture.prototype.update = function (e) {});
var RenderCameraToelement = pc.createScript("renderCameraToelement");
RenderCameraToelement.attributes.add("elementTags", {
  type: "string",
  array: !0,
  description: "Render to elements that have these tags",
}),
  RenderCameraToelement.attributes.add("renderResolution", {
    type: "vec2",
    description: "Resolution to render at",
  }),
  RenderCameraToelement.attributes.add("renderOnce", {
    type: "boolean",
    description: "Renders the first frame only",
  }),
  RenderCameraToelement.attributes.add("fixedCameraSize", {
    type: "boolean",
    default: !1,
  }),
  RenderCameraToelement.attributes.add("forcedResolution", {
    type: "vec2",
    default: [0, 0],
  }),
  (RenderCameraToelement.prototype.initialize = function () {
    this.fixedCameraSize ||
      (this.entity.camera.orthoHeight =
        CameraManager.instance.entity.camera.orthoHeight),
      this.createNewRenderTexture(),
      (this.renderOnceFrameCount = 0),
      this.on(
        "destroy",
        function () {
          this.renderTarget.destroy(), this.texture.destroy();
        },
        this
      );
  }),
  (RenderCameraToelement.prototype.update = function (e) {
    this.renderOnce &&
      ((this.renderOnceFrameCount += 1),
      this.renderOnceFrameCount > 4 && (this.entity.enabled = !1));
  }),
  (RenderCameraToelement.prototype.createNewRenderTexture = function () {
    var e = this.app.graphicsDevice;
    if (this.texture && this.renderTarget) {
      var t = this.renderTarget,
        r = this.texture;
      (this.renderTarget = null),
        (this.texture = null),
        t.destroy(),
        r.destroy();
    }
    let a = e.width,
      n = e.height;
    this.forcedResolution.length() > 0 &&
      ((a = this.forcedResolution.x), (n = this.forcedResolution.y));
    var i = new pc.Texture(e, {
      width: a,
      height: n,
      format: pc.PIXELFORMAT_R8_G8_B8_A8,
      autoMipmap: !0,
      addressU: pc.ADDRESS_CLAMP_TO_EDGE,
      addressV: pc.ADDRESS_CLAMP_TO_EDGE,
    });
    (i.minFilter = pc.FILTER_LINEAR), (i.magFilter = pc.FILTER_LINEAR);
    var o = new pc.RenderTarget(e, i, { depth: !0, flipY: !0, samples: 2 });
    (this.entity.camera.renderTarget = o),
      this.fixedCameraSize ||
        (this.entity.camera.orthoHeight =
          CameraManager.instance.entity.camera.orthoHeight),
      (this.texture = i),
      (this.renderTarget = o),
      this.assignTextureToElements(this.texture);
  }),
  (RenderCameraToelement.prototype.assignTextureToElements = function (e) {
    for (var t, r = 0; r < this.elementTags.length; ++r) {
      t = this.app.root.findByTag(this.elementTags[r]);
      for (var a = 0; a < t.length; ++a) t[a].element.texture = e;
    }
  });
var SetToscreenSize = pc.createScript("setToscreenSize");
(SetToscreenSize.prototype.initialize = function () {
  (this.shadowCamera = GameManager.instance.camera.findByName("Shadow Camera")),
    (this.sausageCamera =
      GameManager.instance.camera.findByName("Sausage Camera")),
    setTimeout(() => {
      this.updateScreens(
        2 * this.app.graphicsDevice._width,
        2 * this.app.graphicsDevice._height
      );
    }, 100),
    this.app.graphicsDevice.on("resizecanvas", (e, a) => {
      this.updateScreens(e, a);
    });
}),
  (SetToscreenSize.prototype.updateScreens = function (e, a) {
    this.entity.screen.referenceResolution.set(e / 2, a / 2),
      (this.shadowCamera.script.renderCameraToelement.renderResolution =
        new pc.Vec2(e / 2, a / 2)),
      this.shadowCamera.script.renderCameraToelement.createNewRenderTexture(),
      (this.sausageCamera.script.renderCameraToelement.renderResolution =
        new pc.Vec2(e / 2, a / 2)),
      this.sausageCamera.script.renderCameraToelement.createNewRenderTexture();
  }),
  (SetToscreenSize.prototype.update = function (e) {});
var SetMass = pc.createScript("setMass");
SetMass.attributes.add("mass", {
  title: "Mass",
  type: "number",
  default: 0.001,
}),
  (SetMass.prototype.initialize = function () {
    this.entity.rigidbody.mass = this.mass;
  }),
  (SetMass.prototype.update = function (t) {});
var ButtonTween = pc.createScript("buttonTween");
(ButtonTween.prototype.initialize = function () {}),
  (ButtonTween.prototype.update = function (t) {});
var Uiscaling = pc.createScript("uiscaling");
Uiscaling.attributes.add("landScapeScaleMultiplier", {
  type: "number",
  default: 1,
}),
  Uiscaling.attributes.add("portraitScaleMultiplier", {
    type: "number",
    default: 1,
  }),
  (Uiscaling.prototype.initialize = function () {
    (this.baseScale = new pc.Vec3().copy(this.entity.getLocalScale())),
      this.app.graphicsDevice.on("resizecanvas", (i, e) => {
        this.resize(i, e);
      }),
      this.resize(
        2 * this.app.graphicsDevice._width,
        2 * this.app.graphicsDevice._height
      );
  }),
  (Uiscaling.prototype.resize = function (i, e) {
    let t = i / 2 / (e / 2),
      a = 0.5625,
      c = (1 * t) / a;
    (c =
      t > 1
        ? (1 * this.landScapeScaleMultiplier) / a
        : (1 * this.portraitScaleMultiplier) / a),
      this.entity.setLocalScale(this.baseScale.x * c, this.baseScale.y * c, c);
  });
var LevelGenerator = pc.createScript("levelGenerator");
LevelGenerator.attributes.add("uiTemplate", {
  type: "asset",
  assetType: "template",
}),
  LevelGenerator.attributes.add("sausageTemplate", {
    type: "asset",
    assetType: "template",
  }),
  LevelGenerator.attributes.add("levelData", {
    type: "asset",
    assetType: "json",
  }),
  LevelGenerator.attributes.add("decks", { type: "asset", assetType: "json" }),
  (LevelGenerator.prototype.initialize = function () {
    LevelGenerator.instance = this;
  }),
  (LevelGenerator.prototype.spawnLevel = function (e) {
    let t;
    return (
      SaveManager.instance.getItem("lastLevelId") == e
        ? (t = SaveManager.instance.getItem("currentLevelDeck"))
        : (SaveManager.instance.setItem("lastLevelId", e),
          (t =
            Object.values(this.levelData.resources[0]).length >
              GameManager.instance.levelId &&
            null !=
              Object.values(this.levelData.resources[0])[
                GameManager.instance.levelId
              ]
              ? Object.values(this.levelData.resources[0])[
                  GameManager.instance.levelId
                ]
              : this.generateLevel(GameManager.instance.levelId)),
          SaveManager.instance.setItem("currentLevelDeck", t)),
      this.instantiateLevel(t)
    );
  }),
  (LevelGenerator.prototype.instantiateLevel = function (e) {
    var t = new pc.Entity({ name: "Level" });
    this.app.root.addChild(t),
      (t.localPosition = new pc.Vec3(0, 0, 0)),
      t.setPosition(new pc.Vec3(0, 0, 0));
    let a = e.patterns;
    "Pattern_Start_Onboarding" != a[0]
      ? ((start = this.app.assets
          .find("Pattern_Start", "template")
          .resource.instantiate()),
        doStuffToEntity(
          this.app.assets.find("Pattern_Start", "template").resource
            ._templateRoot,
          start
        ),
        start.setPosition(-17.198, -2.65, 0),
        t.addChild(start),
        (this.position = start.findByName("End").getPosition()))
      : (this.position = new pc.Vec3(-1.259, 15.747, 0)),
      a.forEach((e) => {
        console.log(e);
        let a = this.app.assets.find(e, "template", e).resource,
          n = this.app.assets.find(e, "template", e).resource.instantiate();
        (n.localPosition = new pc.Vec3(0, 0, 0)),
          n.setPosition(this.position),
          doStuffToEntity(a._templateRoot, n),
          (this.position = n.findByName("End").getPosition()),
          t.addChild(n);
      });
    let n = this.uiTemplate.resource,
      s = this.uiTemplate.resource.instantiate();
    return (
      doStuffToEntity(n._templateRoot, s),
      t.addChild(s),
      ThemeManager.instance.changeTheme(),
      t
    );
  }),
  (LevelGenerator.prototype.generateLevel = function (e) {
    let t = { patterns: [] },
      a = Math.floor(4 * Math.random()) + 3;
    for (let n = 0; n < a; n++) t.patterns.push(this.pickRandomPattern(e));
    return t.patterns.push("Pattern_End"), t;
  }),
  (LevelGenerator.prototype.pickRandomPattern = function (e) {
    let t = 0,
      a = [];
    Object.values(this.decks.resources[0]).forEach((n) => {
      null != n && (e < n.unlockLevel || ((t += n.weight), a.push(n)));
    });
    let n = Math.floor(Math.random() * (1 + t - 0)) + 0,
      s = -1;
    for (; n > 0 && s < a.length; ) s++, (n -= a[s].weight);
    s = Math.min(Math.max(s, 0), a.length);
    let r = a[s];
    return r.index >= r.patterns.length && (r.index = 0), r.patterns[r.index++];
  }),
  (LevelGenerator.prototype.update = function (e) {});
var TestPrefab = pc.createScript("testPrefab");
TestPrefab.attributes.add("toClone", { type: "entity" }),
  (TestPrefab.prototype.initialize = function () {
    let t = this.toClone.clone(!0),
      e = t.getPosition();
    t.setPosition(e.x + 5, e.y, e.z), this.app.root.addChild(t);
  }),
  (TestPrefab.prototype.update = function (t) {});
var LoadSceneOnButtonClick = pc.createScript("loadSceneOnButtonClick");
LoadSceneOnButtonClick.attributes.add("sceneName", { type: "string" }),
  (LoadSceneOnButtonClick.prototype.initialize = function () {
    (this.clicked = !1),
      this.entity.button.on(
        "click",
        function () {
          "Shop" == this.sceneName
            ? GameManager.instance.shopScene()
            : "Game" == this.sceneName
            ? GameManager.instance.startLevel()
            : "Menu" == this.sceneName
            ? GameManager.instance.mainMenu()
            : "Return" == this.sceneName
            ? "Main Menu" == GameManager.instance.lastScreen.name
              ? GameManager.instance.mainMenu()
              : GameManager.instance.startLevel()
            : "Unlock" == this.sceneName
            ? GameManager.instance.unlockLevelSkin("Cosmonaut")
            : "Skip" == this.sceneName &&
              PokiManager.instance.rewarded(
                (e) => {
                  e &&
                    (GameManager.instance.levelId++,
                    (GameManager.instance.levelStartFlag = !0));
                },
                {
                  category: "level-skip",
                  detail: GameManager.instance.levelId.toString(),
                  placement: "gameplay",
                }
              );
        },
        this
      );
  }),
  (LoadSceneOnButtonClick.prototype.ad = async function (e) {});
var Ribbon = pc.createScript("ribbon");
Ribbon.attributes.add("lifetime", { type: "number", default: 0.5 }),
  Ribbon.attributes.add("xoffset", { type: "number", default: -0.8 }),
  Ribbon.attributes.add("yoffset", { type: "number", default: 1 }),
  Ribbon.attributes.add("height", { type: "number", default: 0.4 }),
  Ribbon.attributes.add("color", { type: "rgb", default: [1, 0, 0] }),
  Ribbon.attributes.add("layer", { type: "string" });
var MAX_VERTICES = 600,
  VERTEX_SIZE = 4;
(Ribbon.prototype.create = function (t) {
  (this.timer = 0),
    (this.node = null),
    (this.vertices = []),
    (this.vertexData = new Float32Array(MAX_VERTICES * VERTEX_SIZE)),
    (this.entity.model = null),
    (this.rigidbody = null);
  for (var e = this.entity; e && !this.rigidbody; )
    (this.rigidbody = e.rigidbody), (e = e.parent);
}),
  (Ribbon.prototype.initialize = function () {
    this.once("destroy", this.destroy, this), this.create();
    var t = {
        attributes: { aPositionAge: pc.SEMANTIC_POSITION },
        vshader: [
          "attribute vec4 aPositionAge;",
          "",
          "uniform mat4 matrix_viewProjection;",
          "uniform float trail_time;",
          "",
          "varying float vAge;",
          "",
          "void main(void)",
          "{",
          "    vAge = trail_time - aPositionAge.w;",
          "    gl_Position = matrix_viewProjection * vec4(aPositionAge.xyz, 1.0);",
          "}",
        ].join("\n"),
        fshader: [
          "precision mediump float;",
          "",
          "varying float vAge;",
          "",
          "uniform float trail_lifetime;",
          "",
          "uniform vec3 uColor;",
          "",
          "void main(void)",
          "{",
          "    gl_FragColor = vec4(uColor[0], uColor[1], uColor[2], (0.5 - (vAge / trail_lifetime)) * 1.0);",
          "}",
        ].join("\n"),
      },
      e = new pc.Shader(this.app.graphicsDevice, t),
      i = new pc.scene.Material();
    (i.shader = e),
      i.setParameter("trail_time", 0),
      i.setParameter("trail_lifetime", this.lifetime),
      i.setParameter("uColor", [this.color.r, this.color.g, this.color.b]),
      (i.cull = pc.CULLFACE_NONE),
      (i.blend = !0),
      (i.blendSrc = pc.BLENDMODE_SRC_ALPHA),
      (i.blendDst = pc.BLENDMODE_ONE_MINUS_SRC_ALPHA),
      (i.blendEquation = pc.BLENDMODE_ONE_MINUS_CONSTANT_ALPHA),
      (i.blendType = pc.BLEND_NORMAL),
      (i.depthWrite = !1);
    var o = new pc.VertexFormat(this.app.graphicsDevice, [
        {
          semantic: pc.SEMANTIC_POSITION,
          components: 4,
          type: pc.ELEMENTTYPE_FLOAT32,
        },
      ]),
      s = new pc.VertexBuffer(
        this.app.graphicsDevice,
        o,
        MAX_VERTICES * VERTEX_SIZE,
        pc.USAGE_DYNAMIC
      ),
      r = new pc.scene.Mesh();
    (r.vertexBuffer = s),
      (r.indexBuffer[0] = null),
      (r.primitive[0].type = pc.PRIMITIVE_TRISTRIP),
      (r.primitive[0].base = 0),
      (r.primitive[0].count = 0),
      (r.primitive[0].indexed = !1);
    var a = new pc.scene.GraphNode(),
      n = new pc.scene.MeshInstance(a, r, i);
    (n.layer = pc.scene.LAYER_WORLD),
      n.updateKey(),
      (this.entity.model = new pc.scene.Model()),
      (this.entity.model.graph = a),
      this.entity.model.meshInstances.push(n),
      (this.model = this.entity.model),
      this.setNode(a),
      this.on("attr:color", this.onColorChange);
  }),
  (Ribbon.prototype.reset = function () {
    (this.timer = 0), (this.vertices = []);
  }),
  (Ribbon.prototype.spawn = function () {
    var t = this.node;
    this.vertices.unshift({
      spawnTime: this.timer,
      yaxis: t.up.clone().scale(this.height),
      pos: t.getPosition().clone(),
    });
  }),
  (Ribbon.prototype.clearOld = function () {
    for (var t = this.vertices.length - 1; t >= 0; t--) {
      var e = this.vertices[t];
      if (!(this.timer - e.spawnTime >= this.lifetime)) return;
      this.vertices.pop();
    }
  }),
  (Ribbon.prototype.copyToArrayBuffer = function () {
    for (
      var t = this.xoffset, e = this.yoffset, i = 0;
      i < this.vertices.length;
      i++
    ) {
      var o = this.vertices[i];
      o.yaxis.scale(0.95);
      var s = o.pos,
        r = o.yaxis;
      (this.vertexData[8 * i + 0] = s.x + r.x * t),
        (this.vertexData[8 * i + 1] = s.y + r.y * t),
        (this.vertexData[8 * i + 2] = s.z + r.z * t),
        (this.vertexData[8 * i + 3] = o.spawnTime),
        (this.vertexData[8 * i + 4] = s.x + r.x * e),
        (this.vertexData[8 * i + 5] = s.y + r.y * e),
        (this.vertexData[8 * i + 6] = s.z + r.z * e),
        (this.vertexData[8 * i + 7] = o.spawnTime);
    }
  }),
  (Ribbon.prototype.updateNumActive = function () {
    this.model.meshInstances[0].mesh.primitive[0].count =
      2 * this.vertices.length;
  }),
  (Ribbon.prototype.postUpdate = function (t) {
    var e = this.entity.getPosition();
    if (
      (this.entity.model.graph.setPosition(e.x, e.y, e.z - 0.1), this.rigidbody)
    ) {
      var i = this.rigidbody.linearVelocity,
        o = (180 * Math.atan2(i.y, i.x)) / Math.PI;
      this.entity.model.graph.setEulerAngles(0, 0, o);
    }
    if (
      ((this.timer += t),
      this.model.meshInstances[0].material.setParameter(
        "trail_time",
        this.timer
      ),
      this.clearOld(),
      this.spawn(),
      this.vertices.length > 1)
    ) {
      this.copyToArrayBuffer(), this.updateNumActive();
      var s = this.model.meshInstances[0].mesh.vertexBuffer;
      new Float32Array(s.lock()).set(this.vertexData),
        s.unlock(),
        this.app.scene.containsModel(this.model) ||
          this.app.scene.addModel(this.model);
    } else
      this.app.scene.containsModel(this.model) &&
        this.app.scene.removeModel(this.model);
  }),
  (Ribbon.prototype.setNode = function (t) {
    this.node = t;
  }),
  (Ribbon.prototype.destroy = function () {
    this.reset(),
      this.app.scene.removeModel(this.model),
      this.off("attr:color");
  }),
  (Ribbon.prototype.onColorChange = function (t) {
    this.model.meshInstances[0].material.setParameter("uColor", [
      t.r,
      t.g,
      t.b,
    ]);
  });
var Rotate = pc.createScript("rotate");
Rotate.attributes.add("rotationSpeed", { type: "number" }),
  (Rotate.prototype.initialize = function () {}),
  (Rotate.prototype.update = function (t) {
    this.entity.setEulerAngles(
      this.entity.getEulerAngles().x,
      0,
      this.entity.getEulerAngles().z + this.rotationSpeed
    );
  });
var SetGravity = pc.createScript("setGravity");
SetGravity.attributes.add("gravityScaleY", { type: "number" }),
  SetGravity.attributes.add("baseGravityY", { type: "number" }),
  (SetGravity.prototype.postInitialize = function () {
    this.app.systems.rigidbody.dynamicsWorld.getGravity();
    setTimeout(() => {
      this.entity.rigidbody.body.setGravity(
        new Ammo.btVector3(0, this.baseGravityY * this.gravityScaleY, 0)
      ),
        this.entity.rigidbody.body.setLinearVelocity(
          new Ammo.btVector3(0, 0, 0)
        );
    }, 100);
  });
var Ccd = pc.createScript("ccd");
Ccd.attributes.add("motionThreshold", {
  type: "number",
  default: 1,
  title: "Motion Threshold",
  description: "Number of meters moved in one frame before CCD is enabled",
}),
  Ccd.attributes.add("sweptSphereRadius", {
    type: "number",
    default: 0.2,
    title: "Swept Sphere Radius",
    description:
      "This should be below the half extent of the collision volume. E.g For an object of dimensions 1 meter, try 0.2",
  }),
  Ccd.attributes.add("contactProcessingThreshold", {
    type: "number",
    default: 0,
    title: "Contact Processing Threshold",
    description:
      'This should be below the half extent of the collision volume. E.g For an object of dimensions 1 meter, try 0.2the constraint solver can discard solving contacts, if the distance is above this threshold. 0 by default. \n Note that using contacts with positive distance can improve stability. It increases, however, the chance of colliding with degerate contacts, such as "interior" triangle edges',
  }),
  Ccd.attributes.add("fixedTimeStep", {
    type: "number",
    default: 60,
    title: "Fixed Time Step",
    description:
      "Physics fixed time step, the number of times the physics engine step refreshes per second, changing this to more then 60 can effect performance",
  }),
  (Ccd.prototype.initialize = function () {
    var e;
    (this.app.systems.rigidbody.fixedTimeStep = 1 / this.fixedTimeStep),
      (e = this.entity.rigidbody.body).setCcdMotionThreshold(
        this.motionThreshold
      ),
      e.setCcdSweptSphereRadius(this.sweptSphereRadius),
      e.setContactProcessingThreshold(this.contactProcessingThreshold),
      this.on("attr:motionThreshold", function (t, i) {
        (e = this.entity.rigidbody.body).setCcdMotionThreshold(t);
      }),
      this.on("attr:sweptSphereRadius", function (t, i) {
        (e = this.entity.rigidbody.body).setCcdSweptSphereRadius(t);
      }),
      this.on("attr:contactProcessingThreshold", function (t, i) {
        (e = this.entity.rigidbody.body).setContactProcessingThreshold(t);
      });
  });
doStuffToEntity = function (e, t) {
  if (null != e && "Untitled" != e.name) {
    var a = 0;
    for (a = 0; a < t.children.length; ++a)
      doStuffToEntity(e.children[a], t.children[a]);
  }
  t.hasOwnProperty("c") &&
    null != t.c.element &&
    e &&
    t.setPosition(e.getPosition()),
    null != t.screen && (t.c.screen.scaleBlend = e.c.screen.scaleBlend),
    null != t.sprite &&
      null != e &&
      ("Untitled" == e.name && (e = e.parent.findByName(t.name)),
      null != e &&
        "Untitled" != e.name &&
        e.sprite &&
        ((t.sprite.width = e.sprite.width),
        (t.sprite.height = e.sprite.height)));
};
var GameManager = pc.createScript("gameManager");
GameManager.attributes.add("camera", { type: "entity" }),
  GameManager.attributes.add("menu", { type: "string" }),
  GameManager.attributes.add("shop", { type: "string" }),
  GameManager.attributes.add("sausageTemplate", {
    type: "asset",
    assetType: "template",
  }),
  GameManager.attributes.add("winParticles", { type: "entity" }),
  GameManager.attributes.add("winText", { type: "entity" }),
  GameManager.attributes.add("test", { type: "boolean", default: !1 }),
  GameManager.attributes.add("skinList", { type: "asset", assetType: "json" }),
  (GameManager.prototype.initialize = function () {
    (GameManager.instance = this),
      (this.app.systems.rigidbody.fixedTimeStep = 1 / 120),
      // (console.log = () => {}),
      (this.levelId = SaveManager.instance.getItem("LevelId")),
      (this.levelStarted = !1),
      (this.finished = !1),
      (this.respawned = !1);
  }),
  (GameManager.prototype.postInitialize = function () {
    this.test || SaveManager.instance.showTutorial
      ? SaveManager.instance.showTutorial && this.startLevel()
      : this.mainMenu(),
      setTimeout(() => {});
  }),
  (GameManager.prototype.spawnCharacter = function () {
    let e = this.app.assets
      .find(SaveManager.instance.getItem("CurrentSkin"), "template")
      .resource.instantiate();
    doStuffToEntity(
      this.app.assets.find(
        SaveManager.instance.getItem("CurrentSkin"),
        "template"
      ).resource._templateRoot,
      e
    ),
      e.setPosition(this.level.findByName("SpawnPoint").getPosition()),
      (e.script.playerController.enabled = !0),
      this.app.root.addChild(e),
      (this.playerController = e.script.playerController);
  }),
  (GameManager.prototype.mainMenu = function () {
    PokiSDK.gameplayStop(),
      PokiManager.instance.displayBanner(),
      (this.levelStarted = !1),
      SkinsButton.instance.showButton();
    let e = this.app.assets
      .find("Main Menu", "template")
      .resource.instantiate();
    return (
      doStuffToEntity(
        this.app.assets.find("Main Menu", "template").resource._templateRoot,
        e
      ),
      this.app.root.addChild(e),
      e.setPosition(new pc.Vec3(0, 18, 0)),
      (e.findByName("Logo").findByName("LevelText").element.text =
        // "CHAPTER " + this.levelId + "/" + 10 * Math.ceil(this.levelId / 10)),
        "" + this.levelId + "/" + 10 * Math.ceil(this.levelId / 10)),
      (this.start = this.app.assets
        .find("Pattern_Start", "template")
        .resource.instantiate()),
      doStuffToEntity(
        this.app.assets.find("Pattern_Start", "template").resource
          ._templateRoot,
        this.start
      ),
      this.start.setPosition(-17.198, -2.65 - e.getPosition().y, 0),
      e.addChild(this.start),
      (this.app.timeScale = 1),
      this.level && this.level.destroy(),
      (this.level = e),
      this.playerController && this.playerController.entity.destroy(),
      this.spawnCharacter(),
      (this.playerController.activated = !1),
      e
    );
  }),
  (GameManager.prototype.transitionScene = function () {
    this.level && this.level.destroy(),
      (this.level.name = "Main Menu"),
      this.playerController && this.playerController.entity.destroy();
    let e = this.app.assets
      .find("Transition", "template")
      .resource.instantiate();
    doStuffToEntity(
      this.app.assets.find("Transition", "template").resource._templateRoot,
      e
    ),
      this.app.root.addChild(e);
  }),
  (GameManager.prototype.shopScene = function () {
    (this.levelStarted = !1),
      (this.app.timeScale = 1),
      PokiSDK.gameplayStop(),
      PokiManager.instance.displayBanner(),
      SkinsButton.instance.hideButton(0.1),
      this.playerController && this.playerController.entity.destroy();
    let e = this.app.assets.find("Shop", "template").resource.instantiate();
    return (
      doStuffToEntity(
        this.app.assets.find("Shop", "template").resource._templateRoot,
        e
      ),
      this.app.root.addChild(e),
      (this.lastScreen = this.level),
      this.level && this.level.destroy(),
      (this.level = e),
      e
    );
  }),
  (GameManager.prototype.unlockLevelSkin = function (e) {
    // xixi 第10关解锁皮肤
    // console.log("hehe unlockLevelSkin")
    // return;
    let t = this.app.assets.find("New Skin", "template").resource.instantiate();
    if (
      (doStuffToEntity(
        this.app.assets.find("New Skin", "template").resource._templateRoot,
        t
      ),
      e)
    ) {
      t.script.newSkinPopup.skin = e.name;
      let a = SaveManager.instance.getItem("Skins");
      a.push(e.name), SaveManager.instance.setItem("Skins", a);
    } else {
      let e = Object.values(this.skinList.resources[0]).filter(
        (e) =>
          "REWARDED" == e.type &&
          !SaveManager.instance.getItem("Skins").includes(e.name)
      );
      if (0 == e.length) return;
      let a = Math.round(Math.random() * (e.length - 1 - 0) + 0);
      (t.script.newSkinPopup.rewarded = !0),
        (t.script.newSkinPopup.skin = e[a].name);
    }
    this.app.root.addChild(t);
  }),
  (GameManager.prototype.startLevel = function (e) {
    PokiSDK.gameplayStart(),
      PokiSDK.customEvent("game", "segment", {
        label: "level",
        value: this.levelId,
      }),
      SkinsButton.instance.showButton(),
      (this.app.timeScale = 1),
      setTimeout(() => {
        SaveManager.instance.setItem("LevelId", this.levelId);
        let t = !1;
        e || (t = this.checkSkinUnlock()),
          this.level && this.level.destroy(),
          (this.level = LevelGenerator.instance.spawnLevel(this.levelId)),
          (this.levelStarted = !0),
          this.playerController && this.playerController.entity.destroy(),
          this.spawnCharacter(),
          (this.playerController.activated = !t),
          (this.finished = !1);
      }, 1);
  }),
  (GameManager.prototype.finishedLevel = function () {
    PokiManager.instance.displayBanner(),
      PokiSDK.gameplayStop(),
      PauseButton.instance.hide(),
      (this.finished = !0),
      (this.respawned = !1),
      (this.levelStarted = !1),
      (this.app.timeScale = 1),
      SkipLevel.instance.hideButton();
    let e = { x: 1 };
    CameraManager.instance.flash(2),
      this.winParticles.children.forEach((e) => {
        e.enabled = !0;
      }),
      setTimeout(() => {
        this.entity
          .tween(e)
          .to({ x: 0.1 }, 0.3, pc.Linear)
          .on("update", () => {
            (this.app.timeScale = e.x),
              this.winParticles.children.forEach((e) => {
                e.particlesystem.emitter.fixedTimeStep =
                  0.016 * this.app.timeScale;
              });
          })
          .start(),
          (this.winText.enabled = !0),
          this.winText.findByName("MaskRound").script.maskTransform.test(),
          this.winText
            .tween(new pc.Vec3(0, 0, 180))
            .rotate(new pc.Vec3(0, 0, 0), 0.5, pc.Linear)
            .on("complete", () => {
              this.winText
                .tween(new pc.Vec3(0, 0, 0))
                .rotate(new pc.Vec3(0, 0, -10), 0.5, pc.Linear)
                .on("complete", () => {})
                .start();
            })
            .start();
      }, 200);
    let t = { x: this.camera.camera.orthoHeight };
    this.entity
      .tween(t)
      .to({ x: t.x / 1.5 }, 0.5, pc.Linear)
      .on("update", () => {
        this.camera.camera.orthoHeight = t.x;
      })
      .start(),
      this.winParticles.children.forEach((e) => {
        e.particlesystem.reset(), e.particlesystem.play();
      }),
      setTimeout(() => {
        PokiManager.instance.interstitial(() => {
          this.winParticles.children.forEach((e) => {
            e.enabled = !1;
          });
          let e = { x: this.camera.camera.orthoHeight };
          this.entity
            .tween(e)
            .to({ x: 25 }, 0.3, pc.Linear)
            .on("update", () => {
              this.camera.camera.orthoHeight = e.x;
            })
            .start(),
            this.levelId++,
            (this.winText.enabled = !1),
            this.transitionScene();
        });
      }, 2e3);
  }),
  (GameManager.prototype.checkSkinUnlock = function (e) {
    let t = Object.values(this.skinList.resources[0]).find(
      (e) => e.level == this.levelId
    );
    // xixi 到此关卡，视频解锁机会
    return t && !SaveManager.instance.getItem("Skins").includes(t.name)
      ? (this.unlockLevelSkin(t), !0)
      // xixi 别视频解锁了，挡住流程了
      : false && (1 == Math.round(5 * Math.random() + 0) && this.levelId > 10)
      ? (this.unlockLevelSkin(), !0)
      : (SkinsButton.instance.showButton(), !1);
    return (SkinsButton.instance.showButton(), !1);
  }),
  (GameManager.prototype.postUpdate = function (e) {
    (window.compound = this.app.systems.rigidbody._compounds),
      (window.dynamic = this.app.systems.rigidbody._dynamic),
      (window.kinematic = this.app.systems.rigidbody._kinematic),
      this.levelStartFlag && ((this.levelStartFlag = !1), this.startLevel());
  });
var TriggerVolume = pc.createScript("triggerVolume");
TriggerVolume.prototype.initialize = function () {
  this.entity.collision.on("triggerenter", function (e) {
    !GameManager.instance.finished &&
      e.tags._list.includes("Segment") &&
      GameManager.instance.finishedLevel();
  }),
    this.entity.collision.on("triggerleave", function (e) {});
};
var ShadeNormal = pc.createScript("shadeNormal");
ShadeNormal.attributes.add("target", { type: "entity" }),
  (ShadeNormal.prototype.initialize = function () {
    console.log("Material", this.entity),
      (this.entity.sprite.material = this.target.sprite.material),
      this.entity.sprite.material.update(),
      console.log("Modified", this.entity.sprite),
      console.log("Other", this.target.sprite.material);
  }),
  (ShadeNormal.prototype.update = function (t) {});
var HeightToNormal = pc.createScript("heightToNormal");
HeightToNormal.attributes.add("blurIterations", { type: "number", default: 2 }),
  HeightToNormal.attributes.add("strength", {
    type: "number",
    min: 0,
    max: 1,
    default: 1,
  }),
  (pc.GraphicsDevice.prototype.setUniform = function (e, r) {
    this.scope.resolve(e).setValue(r);
  }),
  (HeightToNormal.createShaders = function (e) {
    var r = [
        "attribute vec2 aPosition;",
        "",
        "varying vec2 vUv0;",
        "",
        "void main()",
        "{",
        "    gl_Position = vec4(aPosition, 0.0, 1.0);",
        "    vUv0 = aPosition.xy * 0.5 + 0.5;",
        "}",
      ].join("\n"),
      t = [
        "precision " + e.graphicsDevice.precision + " float;",
        "",
        "varying vec2 vUv0;",
        "",
        "uniform sampler2D uColorBuffer;",
        "",
        "void main()",
        "{",
        "    gl_FragColor = texture2D(uColorBuffer, vUv0);",
        "}",
      ].join("\n"),
      v = [
        "precision " + e.graphicsDevice.precision + " float;",
        "",
        "varying vec2 vUv0;",
        "",
        "uniform vec2 uResolution;",
        "uniform sampler2D uColorBuffer;",
        "uniform vec2 uDirection;",
        "",
        "vec4 blur(sampler2D image, vec2 uv, vec2 resolution, vec2 direction) {",
        "    vec4 color = vec4(0.0);",
        "    vec2 off1 = vec2(1.411764705882353) * direction;",
        "    vec2 off2 = vec2(3.2941176470588234) * direction;",
        "    vec2 off3 = vec2(5.176470588235294) * direction;",
        "    color += texture2D(image, uv) * 0.1964825501511404;",
        "    color += texture2D(image, uv + (off1 / resolution)) * 0.2969069646728344;",
        "    color += texture2D(image, uv - (off1 / resolution)) * 0.2969069646728344;",
        "    color += texture2D(image, uv + (off2 / resolution)) * 0.09447039785044732;",
        "    color += texture2D(image, uv - (off2 / resolution)) * 0.09447039785044732;",
        "    color += texture2D(image, uv + (off3 / resolution)) * 0.010381362401148057;",
        "    color += texture2D(image, uv - (off3 / resolution)) * 0.010381362401148057;",
        "    return color;",
        "}",
        "",
        "void main()",
        "{",
        "    gl_FragColor = blur(uColorBuffer, vUv0, uResolution, uDirection);",
        "}",
      ].join("\n"),
      o = [
        "precision " + e.graphicsDevice.precision + " float;",
        "",
        "varying vec2 vUv0;",
        "",
        "uniform vec2 uResolution;",
        "uniform float dz;",
        "uniform float invertR;",
        "uniform float invertG;",
        "uniform float invertH;",
        "uniform int type;",
        "uniform sampler2D uHeightMap;",
        "",
        "void main()",
        "{",
        "    vec2 step = vec2(-1.0 / uResolution.x, -1.0 / uResolution.y);",
        "    vec2 tlv = vec2(vUv0.x - step.x, vUv0.y + step.y );",
        "    vec2 lv  = vec2(vUv0.x - step.x, vUv0.y          );",
        "    vec2 blv = vec2(vUv0.x - step.x, vUv0.y - step.y);",
        "    vec2 tv  = vec2(vUv0.x         , vUv0.y + step.y);",
        "    vec2 bv  = vec2(vUv0.x         , vUv0.y - step.y);",
        "    vec2 trv = vec2(vUv0.x + step.x, vUv0.y + step.y );",
        "    vec2 rv  = vec2(vUv0.x + step.x, vUv0.y          );",
        "    vec2 brv = vec2(vUv0.x + step.x, vUv0.y - step.y);",
        "    tlv = vec2(tlv.x >= 0.0 ? tlv.x : (1.0 + tlv.x),  tlv.y >= 0.0\t? tlv.y : (1.0  + tlv.y));",
        "    tlv = vec2(tlv.x < 1.0  ? tlv.x : (tlv.x - 1.0 ), tlv.y < 1.0  ? tlv.y : (tlv.y - 1.0 ));",
        "    lv  = vec2( lv.x >= 0.0 ?  lv.x : (1.0 + lv.x),   lv.y  >= 0.0 ?  lv.y : (1.0  +  lv.y));",
        "    lv  = vec2( lv.x < 1.0  ?  lv.x : ( lv.x - 1.0 ), lv.y  < 1.0  ?  lv.y : ( lv.y - 1.0 ));",
        "    blv = vec2(blv.x >= 0.0 ? blv.x : (1.0 + blv.x),  blv.y >= 0.0 ? blv.y : (1.0  + blv.y));",
        "    blv = vec2(blv.x < 1.0  ? blv.x : (blv.x - 1.0 ), blv.y < 1.0  ? blv.y : (blv.y - 1.0 ));",
        "    tv  = vec2( tv.x >= 0.0 ?  tv.x : (1.0 + tv.x),   tv.y  >= 0.0 ? tv.y  : (1.0  +  tv.y));",
        "    tv  = vec2( tv.x < 1.0  ?  tv.x : ( tv.x - 1.0 ), tv.y  < 1.0  ? tv.y  : ( tv.y - 1.0 ));",
        "    bv  = vec2( bv.x >= 0.0 ?  bv.x : (1.0 + bv.x),   bv.y  >= 0.0 ? bv.y  : (1.0  +  bv.y));",
        "    bv  = vec2( bv.x < 1.0  ?  bv.x : ( bv.x - 1.0 ), bv.y  < 1.0  ? bv.y  : ( bv.y - 1.0 ));",
        "    trv = vec2(trv.x >= 0.0 ? trv.x : (1.0 + trv.x),  trv.y >= 0.0 ? trv.y : (1.0  + trv.y));",
        "    trv = vec2(trv.x < 1.0  ? trv.x : (trv.x - 1.0 ), trv.y < 1.0  ? trv.y : (trv.y - 1.0 ));",
        "    rv  = vec2( rv.x >= 0.0 ?  rv.x : (1.0 + rv.x),   rv.y  >= 0.0 ? rv.y  : (1.0  +  rv.y));",
        "    rv  = vec2( rv.x < 1.0  ?  rv.x : ( rv.x - 1.0 ), rv.y  < 1.0  ? rv.y  : ( rv.y - 1.0 ));",
        "    brv = vec2(brv.x >= 0.0 ? brv.x : (1.0 + brv.x),  brv.y >= 0.0 ? brv.y : (1.0  + brv.y));",
        "    brv = vec2(brv.x < 1.0  ? brv.x : (brv.x - 1.0 ), brv.y < 1.0  ? brv.y : (brv.y - 1.0 ));",
        "    float tl = abs(texture2D(uHeightMap, tlv).r);",
        "    float l  = abs(texture2D(uHeightMap, lv ).r);",
        "    float bl = abs(texture2D(uHeightMap, blv).r);",
        "    float t  = abs(texture2D(uHeightMap, tv ).r);",
        "    float b  = abs(texture2D(uHeightMap, bv ).r);",
        "    float tr = abs(texture2D(uHeightMap, trv).r);",
        "    float r  = abs(texture2D(uHeightMap, rv ).r);",
        "    float br = abs(texture2D(uHeightMap, brv).r);",
        "    float dx = 0.0, dy = 0.0;",
        "    if (type == 0) // Sobel",
        "    {",
        "        dx = tl + l*2.0 + bl - tr - r*2.0 - br;",
        "        dy = tl + t*2.0 + tr - bl - b*2.0 - br;",
        "    }",
        "    else // Scharr",
        "    {",
        "        dx = tl*3.0 + l*10.0 + bl*3.0 - tr*3.0 - r*10.0 - br*3.0;",
        "        dy = tl*3.0 + t*10.0 + tr*3.0 - bl*3.0 - b*10.0 - br*3.0;",
        "    }",
        "    vec3 normal = normalize(vec3(dx * invertR * invertH * 255.0, dy * invertG * invertH * 255.0, dz));",
        "    gl_FragColor = vec4(normal.xyz * 0.5 + 0.5, 1.0);",
        "}",
      ].join("\n"),
      createShader = function (r, t) {
        var v = {
          attributes: { aPosition: pc.SEMANTIC_POSITION },
          vshader: r,
          fshader: t,
        };
        return new pc.Shader(e.graphicsDevice, v);
      };
    (HeightToNormal.copyShader = createShader(r, t)),
      (HeightToNormal.blurShader = createShader(r, v)),
      (HeightToNormal.normalMapShader = createShader(r, o));
  }),
  (HeightToNormal.prototype.createTexture = function (e, r) {
    return new pc.Texture(this.app.graphicsDevice, {
      width: e,
      height: r,
      format: pc.PIXELFORMAT_R8_G8_B8,
      minFilter: pc.FILTER_LINEAR,
      magFilter: pc.FILTER_LINEAR,
      addressU: pc.ADDRESS_CLAMP_TO_EDGE,
      addressV: pc.ADDRESS_CLAMP_TO_EDGE,
      mipmaps: !1,
      anisotropy: 8,
    });
  }),
  (HeightToNormal.prototype.createRenderTarget = function (e) {
    return new pc.RenderTarget(this.app.graphicsDevice, e, { depth: !1 });
  }),
  (HeightToNormal.prototype.createQuad = function () {
    var e = this.app,
      r = new pc.VertexFormat(e.graphicsDevice, [
        {
          semantic: pc.SEMANTIC_POSITION,
          components: 2,
          type: pc.ELEMENTTYPE_FLOAT32,
        },
      ]),
      t = new pc.VertexBuffer(e.graphicsDevice, r, 4);
    return (
      new Float32Array(t.lock()).set([-1, -1, 1, -1, -1, 1, 1, 1]),
      t.unlock(),
      t
    );
  }),
  (HeightToNormal.prototype.initialize = function () {
    HeightToNormal.copyShader || HeightToNormal.createShaders(this.app),
      (this.quad = this.createQuad());
  }),
  (HeightToNormal.prototype.generate = function (e) {
    var r = this.app.graphicsDevice,
      t = e.width,
      v = e.height,
      o = this.createTexture(t, v),
      i = this.createRenderTarget(o),
      a = this.createTexture(t, v),
      l = this.createRenderTarget(a);
    (i.colorBuffer.minFilter = pc.FILTER_LINEAR),
      (i.colorBuffer.mipmaps = !1),
      r.setUniform("uColorBuffer", e),
      pc.drawFullscreenQuad(r, l, this.quad, HeightToNormal.copyShader);
    for (var c = 0; c < this.blurIterations; c++) {
      r.setUniform("uColorBuffer", l.colorBuffer),
        r.setUniform("uResolution", new Float32Array([e.width, e.height]));
      r.setUniform(
        "uDirection",
        new Float32Array(c % 2 == 0 ? [0.2, 0] : [0, 0.2])
      ),
        pc.drawFullscreenQuad(r, i, this.quad, HeightToNormal.blurShader);
      var n = i;
      (i = l), (l = n);
    }
    return (
      (i.colorBuffer.minFilter = pc.FILTER_LINEAR_MIPMAP_LINEAR),
      (i.colorBuffer.mipmaps = !0),
      r.setUniform("uHeightMap", l.colorBuffer),
      r.setUniform("uResolution", new Float32Array([e.width, e.height])),
      r.setUniform("dz", 100 * this.strength),
      r.setUniform("invertR", -1),
      r.setUniform("invertG", 1),
      r.setUniform("invertH", 1),
      r.setUniform("type", 0),
      pc.drawFullscreenQuad(r, i, this.quad, HeightToNormal.normalMapShader),
      { normalMap: i.colorBuffer, heightMap: l.colorBuffer }
    );
  });
var MaskTransform = pc.createScript("maskTransform");
MaskTransform.attributes.add("baseText", { type: "entity" }),
  (MaskTransform.prototype.initialize = function () {
    (this.initPos = new pc.Vec3().copy(this.entity.getLocalPosition())),
      (this.initPosChild = new pc.Vec3().copy(
        this.entity.children[0].getLocalPosition()
      ));
  }),
  (MaskTransform.prototype.test = function (t) {
    this.entity.setLocalPosition(
      this.initPos.x,
      this.initPos.y,
      this.initPos.z
    ),
      this.entity.children[0].setLocalPosition(
        this.initPosChild.x,
        this.initPosChild.y,
        this.initPosChild.z
      ),
      this.entity
        .tween(this.entity.getLocalPosition())
        .to(
          new pc.Vec3(
            this.entity.getLocalPosition().x + 90,
            this.entity.getLocalPosition().y,
            this.entity.getLocalPosition().z
          ),
          2,
          pc.Linear
        )
        .on("update", () => {
          this.entity.children[0].setPosition(
            this.baseText.getPosition().x,
            this.baseText.getPosition().y,
            this.baseText.getPosition().z
          );
        })
        .on("complete", () => {})
        .start();
  });
var PauseButton = pc.createScript("pauseButton");
PauseButton.attributes.add("sprites", {
  type: "asset",
  assetType: "sprite",
  array: !0,
}),
  (PauseButton.prototype.initialize = function () {
    (PauseButton.instance = this),
      (this.pauseSprite = this.sprites[0].resource),
      (this.homeSprite = this.sprites[1].resource);
    let mobileCheck = function () {
      let t = !1;
      var e;
      return (
        (e = navigator.userAgent || navigator.vendor || window.opera),
        (/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/i.test(
          e
        ) ||
          /1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(
            e.substr(0, 4)
          )) &&
          (t = !0),
        t
      );
    };
    console.log(mobileCheck()),
      mobileCheck() &&
        this.entity.setLocalPosition(
          this.entity.getLocalPosition().x,
          this.entity.getLocalPosition().y - 50,
          this.entity.getLocalPosition().z
        ),
      (this.paused = !1),
      this.entity.parent.findByName("PlayButton").button.on("click", () => {
        this.unPause();
      }),
      this.entity.button.on(
        "click",
        () => {
          GameManager.instance.finished ||
            (this.paused
              ? PokiManager.instance.interstitial(() => {
                  GameManager.instance.mainMenu();
                })
              : this.pause());
        },
        this
      );
  }),
  (PauseButton.prototype.show = function (t) {
    this.entity.enabled = !0;
    let e = this.entity.tween(this.entity.getLocalPosition());
    e.to(
      new pc.Vec3(37.948, this.entity.getLocalPosition().y, 0),
      1,
      pc.ElasticOut
    )
      .on("complete", () => {})
      .on("update", () => {
        e.manager.setTs(1 / this.app.timeScale);
      })
      .start();
  }),
  (PauseButton.prototype.hide = function (t) {
    let e = this.entity.tween(this.entity.getLocalPosition());
    e.to(
      new pc.Vec3(-23.872, this.entity.getLocalPosition().y, 0),
      1,
      pc.ElasticIn
    )
      .on("complete", () => {
        this.entity.enabled = !1;
      })
      .on("update", () => {
        e.manager.setTs(1 / this.app.timeScale);
      })
      .start();
  }),
  (PauseButton.prototype.pause = function (t) {
    PokiSDK.gameplayStop(),
      (this.paused = !0),
      (this.previousTimeScale = this.app.timeScale),
      (this.app.timeScale = 0),
      (GameManager.instance.playerController.activated = !1),
      (GameManager.instance.playerController.paused = !0),
      (this.entity.element._image.sprite = this.homeSprite),
      (this.entity.parent.findByName("PlayButton").enabled = !0);
  }),
  (PauseButton.prototype.unPause = function (t) {
    PokiSDK.gameplayStart(),
      (this.paused = !1),
      (this.app.timeScale = 1),
      (this.entity.parent.findByName("PlayButton").enabled = !1),
      (this.entity.element._image.sprite = this.pauseSprite),
      (GameManager.instance.playerController.activated = !0),
      (GameManager.instance.playerController.unPause = !0);
  }),
  (PauseButton.prototype.update = function (t) {});
var DestroyEntity = pc.createScript("destroyEntity");
(DestroyEntity.prototype.initialize = function () {
  this.entity.destroy();
}),
  (DestroyEntity.prototype.update = function (t) {});
var ProgressBar = pc.createScript("progressBar");
ProgressBar.attributes.add("curLevel", { type: "entity" }),
  ProgressBar.attributes.add("nextLevel", { type: "entity" }),
  (ProgressBar.prototype.initialize = function () {
    (this.percentage = 0),
      (this.curLevel.element.text = GameManager.instance.levelId),
      (this.nextLevel.element.text = GameManager.instance.levelId + 1);
  }),
  (ProgressBar.prototype.update = function (e) {
    this.entity.element.width = Math.min(
      Math.max(
        (9.288 *
          GameManager.instance.playerController.sausageSegments[0].getPosition()
            .x) /
          GameManager.instance.level.findByName("Pattern_Finish").getPosition()
            .x,
        0
      ),
      9.288
    );
  });
var Tester = pc.createScript("tester");
(Tester.prototype.initialize = function () {
  setTimeout(() => {
    (this.entity.script.hingeConstraint.enabled = !0),
      this.entity.script.hingeConstraint.createConstraint();
  }, 100),
    this.on("enable", function () {}),
    this.on("disable", function () {
      this.entity.script.hingeConstraint.enabled = !1;
    });
}),
  (Tester.prototype.update = function (t) {});
var CompoundConstraint = pc.createScript("compoundConstraint");
(CompoundConstraint.prototype.initialize = function () {
  setTimeout(() => {
    this.entity.parent.script.hingeConstraint.createConstraint(),
      (this.entity.parent.script.hingeConstraint.enabled = !0);
  }, 1),
    this.on("enable", function () {
      this.entity.parent.script.hingeConstraint.createConstraint(),
        (this.entity.parent.script.hingeConstraint.enabled = !0);
    });
}),
  (CompoundConstraint.prototype.update = function (t) {});
var SpringTimer = pc.createScript("springTimer");
(SpringTimer.prototype.postInitialize = function () {
  this.done = !1;
}),
  (SpringTimer.prototype.postUpdate = function (t) {
    this.done ||
      ((this.done = !0),
      (this.entity.rigidbody.enabled = !0),
      this.entity.script.hingeConstraint &&
        this.entity.script.hingeConstraint.createConstraint(),
      this.entity.script.pointToPointConstraint &&
        this.entity.script.pointToPointConstraint.createConstraint());
  });
var SpringTest = pc.createScript("springTest");
(SpringTest.prototype.postInitialize = function () {
  this.done = !1;
}),
  (SpringTest.prototype.postUpdate = function (t) {
    this.done ||
      ((this.done = !0),
      (this.entity.script.springJoint.enabled = !0),
      this.entity.script.springJoint.createConstraint());
  });
var SpawnTemplate = pc.createScript("spawnTemplate");
SpawnTemplate.attributes.add("template", {
  type: "asset",
  assetType: "template",
}),
  (SpawnTemplate.prototype.initialize = function () {
    setInterval(() => {
      let t = this.template.resource.instantiate();
      doStuffToEntity(this.template.resource._templateRoot, t),
        t.setPosition(0.976, -2.65, 0),
        this.app.root.addChild(t),
        setTimeout(() => {
          t.destroy();
        }, 250);
    }, 1e3);
  }),
  (SpawnTemplate.prototype.postInitialize = function (t) {}),
  (SpawnTemplate.prototype.postUpdate = function (t) {});
var Contactpoint = pc.createScript("contactpoint");
Contactpoint.attributes.add("contactProcessingThreshold", {
  type: "number",
  default: 1e-6,
  title: "Contact Processing Threshold",
  description:
    'This should be below the half extent of the collision volume. E.g For an object of dimensions 1 meter, try 0.2 the constraint solver can discard solving contacts, if the distance is above this threshold. 0 by default. \n Note that using contacts with positive distance can improve stability. It increases, however, the chance of colliding with degerate contacts, such as "interior" triangle edges',
}),
  (Contactpoint.prototype.initialize = function () {
    this.entity.rigidbody.body.setContactProcessingThreshold(
      this.contactProcessingThreshold
    ),
      this.on("attr:contactProcessingThreshold", function (t, o) {
        this.entity.rigidbody.body.setContactProcessingThreshold(t);
      });
  }),
  (Contactpoint.prototype.update = function (t) {});
var Blink = pc.createScript("blink");
(Blink.prototype.initialize = function () {
  setInterval(() => {
    (this.entity.enabled = !1), (this.entity.enabled = !0);
  }, 500);
}),
  (Blink.prototype.update = function (t) {});
var Rocket = pc.createScript("rocket");
Rocket.attributes.add("particles", { title: "Particles", type: "entity" }),
  Rocket.attributes.add("power", {
    title: "Power",
    type: "number",
    default: 2,
  }),
  Rocket.attributes.add("maxPower", {
    title: "MaxPower",
    type: "number",
    default: 40,
  }),
  Rocket.attributes.add("otherCollider", { type: "entity", default: 40 }),
  (Rocket.prototype.initialize = function () {
    (this.rb = this.entity.rigidbody),
      (this.activated = !1),
      this.entity.collision.on(
        "collisionstart",
        (t) => {
          this.activate(t);
        },
        this
      ),
      this.otherCollider &&
        this.otherCollider.collision.on(
          "collisionstart",
          (t) => {
            this.activate(t);
          },
          this
        );
  }),
  (Rocket.prototype.activate = function (t) {
    !this.activated &&
      t.other.tags._list.includes("Segment") &&
      ((this.activated = !0), this.particles.particlesystem.play());
  }),
  (Rocket.prototype.update = function (t) {
    this.activated &&
      ((this.rb.linearVelocity = this.entity.up.mulScalar(this.power)),
      this.power < this.maxPower && (this.power *= 1.05));
  });
var DelayActivation = pc.createScript("delayActivation");
(DelayActivation.prototype.initialize = function () {}),
  (DelayActivation.prototype.update = function (t) {});
var SpringRenderer = pc.createScript("springRenderer");
SpringRenderer.attributes.add("springEntity", { type: "entity" }),
  (SpringRenderer.prototype.initialize = function () {
    null != this.springEntity
      ? ((this.springJoint = this.springEntity.script.springJoint),
        null == this.springJoint &&
          (this.springJoint = this.springEntity.script.spring))
      : ((this.springJoint = this.entity.script.springJoint),
        null == this.springJoint &&
          (this.springJoint = this.entity.script.spring)),
      (this.spriteRenderer = this.entity.children[0].sprite),
      null != this.springJoint &&
        ((this.scaleX = this.entity.getLocalScale().x),
        (this.scaleY = this.entity.getLocalScale().y),
        (this.width = this.entity.children[0].sprite.width),
        (this.connectedTranform = this.springJoint.entityB));
  }),
  (SpringRenderer.prototype.updateSprite = function () {
    if (null == this.connectedTranform || null == this.spriteRenderer) return;
    new pc.Vec3(
      0,
      (this.entity.getLocalPosition().y +
        this.connectedTranform.getLocalPosition().y) /
        2,
      0
    );
    (this.size = new pc.Vec2(
      this.spriteRenderer.width,
      this.spriteRenderer.height
    )),
      (this.size.x = this.entity
        .getPosition()
        .sub(this.connectedTranform.getPosition())),
      this.entity.setLocalScale(
        this.scaleX,
        Math.abs(
          this.entity
            .getPosition()
            .sub(this.connectedTranform.getPosition())
            .length()
        ) /
          this.width /
          this.entity.parent.getScale().y,
        1
      ),
      (this.spriteRenderer.height = this.size.y);
    let t = this.entity.getPosition().sub(this.connectedTranform.getPosition());
    this.entity.setRotation(
      new pc.Quat().setFromEulerAngles(
        new pc.Vec3(0, 0, Math.atan2(t.y, t.x) * (180 / Math.PI) + 90)
      )
    );
  }),
  (SpringRenderer.prototype.update = function (t) {
    this.updateSprite();
  });
var BoxeGlove = pc.createScript("boxeGlove");
BoxeGlove.attributes.add("force", { type: "number", default: 30 }),
  BoxeGlove.attributes.add("collide", { type: "entity" }),
  (BoxeGlove.prototype.initialize = function () {
    (this.waitingLaunch = !1),
      (this.rb = this.entity.rigidbody),
      this.collide.collision.on(
        "triggerenter",
        (t) => {
          !this.waitingLaunch &&
            t.tags._list.includes("Segment") &&
            this.launch();
        },
        this
      );
  }),
  (BoxeGlove.prototype.launch = function () {
    this.entity.getRotation().getEulerAngles();
    let t = new pc.Vec3().copy(this.entity.up).mulScalar(2 * this.force),
      e = new pc.Vec3().copy(this.entity.up).mulScalar(this.force);
    (e.y = -e.y),
      GameManager.instance.playerController.dash(e),
      GameManager.instance.playerController.unstick(),
      this.rb.applyImpulse(t.x, t.y, t.z),
      (this.waitingLaunch = !0),
      setTimeout(() => {
        this.waitingLaunch = !1;
      }, 500);
  }),
  (BoxeGlove.prototype.update = function (t) {});
var SetFriction = pc.createScript("setFriction");
(SetFriction.prototype.initialize = function () {
  console.log((this.entity.rigidbody.friction = 0));
}),
  (SetFriction.prototype.update = function (t) {});
var SpringJointPivot = pc.createScript("springJointPivot");
(SpringJointPivot.prototype.initialize = function () {
  (this.spring = this.entity.script.springJoint),
    console.log(),
    (this.connectedTo = this.spring.entityB),
    (this.spring.pivotA = this.entity
      .getPosition()
      .sub(this.connectedTo.getPosition())
      .mulScalar(-1)),
    this.spring.createConstraint(),
    console.log(
      "Offset",
      this.entity
        .getPosition()
        .sub(this.connectedTo.getPosition())
        .mulScalar(-1)
    );
}),
  (SpringJointPivot.prototype.update = function (t) {});
var PrintRotation = pc.createScript("printRotation");
(PrintRotation.prototype.initialize = function () {}),
  (PrintRotation.prototype.update = function (t) {});
var Flash = pc.createScript("flash");
(Flash.prototype.initialize = function () {}),
  (Flash.prototype.update = function (t) {});
pc.extend(
  pc,
  (function () {
    var TweenManager = function (t) {
      (this._app = t), (this._tweens = []), (this._add = []);
    };
    TweenManager.prototype = {
      add: function (t) {
        return this._add.push(t), t;
      },
      setTs: function (t) {
        return (this.ts = t), this.ts;
      },
      update: function (t) {
        for (var i = 0, e = this._tweens.length; i < e; )
          this._tweens[i].update(t, this.ts)
            ? i++
            : (this._tweens.splice(i, 1), e--);
        if (this._add.length) {
          for (let t = 0; t < this._add.length; t++)
            this._tweens.indexOf(this._add[t]) > -1 ||
              this._tweens.push(this._add[t]);
          this._add.length = 0;
        }
      },
    };
    var Tween = function (t, i, e) {
        pc.events.attach(this),
          (this.manager = i),
          e && (this.entity = null),
          (this.time = 0),
          (this.complete = !1),
          (this.playing = !1),
          (this.stopped = !0),
          (this.pending = !1),
          (this.target = t),
          (this.duration = 0),
          (this._currentDelay = 0),
          (this.timeScale = 1),
          (this.test = 10),
          (this._reverse = !1),
          (this._delay = 0),
          (this._yoyo = !1),
          (this._count = 0),
          (this._numRepeats = 0),
          (this._repeatDelay = 0),
          (this._from = !1),
          (this._slerp = !1),
          (this._fromQuat = new pc.Quat()),
          (this._toQuat = new pc.Quat()),
          (this._quat = new pc.Quat()),
          (this.easing = pc.Linear),
          (this._sv = {}),
          (this._ev = {});
      },
      _parseProperties = function (t) {
        var i;
        return (
          t instanceof pc.Vec2
            ? (i = { x: t.x, y: t.y })
            : t instanceof pc.Vec3
            ? (i = { x: t.x, y: t.y, z: t.z })
            : t instanceof pc.Vec4 || t instanceof pc.Quat
            ? (i = { x: t.x, y: t.y, z: t.z, w: t.w })
            : t instanceof pc.Color
            ? ((i = { r: t.r, g: t.g, b: t.b }), void 0 !== t.a && (i.a = t.a))
            : (i = t),
          i
        );
      };
    Tween.prototype = {
      to: function (t, i, e, s, n, r) {
        return (
          (this._properties = _parseProperties(t)),
          (this.duration = i),
          e && (this.easing = e),
          s && this.delay(s),
          n && this.repeat(n),
          r && this.yoyo(r),
          this
        );
      },
      from: function (t, i, e, s, n, r) {
        return (
          (this._properties = _parseProperties(t)),
          (this.duration = i),
          e && (this.easing = e),
          s && this.delay(s),
          n && this.repeat(n),
          r && this.yoyo(r),
          (this._from = !0),
          this
        );
      },
      rotate: function (t, i, e, s, n, r) {
        return (
          (this._properties = _parseProperties(t)),
          (this.duration = i),
          e && (this.easing = e),
          s && this.delay(s),
          n && this.repeat(n),
          r && this.yoyo(r),
          (this._slerp = !0),
          this
        );
      },
      start: function () {
        var t, i, e, s;
        if (
          ((this.playing = !0),
          (this.complete = !1),
          (this.stopped = !1),
          (this._count = 0),
          (this.pending = this._delay > 0),
          this._reverse && !this.pending
            ? (this.time = this.duration)
            : (this.time = 0),
          this._from)
        ) {
          for (t in this._properties)
            this._properties.hasOwnProperty(t) &&
              ((this._sv[t] = this._properties[t]),
              (this._ev[t] = this.target[t]));
          this._slerp &&
            (this._toQuat.setFromEulerAngles(
              this.target.x,
              this.target.y,
              this.target.z
            ),
            (i =
              void 0 !== this._properties.x
                ? this._properties.x
                : this.target.x),
            (e =
              void 0 !== this._properties.y
                ? this._properties.y
                : this.target.y),
            (s =
              void 0 !== this._properties.z
                ? this._properties.z
                : this.target.z),
            this._fromQuat.setFromEulerAngles(i, e, s));
        } else {
          for (t in this._properties)
            this._properties.hasOwnProperty(t) &&
              ((this._sv[t] = this.target[t]),
              (this._ev[t] = this._properties[t]));
          this._slerp &&
            ((i =
              void 0 !== this._properties.x
                ? this._properties.x
                : this.target.x),
            (e =
              void 0 !== this._properties.y
                ? this._properties.y
                : this.target.y),
            (s =
              void 0 !== this._properties.z
                ? this._properties.z
                : this.target.z),
            void 0 !== this._properties.w
              ? (this._fromQuat.copy(this.target),
                this._toQuat.set(i, e, s, this._properties.w))
              : (this._fromQuat.setFromEulerAngles(
                  this.target.x,
                  this.target.y,
                  this.target.z
                ),
                this._toQuat.setFromEulerAngles(i, e, s)));
        }
        return (this._currentDelay = this._delay), this.manager.add(this), this;
      },
      pause: function () {
        this.playing = !1;
      },
      resume: function () {
        this.playing = !0;
      },
      stop: function () {
        (this.playing = !1), (this.stopped = !0);
      },
      delay: function (t) {
        return (this._delay = t), (this.pending = !0), this;
      },
      repeat: function (t, i) {
        return (
          (this._count = 0),
          (this._numRepeats = t),
          (this._repeatDelay = i || 0),
          this
        );
      },
      loop: function (t) {
        return (
          t
            ? ((this._count = 0), (this._numRepeats = 1 / 0))
            : (this._numRepeats = 0),
          this
        );
      },
      yoyo: function (t) {
        return (this._yoyo = t), this;
      },
      reverse: function () {
        return (this._reverse = !this._reverse), this;
      },
      chain: function () {
        for (var t = arguments.length; t--; )
          t > 0
            ? (arguments[t - 1]._chained = arguments[t])
            : (this._chained = arguments[t]);
        return this;
      },
      update: function (t, i) {
        let e = this.timeScale;
        if ((i && (e = i), this.stopped)) return !1;
        if (!this.playing) return !0;
        if (
          (!this._reverse || this.pending
            ? (this.time += t * e)
            : (this.time -= t * e),
          this.pending)
        ) {
          if (!(this.time > this._currentDelay)) return !0;
          this._reverse
            ? (this.time = this.duration - (this.time - this._currentDelay))
            : (this.time -= this._currentDelay),
            (this.pending = !1);
        }
        var s = 0;
        ((!this._reverse && this.time > this.duration) ||
          (this._reverse && this.time < 0)) &&
          (this._count++,
          (this.complete = !0),
          (this.playing = !1),
          this._reverse
            ? ((s = this.duration - this.time), (this.time = 0))
            : ((s = this.time - this.duration), (this.time = this.duration)));
        var n,
          r,
          h = 0 === this.duration ? 1 : this.time / this.duration,
          a = this.easing(h);
        for (var o in this._properties)
          this._properties.hasOwnProperty(o) &&
            ((n = this._sv[o]),
            (r = this._ev[o]),
            (this.target[o] = n + (r - n) * a));
        if (
          (this._slerp && this._quat.slerp(this._fromQuat, this._toQuat, a),
          this.entity &&
            (this.entity._dirtifyLocal(),
            this.element &&
              this.entity.element &&
              (this.entity.element[this.element] = this.target),
            this._slerp && this.entity.setLocalRotation(this._quat)),
          this.fire("update", t),
          this.complete)
        ) {
          var u = this._repeat(s);
          return (
            u
              ? this.fire("loop")
              : (this.fire("complete", s),
                this.entity && this.entity.off("destroy", this.stop, this),
                this._chained && this._chained.start()),
            u
          );
        }
        return !0;
      },
      _repeat: function (t) {
        if (this._count < this._numRepeats) {
          if (
            (this._reverse ? (this.time = this.duration - t) : (this.time = t),
            (this.complete = !1),
            (this.playing = !0),
            (this._currentDelay = this._repeatDelay),
            (this.pending = !0),
            this._yoyo)
          ) {
            for (var i in this._properties) {
              var e = this._sv[i];
              (this._sv[i] = this._ev[i]), (this._ev[i] = e);
            }
            this._slerp &&
              (this._quat.copy(this._fromQuat),
              this._fromQuat.copy(this._toQuat),
              this._toQuat.copy(this._quat));
          }
          return !0;
        }
        return !1;
      },
    };
    var BounceOut = function (t) {
        return t < 1 / 2.75
          ? 7.5625 * t * t
          : t < 2 / 2.75
          ? 7.5625 * (t -= 1.5 / 2.75) * t + 0.75
          : t < 2.5 / 2.75
          ? 7.5625 * (t -= 2.25 / 2.75) * t + 0.9375
          : 7.5625 * (t -= 2.625 / 2.75) * t + 0.984375;
      },
      BounceIn = function (t) {
        return 1 - BounceOut(1 - t);
      };
    return {
      TweenManager: TweenManager,
      Tween: Tween,
      Linear: function (t) {
        return t;
      },
      QuadraticIn: function (t) {
        return t * t;
      },
      QuadraticOut: function (t) {
        return t * (2 - t);
      },
      QuadraticInOut: function (t) {
        return (t *= 2) < 1 ? 0.5 * t * t : -0.5 * (--t * (t - 2) - 1);
      },
      CubicIn: function (t) {
        return t * t * t;
      },
      CubicOut: function (t) {
        return --t * t * t + 1;
      },
      CubicInOut: function (t) {
        return (t *= 2) < 1 ? 0.5 * t * t * t : 0.5 * ((t -= 2) * t * t + 2);
      },
      QuarticIn: function (t) {
        return t * t * t * t;
      },
      QuarticOut: function (t) {
        return 1 - --t * t * t * t;
      },
      QuarticInOut: function (t) {
        return (t *= 2) < 1
          ? 0.5 * t * t * t * t
          : -0.5 * ((t -= 2) * t * t * t - 2);
      },
      QuinticIn: function (t) {
        return t * t * t * t * t;
      },
      QuinticOut: function (t) {
        return --t * t * t * t * t + 1;
      },
      QuinticInOut: function (t) {
        return (t *= 2) < 1
          ? 0.5 * t * t * t * t * t
          : 0.5 * ((t -= 2) * t * t * t * t + 2);
      },
      SineIn: function (t) {
        return 0 === t ? 0 : 1 === t ? 1 : 1 - Math.cos((t * Math.PI) / 2);
      },
      SineOut: function (t) {
        return 0 === t ? 0 : 1 === t ? 1 : Math.sin((t * Math.PI) / 2);
      },
      SineInOut: function (t) {
        return 0 === t ? 0 : 1 === t ? 1 : 0.5 * (1 - Math.cos(Math.PI * t));
      },
      ExponentialIn: function (t) {
        return 0 === t ? 0 : Math.pow(1024, t - 1);
      },
      ExponentialOut: function (t) {
        return 1 === t ? 1 : 1 - Math.pow(2, -10 * t);
      },
      ExponentialInOut: function (t) {
        return 0 === t
          ? 0
          : 1 === t
          ? 1
          : (t *= 2) < 1
          ? 0.5 * Math.pow(1024, t - 1)
          : 0.5 * (2 - Math.pow(2, -10 * (t - 1)));
      },
      CircularIn: function (t) {
        return 1 - Math.sqrt(1 - t * t);
      },
      CircularOut: function (t) {
        return Math.sqrt(1 - --t * t);
      },
      CircularInOut: function (t) {
        return (t *= 2) < 1
          ? -0.5 * (Math.sqrt(1 - t * t) - 1)
          : 0.5 * (Math.sqrt(1 - (t -= 2) * t) + 1);
      },
      BackIn: function (t) {
        var i = 1.70158;
        return t * t * ((i + 1) * t - i);
      },
      BackOut: function (t) {
        var i = 1.70158;
        return --t * t * ((i + 1) * t + i) + 1;
      },
      BackInOut: function (t) {
        var i = 2.5949095;
        return (t *= 2) < 1
          ? t * t * ((i + 1) * t - i) * 0.5
          : 0.5 * ((t -= 2) * t * ((i + 1) * t + i) + 2);
      },
      BounceIn: BounceIn,
      BounceOut: BounceOut,
      BounceInOut: function (t) {
        return t < 0.5
          ? 0.5 * BounceIn(2 * t)
          : 0.5 * BounceOut(2 * t - 1) + 0.5;
      },
      ElasticIn: function (t) {
        var i,
          e = 0.1;
        return 0 === t
          ? 0
          : 1 === t
          ? 1
          : (!e || e < 1
              ? ((e = 1), (i = 0.1))
              : (i = (0.4 * Math.asin(1 / e)) / (2 * Math.PI)),
            -e *
              Math.pow(2, 10 * (t -= 1)) *
              Math.sin(((t - i) * (2 * Math.PI)) / 0.4));
      },
      ElasticOut: function (t) {
        var i,
          e = 0.1;
        return 0 === t
          ? 0
          : 1 === t
          ? 1
          : (!e || e < 1
              ? ((e = 1), (i = 0.1))
              : (i = (0.4 * Math.asin(1 / e)) / (2 * Math.PI)),
            e *
              Math.pow(2, -10 * t) *
              Math.sin(((t - i) * (2 * Math.PI)) / 0.4) +
              1);
      },
      ElasticInOut: function (t) {
        var i,
          e = 0.1,
          s = 0.4;
        return 0 === t
          ? 0
          : 1 === t
          ? 1
          : (!e || e < 1
              ? ((e = 1), (i = 0.1))
              : (i = (s * Math.asin(1 / e)) / (2 * Math.PI)),
            (t *= 2) < 1
              ? e *
                Math.pow(2, 10 * (t -= 1)) *
                Math.sin(((t - i) * (2 * Math.PI)) / s) *
                -0.5
              : e *
                  Math.pow(2, -10 * (t -= 1)) *
                  Math.sin(((t - i) * (2 * Math.PI)) / s) *
                  0.5 +
                1);
      },
    };
  })()
),
  (function () {
    (pc.Application.prototype.addTweenManager = function () {
      (this._tweenManager = new pc.TweenManager(this)),
        this.on("update", function (t) {
          this._tweenManager.update(t);
        });
    }),
      (pc.Application.prototype.tween = function (t) {
        return new pc.Tween(t, this._tweenManager);
      }),
      (pc.Entity.prototype.tween = function (t, i) {
        var e = this._app.tween(t);
        return (
          (e.entity = this),
          this.once("destroy", e.stop, e),
          i && i.element && (e.element = i.element),
          e
        );
      });
    var t = pc.Application.getApplication();
    t && t.addTweenManager();
  })();
"undefined" != typeof document &&
  /*! FPSMeter 0.3.1 - 9th May 2013 | https://github.com/Darsain/fpsmeter */
  ((function (t, e) {
    function s(t, e) {
      for (var n in e)
        try {
          t.style[n] = e[n];
        } catch (t) {}
      return t;
    }
    function H(t) {
      return null == t
        ? String(t)
        : "object" == typeof t || "function" == typeof t
        ? Object.prototype.toString
            .call(t)
            .match(/\s([a-z]+)/i)[1]
            .toLowerCase() || "object"
        : typeof t;
    }
    function R(t, e) {
      if ("array" !== H(e)) return -1;
      if (e.indexOf) return e.indexOf(t);
      for (var n = 0, o = e.length; n < o; n++) if (e[n] === t) return n;
      return -1;
    }
    function I() {
      var t,
        e = arguments;
      for (t in e[1])
        if (e[1].hasOwnProperty(t))
          switch (H(e[1][t])) {
            case "object":
              e[0][t] = I({}, e[0][t], e[1][t]);
              break;
            case "array":
              e[0][t] = e[1][t].slice(0);
              break;
            default:
              e[0][t] = e[1][t];
          }
      return 2 < e.length
        ? I.apply(null, [e[0]].concat(Array.prototype.slice.call(e, 2)))
        : e[0];
    }
    function N(t) {
      return 1 === (t = Math.round(255 * t).toString(16)).length ? "0" + t : t;
    }
    function S(t, e, n, o) {
      t.addEventListener
        ? t[o ? "removeEventListener" : "addEventListener"](e, n, !1)
        : t.attachEvent && t[o ? "detachEvent" : "attachEvent"]("on" + e, n);
    }
    function D(t, e) {
      function g(t, e, n, o) {
        return h[0 | t][Math.round(Math.min(((e - n) / (o - n)) * M, M))];
      }
      function r() {
        F.legend.fps !== q &&
          ((F.legend.fps = q), (F.legend[c] = q ? "FPS" : "ms")),
          (b = q ? v.fps : v.duration),
          (F.count[c] = 999 < b ? "999+" : b.toFixed(99 < b ? 0 : O.decimals));
      }
      function m() {
        for (
          l = n(),
            P < l - O.threshold &&
              ((v.fps -= v.fps / Math.max(1, (60 * O.smoothing) / O.interval)),
              (v.duration = 1e3 / v.fps)),
            w = O.history;
          w--;

        )
          (T[w] = 0 === w ? v.fps : T[w - 1]),
            (j[w] = 0 === w ? v.duration : j[w - 1]);
        if ((r(), O.heat)) {
          if (z.length)
            for (w = z.length; w--; )
              z[w].el.style[o[z[w].name].heatOn] = q
                ? g(o[z[w].name].heatmap, v.fps, 0, O.maxFps)
                : g(o[z[w].name].heatmap, v.duration, O.threshold, 0);
          if (F.graph && o.column.heatOn)
            for (w = C.length; w--; )
              C[w].style[o.column.heatOn] = q
                ? g(o.column.heatmap, T[w], 0, O.maxFps)
                : g(o.column.heatmap, j[w], O.threshold, 0);
        }
        if (F.graph)
          for (y = 0; y < O.history; y++)
            C[y].style.height =
              (q
                ? T[y]
                  ? Math.round((x / O.maxFps) * Math.min(T[y], O.maxFps))
                  : 0
                : j[y]
                ? Math.round((x / O.threshold) * Math.min(j[y], O.threshold))
                : 0) + "px";
      }
      function k() {
        20 > O.interval
          ? ((p = i(k)), m())
          : ((p = setTimeout(k, O.interval)), (f = i(m)));
      }
      function G(t) {
        (t = t || window.event).preventDefault
          ? (t.preventDefault(), t.stopPropagation())
          : ((t.returnValue = !1), (t.cancelBubble = !0)),
          v.toggle();
      }
      function U() {
        O.toggleOn && S(F.container, O.toggleOn, G, 1),
          t.removeChild(F.container);
      }
      function V() {
        if (
          (F.container && U(),
          (o = D.theme[O.theme]),
          !(h = o.compiledHeatmaps || []).length && o.heatmaps.length)
        ) {
          for (y = 0; y < o.heatmaps.length; y++)
            for (h[y] = [], w = 0; w <= M; w++) {
              var e,
                n = h[y],
                a = w;
              e = (0.33 / M) * w;
              var i = o.heatmaps[y].saturation,
                l = o.heatmaps[y].lightness,
                p = void 0,
                c = void 0,
                u = void 0,
                d = (u = void 0),
                f = (p = c = void 0);
              f = void 0;
              0 === (u = 0.5 >= l ? l * (1 + i) : l + i - l * i)
                ? (e = "#000")
                : ((c = (u - (d = 2 * l - u)) / u),
                  (f = (e *= 6) - (p = Math.floor(e))),
                  (f *= u * c),
                  0 === p || 6 === p
                    ? ((p = u), (c = d + f), (u = d))
                    : 1 === p
                    ? ((p = u - f), (c = u), (u = d))
                    : 2 === p
                    ? ((p = d), (c = u), (u = d + f))
                    : 3 === p
                    ? ((p = d), (c = u - f))
                    : 4 === p
                    ? ((p = d + f), (c = d))
                    : ((p = u), (c = d), (u -= f)),
                  (e = "#" + N(p) + N(c) + N(u))),
                (n[a] = e);
            }
          o.compiledHeatmaps = h;
        }
        for (var b in ((F.container = s(
          document.createElement("div"),
          o.container
        )),
        (F.count = F.container.appendChild(
          s(document.createElement("div"), o.count)
        )),
        (F.legend = F.container.appendChild(
          s(document.createElement("div"), o.legend)
        )),
        (F.graph = O.graph
          ? F.container.appendChild(s(document.createElement("div"), o.graph))
          : 0),
        (z.length = 0),
        F))
          F[b] && o[b].heatOn && z.push({ name: b, el: F[b] });
        if (((C.length = 0), F.graph))
          for (
            F.graph.style.width =
              O.history * o.column.width +
              (O.history - 1) * o.column.spacing +
              "px",
              w = 0;
            w < O.history;
            w++
          )
            (C[w] = F.graph.appendChild(
              s(document.createElement("div"), o.column)
            )),
              (C[w].style.position = "absolute"),
              (C[w].style.bottom = 0),
              (C[w].style.right =
                w * o.column.width + w * o.column.spacing + "px"),
              (C[w].style.width = o.column.width + "px"),
              (C[w].style.height = "0px");
        s(F.container, O),
          r(),
          t.appendChild(F.container),
          F.graph && (x = F.graph.clientHeight),
          O.toggleOn &&
            ("click" === O.toggleOn && (F.container.style.cursor = "pointer"),
            S(F.container, O.toggleOn, G));
      }
      "object" === H(t) &&
        undefined === t.nodeType &&
        ((e = t), (t = document.body)),
        t || (t = document.body);
      var o,
        h,
        l,
        p,
        f,
        x,
        b,
        w,
        y,
        v = this,
        O = I({}, D.defaults, e || {}),
        F = {},
        C = [],
        M = 100,
        z = [],
        E = O.threshold,
        A = 0,
        P = n() - E,
        T = [],
        j = [],
        q = "fps" === O.show;
      (v.options = O),
        (v.fps = 0),
        (v.duration = 0),
        (v.isPaused = 0),
        (v.tickStart = function () {
          A = n();
        }),
        (v.tick = function () {
          (l = n()),
            (E += (l - P - E) / O.smoothing),
            (v.fps = 1e3 / E),
            (v.duration = A < P ? E : l - A),
            (P = l);
        }),
        (v.pause = function () {
          return (
            p && ((v.isPaused = 1), clearTimeout(p), a(p), a(f), (p = f = 0)), v
          );
        }),
        (v.resume = function () {
          return p || ((v.isPaused = 0), k()), v;
        }),
        (v.set = function (t, e) {
          return (
            (O[t] = e),
            (q = "fps" === O.show),
            -1 !== R(t, u) && V(),
            -1 !== R(t, d) && s(F.container, O),
            v
          );
        }),
        (v.showDuration = function () {
          return v.set("show", "ms"), v;
        }),
        (v.showFps = function () {
          return v.set("show", "fps"), v;
        }),
        (v.toggle = function () {
          return v.set("show", q ? "ms" : "fps"), v;
        }),
        (v.hide = function () {
          return v.pause(), (F.container.style.display = "none"), v;
        }),
        (v.show = function () {
          return v.resume(), (F.container.style.display = "block"), v;
        }),
        (v.destroy = function () {
          v.pause(), U(), (v.tick = v.tickStart = function () {});
        }),
        V(),
        k();
    }
    var n,
      o = t.performance;
    n =
      o && (o.now || o.webkitNow)
        ? o[o.now ? "now" : "webkitNow"].bind(o)
        : function () {
            return +new Date();
          };
    for (
      var a = t.cancelAnimationFrame || t.cancelRequestAnimationFrame,
        i = t.requestAnimationFrame,
        h = 0,
        l = 0,
        p = (o = ["moz", "webkit", "o"]).length;
      l < p && !a;
      ++l
    )
      i =
        (a =
          t[o[l] + "CancelAnimationFrame"] ||
          t[o[l] + "CancelRequestAnimationFrame"]) &&
        t[o[l] + "RequestAnimationFrame"];
    a ||
      ((i = function (e) {
        var o = n(),
          a = Math.max(0, 16 - (o - h));
        return (
          (h = o + a),
          t.setTimeout(function () {
            e(o + a);
          }, a)
        );
      }),
      (a = function (t) {
        clearTimeout(t);
      }));
    var c =
      "string" === H(document.createElement("div").textContent)
        ? "textContent"
        : "innerText";
    (D.extend = I),
      (window.FPSMeter = D),
      (D.defaults = {
        interval: 100,
        smoothing: 10,
        show: "fps",
        toggleOn: "click",
        decimals: 1,
        maxFps: 60,
        threshold: 100,
        position: "absolute",
        zIndex: 10,
        left: "5px",
        top: "5px",
        right: "auto",
        bottom: "auto",
        margin: "0 0 0 0",
        theme: "dark",
        heat: 0,
        graph: 0,
        history: 20,
      });
    var u = ["toggleOn", "theme", "heat", "graph", "history"],
      d = "position zIndex left top right bottom margin".split(" ");
  })(window),
  (function (t, e) {
    e.theme = {};
    var n = (e.theme.base = {
      heatmaps: [],
      container: {
        heatOn: null,
        heatmap: null,
        padding: "5px",
        minWidth: "95px",
        height: "30px",
        lineHeight: "30px",
        textAlign: "right",
        textShadow: "none",
      },
      count: {
        heatOn: null,
        heatmap: null,
        position: "absolute",
        top: 0,
        right: 0,
        padding: "5px 10px",
        height: "30px",
        fontSize: "24px",
        fontFamily: "Consolas, Andale Mono, monospace",
        zIndex: 2,
      },
      legend: {
        heatOn: null,
        heatmap: null,
        position: "absolute",
        top: 0,
        left: 0,
        padding: "5px 10px",
        height: "30px",
        fontSize: "12px",
        lineHeight: "32px",
        fontFamily: "sans-serif",
        textAlign: "left",
        zIndex: 2,
      },
      graph: {
        heatOn: null,
        heatmap: null,
        position: "relative",
        boxSizing: "padding-box",
        MozBoxSizing: "padding-box",
        height: "100%",
        zIndex: 1,
      },
      column: { width: 4, spacing: 1, heatOn: null, heatmap: null },
    });
    (e.theme.dark = e.extend({}, n, {
      heatmaps: [{ saturation: 0.8, lightness: 0.8 }],
      container: {
        background: "#222",
        color: "#fff",
        border: "1px solid #1a1a1a",
        textShadow: "1px 1px 0 #222",
      },
      count: { heatOn: "color" },
      column: { background: "#3f3f3f" },
    })),
      (e.theme.light = e.extend({}, n, {
        heatmaps: [{ saturation: 0.5, lightness: 0.5 }],
        container: {
          color: "#666",
          background: "#fff",
          textShadow:
            "1px 1px 0 rgba(255,255,255,.5), -1px -1px 0 rgba(255,255,255,.5)",
          boxShadow: "0 0 0 1px rgba(0,0,0,.1)",
        },
        count: { heatOn: "color" },
        column: { background: "#eaeaea" },
      })),
      (e.theme.colorful = e.extend({}, n, {
        heatmaps: [{ saturation: 0.5, lightness: 0.6 }],
        container: {
          heatOn: "backgroundColor",
          background: "#888",
          color: "#fff",
          textShadow: "1px 1px 0 rgba(0,0,0,.2)",
          boxShadow: "0 0 0 1px rgba(0,0,0,.1)",
        },
        column: { background: "#777", backgroundColor: "rgba(0,0,0,.2)" },
      })),
      (e.theme.transparent = e.extend({}, n, {
        heatmaps: [{ saturation: 0.8, lightness: 0.5 }],
        container: {
          padding: 0,
          color: "#fff",
          textShadow: "1px 1px 0 rgba(0,0,0,.5)",
        },
        count: { padding: "0 5px", height: "40px", lineHeight: "40px" },
        legend: { padding: "0 5px", height: "40px", lineHeight: "42px" },
        graph: { height: "40px" },
        column: {
          width: 5,
          background: "#999",
          heatOn: "backgroundColor",
          opacity: 0.5,
        },
      }));
  })(window, FPSMeter));
var Fps = pc.createScript("fps");
(Fps.prototype.initialize = function () {
  this.fps = new FPSMeter({ heat: !0, graph: !0 });
}),
  (Fps.prototype.update = function (t) {
    this.fps.tick();
  });
pc.script.createLoadingScreen(function (A) {
  PokiSDK.setDebug(!1);
  try {
    PokiSDK.init()
      .then(() => {})
      .catch(() => {
        console.error("Adblock enabled"), (this.adblockEnabled = !0);
      });
  } catch (A) {
    console.error("Adblock enabled"), (adblockEnabled = !0);
  }
  PokiSDK.gameLoadingStart(),
    window.addEventListener("keydown", (A) => {
      ["ArrowDown", "ArrowUp", " "].includes(A.key) && A.preventDefault();
    }),
    window.addEventListener("wheel", (A) => A.preventDefault(), {
      passive: !1,
    });
  var g, C;
  (g = [
    "body {",
    "    background-color: #000000;",
    "}",
    "",
    "#application-splash-wrapper {",
    "    position: absolute;",
    "    top: 0;",
    "    left: 0;",
    "    height: 100%;",
    "    width: 100%;",
    "    background-color: #000000;",
    "}",
    "",
    "#application-splash {",
    "    position: absolute;",
    "    top: calc(50% - -150px);",
    "    width: 264px;",
    "    left: calc(50% - 132px);",
    "}",
    "",
    "",
    "#progress-bar-container {",
    "    margin: 20px auto 0 auto;",
    "display: inline;",
    "    height: 2px;",
    "    width: 100%;",
    "}",
    "",
    "#progress-bar {",
    "    width: 0%;",
    "    height: 48.9px;",
    "    background:linear-gradient(90deg, rgba(227,54,129,1) 0%, rgba(244,192,60,1) 100%);",
    "     display: flex;",
    " position: absolute;",
    " order: 0;",
    "left: -27px;",
    "top: 0px;",
    "}",
    "#logo {",
    "position: relative;",
    "width: 100%;",
    "height: 100%;",
    "left: -10%;",
    "top: 50%;",
    "}",
    "#mask {",
    "position: fixed;",
    "width: 10%;",
    "height: 10%;",
    "left: 331px;",
    "top: 45%;",
    "}",
    "#cross {",
    "position: absolute;",
    "top: -0.321px;",
    "left: 237px;",
    "height: 50px;",
    "width: auto;",
    "}",
    "#title {",
    "             position: absolute;",
    " transform-origin: center center;",
    " top: 10%;",
    " width: 500px;",
    " left: calc(50% - 243px);",
    "}",
    "@keyframes slide-in {",
    "0% {",
    "transform:  rotate(0deg);",
    "}",
    "100% {",
    "transform:  rotate(360deg);",
    "}",
    "}",
    "@keyframes gradientAnim {",
    "0% {",
    "transform:  rotate(0deg);",
    "}",
    "100% {",
    "transform:  rotate(360deg);",
    "}",
    "}",
  ].join("\n")),
    ((C = document.createElement("style")).type = "text/css"),
    C.styleSheet
      ? (C.styleSheet.cssText = g)
      : C.appendChild(document.createTextNode(g)),
    document.head.appendChild(C),
    (function () {
      var A = document.createElement("div");
      (A.id = "application-splash-wrapper"),
        (A.src = ""),
        document.body.appendChild(A);
      var g = document.createElement("div");
      (g.id = "application-splash"),
        A.appendChild(g),
        (g.style.display = "none");
      var C = document.createElement("div");
      (C.id = "progress-bar-container"), g.appendChild(C);
      var I = document.createElement("div");
      (I.id = "progress-bar"), C.appendChild(I);
      var B = document.createElement("img");
      (B.src = ""),
        (B.id = "cross"),
        C.appendChild(B),
        (B.onload = function () {
          g.style.display = "inline";
        });
      var w = document.createElement("img");
      (w.src = ""),
        (w.id = "logo"),
        C.appendChild(w),
        (w.onload = function () {
          g.style.display = "inline";
        });
      var D = document.createElement("img");
      (D.src = ""),
        (D.id = "title"),
        A.appendChild(D);
    })(),
    A.on("preload:end", function () {
      A.off("preload:progress");
    }),
    A.on("preload:progress", function (A) {
      var g = document.getElementById("progress-bar");
      g &&
        ((A = Math.min(1, Math.max(0, A))), (g.style.width = 118.5 * A + "%"));
    }),
    A.on("start", function () {
      var A = document.getElementById("application-splash-wrapper");
      A.parentElement.removeChild(A);
    });
});
var SetAngularFactor = pc.createScript("setAngularFactor");
(SetAngularFactor.prototype.initialize = function () {
  this.entity.rigidbody.body.setAngularFactor(new Ammo.btVector3(0, 0, 1));
}),
  (SetAngularFactor.prototype.update = function (t) {});
var Parallax = pc.createScript("parallax");
Parallax.attributes.add("horizontalSpeed", { type: "number" }),
  Parallax.attributes.add("verticalSpeed", { type: "number" }),
  (Parallax.prototype.initialize = function () {}),
  (Parallax.prototype.update = function (a) {
    if (CameraManager.instance.enabled) {
      let a = CameraManager.instance.entity.getPosition();
      this.entity.getPosition();
      this.entity.setPosition(
        -a.x * this.horizontalSpeed,
        -a.y - this.verticalSpeed,
        0
      );
    }
  });
var MaterialSet = pc.createScript("materialSet");
(MaterialSet.prototype.initialize = function () {}),
  (MaterialSet.prototype.update = function (t) {});
var SetSpriteColor = pc.createScript("setSpriteColor");
(SetSpriteColor.prototype.initialize = function () {
  this.entity.sprite &&
    (this.entity.tags._list.includes("NotSticky")
      ? (this.entity.sprite.color = ThemeManager.instance.colorBouncyGpeColor)
      : (this.entity.sprite.color = ThemeManager.instance.colorStickyGpeColor));
}),
  (SetSpriteColor.prototype.update = function (t) {});
var ThemeManager = pc.createScript("themeManager");
ThemeManager.attributes.add("themes", { type: "asset", assetType: "json" }),
  ThemeManager.attributes.add("pattern", { type: "entity" }),
  ThemeManager.attributes.add("background", { type: "entity" }),
  ThemeManager.attributes.add("gradient1", { type: "entity" }),
  ThemeManager.attributes.add("gradient2", { type: "entity" }),
  (ThemeManager.prototype.initialize = function () {
    (ThemeManager.instance = this), this.changeTheme();
  }),
  (ThemeManager.prototype.changeTheme = function () {
    let e = Math.floor(
      (GameManager.instance.levelId / 10) %
        Object.values(this.themes.resource).length
    );
    (this.theme = Object.values(this.themes.resource)[e]),
      (GameManager.instance.camera.camera.clearColor =
        new pc.Color().fromString(this.theme.backgroundColor)),
      (this.gradient1.sprite.color = new pc.Color().fromString(
        this.theme.backgroundElementsColor
      )),
      (this.gradient2.sprite.color = this.gradient1.sprite.color),
      (this.pattern.sprite.spriteAsset = this.app.assets.find(
        this.theme.backgroundPattern,
        "sprite"
      )),
      (this.pattern.sprite.color = this.gradient1.sprite.color),
      (this.colorStickyGpeColor = new pc.Color().fromString(
        this.theme.stickyGpeColor
      )),
      (this.colorBouncyGpeColor = new pc.Color().fromString(
        this.theme.bouncyGpeColor
      ));
  });
var SaveManager = pc.createScript("saveManager");
(SaveManager.prototype.initialize = async function () {
  (SaveManager.instance = this),
    null === this.getItem("FirstConncection") &&
      (this.setItem("FirstConncection", !0),
      (this.showTutorial = !0),
      this.setItem("LevelId", 0),
      this.setItem("lastLevelId", -1),
      this.setItem("currentLevelDeck", []),
      this.setItem("Skins", ["Basic"]),
      this.setItem("CurrentSkin", "Basic"));
}),
  (SaveManager.prototype.getItem = function (e) {
    try {
      return JSON.parse(localStorage.getItem(e));
    } catch (t) {
      return (
        console.error(t),
        "First Connection" == e
          ? this.firstConnection
          : "LevelId" == e
          ? this.levelId
          : "lastLevelId" == e
          ? this.lastLevelId
          : "currentLevelDeck" == e
          ? this.currentLevelDeck
          : "Skins" == e
          ? this.skins
          : "CurrentSkin" == e
          ? this.currentSkin
          : null
      );
    }
  }),
  (SaveManager.prototype.setItem = function (e, t) {
    try {
      localStorage.setItem(e, JSON.stringify(t));
    } catch (n) {
      console.error(n),
        "First Connection" == e
          ? (this.firstConnection = t)
          : "LevelId" == e
          ? (this.levelId = t)
          : "lastLevelId" == e
          ? (this.lastLevelId = t)
          : "currentLevelDeck" == e
          ? (this.currentLevelDeck = t)
          : "Skins" == e
          ? (this.skins = t)
          : "CurrentSkin" == e && (this.currentSkin = t);
    }
  });
var ElementScaling = pc.createScript("elementScaling");
ElementScaling.attributes.add("landScapeScaleMultiplier", {
  type: "number",
  default: 1,
}),
  ElementScaling.attributes.add("portraitScaleMultiplier", {
    type: "number",
    default: 1,
  }),
  (ElementScaling.prototype.initialize = function () {
    (this.baseScale = new pc.Vec3().copy(this.entity.getLocalScale())),
      this.app.graphicsDevice.on("resizecanvas", (e, t) => {
        this.resize(e, t);
      });
  }),
  (ElementScaling.prototype.resize = function (e, t) {
    let i = e / 2 / (t / 2),
      a = (1 * i) / 0.9;
    (a =
      i > 1
        ? this.landScapeScaleMultiplier * a
        : (this.portraitScaleMultiplier * i) / 0.9),
      this.entity.setLocalScale(this.baseScale.x * a, this.baseScale.y * a, a);
  }),
  (ElementScaling.prototype.update = function (e) {});
var SkinBlister = pc.createScript("skinBlister");
SkinBlister.attributes.add("toColorize", { type: "entity", array: !0 }),
  SkinBlister.attributes.add("type", { type: "string" }),
  SkinBlister.attributes.add("levelText", { type: "entity" }),
  SkinBlister.attributes.add("skinSpawnPoint", { type: "entity" }),
  (SkinBlister.prototype.initialize = function () {
    (this.unlocked = !1),
      (this.skinName = ""),
      (this.entity.rigidbody.enabled = !0),
      (this.createConstraint = !0);
  }),
  (SkinBlister.prototype.postInitialize = function () {}),
  (SkinBlister.prototype.setSelected = function (t) {
    if (
      ((this.selected = t), (this.entity.findByName("Selected").enabled = t), t)
    ) {
      SaveManager.instance.setItem("CurrentSkin", this.skin),
        (ShopManager.instance.selectedSkin = this),
        ShopManager.instance.displaySkin();
      let t = { x: 0.3 };
      (this.tween = this.entity.findByName("Selected").children[0].tween(t)),
        this.tween
          .to({ x: 1 }, 0.6, pc.Linear)
          .on("update", () => {
            this.entity.findByName("Selected").children[0].sprite.opacity = t.x;
          })
          .yoyo(!0)
          .loop(!0)
          .start();
      let e = { x: 0 };
      (this.tween2 = this.entity.findByName("Selected").children[1].tween(e)),
        this.tween2
          .to({ x: 0.25 }, 0.6, pc.Linear)
          .on("update", () => {
            this.entity.findByName("Selected").children[1].sprite.opacity = e.x;
          })
          .yoyo(!0)
          .loop(!0)
          .start();
    } else
      this.tween &&
        this.tween2 &&
        (this.tween2.stop(),
        this.tween.stop(),
        (this.tween2 = null),
        (this.tween = null));
  }),
  (SkinBlister.prototype.setRewarded = function () {
    this.entity.findByName("Button").button.off("click"),
      this.entity.findByName("Button").button.on("click", () => {
        2 == ShopManager.instance.axis &&
          PokiManager.instance.rewarded(
            (t) => {
              console.log(t),
                t &&
                  (ShopManager.instance.unlockSkin(this),
                  GameManager.startLevel());
            },
            { category: "cosmetic", detail: this.skin, placement: "gameplay" }
          );
      }),
      (this.entity.findByName("Unlocked").enabled = !1),
      (this.entity.findByName("LockedRewarded").enabled = !0),
      (this.entity.findByName("Locked").enabled = !1);
  }),
  (SkinBlister.prototype.setBasic = function (lvl) {
    this.entity.findByName("Button").button.off("click"),
      this.entity.findByName("Button").button.on("click", () => {
        console.log("Level"); // hint：点击“等级解锁”皮肤 xixi
        window.plugins.toast.showWithOptions({
            message: lvl + ' 关解锁', 
            duration: 600, // which is 2000 ms. "long" is 4000. Or specify the nr of ms yourself.
            position: "center",
            addPixelsY: 200  // added a negative value to move it up a bit (default 0)
          })
      }),
      (this.entity.findByName("Unlocked").enabled = !1),
      (this.entity.findByName("LockedRewarded").enabled = !1),
      (this.entity.findByName("Locked").enabled = !0);
  }),
  (SkinBlister.prototype.unlock = function () {
    this.entity.findByName("Button").button.off("click"),
      this.entity.findByName("Button").button.on("click", () => {
        2 == ShopManager.instance.axis &&
          (this.selected
            ? GameManager.instance.startLevel()
            : (ShopManager.instance.selectedSkin.setSelected(!1),
              this.setSelected(!0)));
      }),
      (this.entity.findByName("Unlocked").enabled = !0),
      (this.entity.findByName("LockedRewarded").enabled = !1),
      (this.entity.findByName("Locked").enabled = !1),
      (this.unlocked = !0);
  }),
  (SkinBlister.prototype.postUpdate = function (t) {
    this.createConstraint &&
      ((this.createConstraint = !1),
      (this.entity.script.hingeConstraint.enabled = !0),
      this.entity.script.hingeConstraint.createConstraint());
  });
var ShopManager = pc.createScript("shopManager");
ShopManager.attributes.add("skinData", { type: "asset", assetType: "json" }),
  ShopManager.attributes.add("blistersLevel", { type: "entity" }),
  ShopManager.attributes.add("blistersRewarded", { type: "entity" }),
  ShopManager.attributes.add("pages", { type: "entity", array: !0 }),
  ShopManager.attributes.add("pageNext", { type: "entity" }),
  ShopManager.attributes.add("pagePrevious", { type: "entity" }),
  ShopManager.attributes.add("immovable", { type: "entity" }),
  ShopManager.attributes.add("skinSpawnPoint", { type: "entity" }),
  ShopManager.attributes.add("numberSkins", { type: "entity" }),
  ShopManager.attributes.add("slideLimit", { type: "number" }),
  ShopManager.attributes.add("maxHeight", { type: "number" }),
  ShopManager.attributes.add("minHeight", { type: "number" }),
  (ShopManager.prototype.initialize = function () {
    (ShopManager.instance = this),
      (this.app.timeScale = 1),
      (this.selectedPage = this.pages[1]),
      this.populateShop(),
      this.pageNext.button.on("click", (e) => {
        this.nextPage();
      }),
      this.pagePrevious.button.on("click", (e) => {
        this.previousPage();
      });
    var e = this.app.touch;
    e
      ? (e.on(pc.EVENT_TOUCHSTART, this.onMouseDown, this),
        e.on(pc.EVENT_TOUCHMOVE, this.onMouseMove, this),
        e.on(pc.EVENT_TOUCHEND, this.onMouseUp, this))
      : (this.app.mouse.on(pc.EVENT_MOUSEMOVE, this.onMouseMove, this),
        this.app.mouse.on(pc.EVENT_MOUSEDOWN, this.onMouseDown, this),
        this.app.mouse.on(pc.EVENT_MOUSEUP, this.onMouseUp, this)),
      this.on(
        "destroy",
        function () {
          this.app.mouse &&
            (this.app.mouse.off(pc.EVENT_MOUSEDOWN, this.onMouseDown, this),
            this.app.mouse.off(pc.EVENT_MOUSEUP, this.onMouseUp, this),
            this.app.mouse.off(pc.EVENT_MOUSEMOVE, this.onMouseMove, this)),
            this.app.touch &&
              (this.app.touch.off(pc.EVENT_TOUCHSTART, this.onMouseDown, this),
              this.app.touch.off(pc.EVENT_TOUCHEND, this.onMouseUp, this),
              this.app.touch.off(pc.EVENT_TOUCHMOVE, this.onMouseMove, this));
        },
        this
      ),
      (this.shopOffsetY = 0),
      (this.lastShopOffsetY = 0),
      (this.yVelocity = 0),
      (this._time = pc.now()),
      this.immovable.setPosition(
        CameraManager.instance.entity.getPosition().x,
        CameraManager.instance.entity.getPosition().y - 5,
        this.immovable.getPosition().z
      );
  }),
  (ShopManager.prototype.nextPage = function () {
    this.pages[this.pages.indexOf(this.selectedPage) + 1] &&
      ((this.selectedPage =
        this.pages[this.pages.indexOf(this.selectedPage) + 1]),
      (CameraManager.instance.shopOffsetY = 0),
      (this.shopOffsetY = CameraManager.instance.shopOffsetY));
  }),
  (ShopManager.prototype.previousPage = function () {
    this.pages[this.pages.indexOf(this.selectedPage) - 1] &&
      ((this.selectedPage =
        this.pages[this.pages.indexOf(this.selectedPage) - 1]),
      (CameraManager.instance.shopOffsetY = 0),
      (this.shopOffsetY = CameraManager.instance.shopOffsetY));
  }),
  (ShopManager.prototype.populateShop = function () {
    let e;
    Object.values(this.skinData.resources[0]).forEach((t, s) => {
      if ("BASIC" == t.type) {
        if (null == this.blistersLevel.children[t.order - 1]) return;
        (e = this.blistersLevel.children[t.order - 1].script.skinBlister),
          e.setBasic(t.level.toString()),
          // (e.levelText.element.text = "LV." + t.level.toString()),
          (e.levelText.element.text = "@" + t.level.toString()),
          e.levelText.parent.setRotation(
            new pc.Quat().setFromEulerAngles(0, 0, randomNumber(-35, 35))
          );
      } else if ("REWARDED" == t.type) {
        if (null == this.blistersRewarded.children[t.order - 1]) return;
        (e = this.blistersRewarded.children[t.order - 1].script.skinBlister),
          e.setRewarded();
      }
      let i;
      (e.skin = t.name),
        SaveManager.instance.getItem("Skins").includes(t.name) && e.unlock(),
        e.setSelected(!1),
        SaveManager.instance.getItem("CurrentSkin") == t.name &&
          (e.setSelected(!0), (e.selected = !0), (this.currentSkin = e)),
        e.toColorize.forEach((e) => {
          e.sprite.color = new pc.Color().fromString(t.packagingColor);
        }),
        this.app.assets.find(t.name, "template")
          ? ((i = this.app.assets
              .find(t.name, "template")
              .resource.instantiate()),
            doStuffToEntity(
              this.app.assets.find(t.name, "template").resource._templateRoot,
              i
            ))
          : ((i = this.app.assets
              .find("Basic", "template")
              .resource.instantiate()),
            doStuffToEntity(
              this.app.assets.find("Basic", "template").resource._templateRoot,
              i
            )),
        (i.script.playerController.enabled = !1),
        (i.script.skinDisplayer.enabled = !0),
        (this.numberSkins.element.text =
          SaveManager.instance.getItem("Skins").length.toString() + " / 15"),
        console.log(SaveManager.instance.getItem("Skins").length);
      let a = e.skinSpawnPoint.getPosition();
      i.setPosition(a.x, a.y, a.z), this.entity.parent.addChild(i);
    });
  }),
  (ShopManager.prototype.unlockSkin = function (e) {
    let t = SaveManager.instance.getItem("Skins");
    t.push(e.skin),
      SaveManager.instance.setItem("Skins", t),
      e.unlock(),
      ShopManager.instance.selectedSkin.setSelected(!1),
      e.setSelected(!0);
  }),
  (ShopManager.prototype.displaySkin = function (e) {
    this.displayedSkin && this.displayedSkin.destroy(),
      (this.displayedSkin = this.app.assets
        .find(this.selectedSkin.skin, "template")
        .resource.instantiate()),
      doStuffToEntity(
        this.app.assets.find(this.selectedSkin.skin, "template").resource
          ._templateRoot,
        this.displayedSkin
      ),
      (this.displayedSkin.script.playerController.enabled = !1),
      (this.displayedSkin.script.skinDisplayer.enabled = !0);
    let t = this.skinSpawnPoint.getPosition();
    this.displayedSkin.setPosition(t.x, t.y, t.z),
      this.entity.parent.addChild(this.displayedSkin);
  }),
  (ShopManager.prototype.postUpdate = function (e) {
    CameraManager.instance &&
      this.prev &&
      ((this.yVelocity =
        CameraManager.instance.getPosition().y - this.prev / e),
      (this.prev = CameraManager.instance.getPosition().y)),
      this.immovable.setPosition(
        this.immovable.getPosition().x,
        CameraManager.instance.entity.getPosition().y - 5,
        this.immovable.getPosition().z
      );
  }),
  (ShopManager.prototype.onMouseDown = function (e) {
    let t, s;
    e.hasOwnProperty("touches")
      ? ((t = e.touches[0].touch.clientX), (s = e.touches[0].touch.clientY))
      : ((t = e.x), (s = e.y)),
      (this.mouseDown = !0),
      (this.startPos = new pc.Vec2(t, s)),
      (this.prev = 0),
      (this.axis = 2),
      e.event.preventDefault();
  }),
  (ShopManager.prototype.onMouseMove = function (e) {
    let t, s;
    e.hasOwnProperty("touches")
      ? ((t = e.touches[0].touch.clientX), (s = e.touches[0].touch.clientY))
      : ((t = e.x), (s = e.y)),
      this.mouseDown &&
        ((this.mousePos = new pc.Vec2(t, s)),
        (this.delta = this.mousePos
          .sub(this.startPos)
          .mulScalar(
            (2 * GameManager.instance.camera.camera.orthoHeight) /
              window.innerHeight
          )),
        (this.delta.x < -1 || this.delta.x > 1) && 2 == this.axis
          ? (this.axis = 0)
          : (this.delta.y < -1 || this.delta.y > 1) &&
            2 == this.axis &&
            (this.axis = 1),
        0 == this.axis
          ? (CameraManager.instance.shopOffsetX = -this.delta.x)
          : (CameraManager.instance.shopOffsetY = clamp(
              this.shopOffsetY + this.delta.y,
              1 == this.pages.indexOf(this.selectedPage) ? -27 : -5,
              5
            ))),
      e.event.preventDefault();
  }),
  (ShopManager.prototype.onMouseUp = function (e) {
    if (1 == this.axis) {
      let e = 1 == this.pages.indexOf(this.selectedPage) ? 22 : 0;
      CameraManager.instance.shopOffsetY > 0
        ? (this.shopOffsetY = 0)
        : CameraManager.instance.shopOffsetY < -e
        ? (this.shopOffsetY = -e)
        : (this.shopOffsetY = CameraManager.instance.shopOffsetY),
        (CameraManager.instance.shopOffsetY = this.shopOffsetY);
    } else
      this.delta &&
        (this.delta.x > this.slideLimit
          ? this.nextPage()
          : this.delta.x < -this.slideLimit && this.previousPage());
    (CameraManager.instance.shopOffsetX = 0),
      (this.mouseDown = !1),
      (this.axis = 3),
      e.event.preventDefault();
  });
var SkinDisplayer = pc.createScript("skinDisplayer");
(SkinDisplayer.prototype.initialize = function () {
  setTimeout(() => {
    this.entity.script.playerController.changeSpringStifness(10);
  }),
    this.impulseRoutine();
}),
  (SkinDisplayer.prototype.impulseRoutine = function (i) {
    this.entity.script &&
      (this.entity.script.playerController.head.rigidbody.applyImpulse(
        randomNumber(-0.5, 0.5),
        0,
        0
      ),
      setTimeout(() => {
        this.impulseRoutine();
      }, randomNumber(0, 15e3)));
  }),
  (SkinDisplayer.prototype.postUpdate = function (i) {});
randomNumber = function (n, r) {
  return Math.random() * (r - n) + n;
};
var ShopBackground = pc.createScript("shopBackground");
ShopBackground.attributes.add("offset", { type: "number" }),
  (ShopBackground.prototype.initialize = function () {
    this.ogPosition =
      this.entity.getPosition().y -
      CameraManager.instance.entity.getPosition().y;
  }),
  (ShopBackground.prototype.postUpdate = function (t) {
    this.entity.setPosition(
      this.entity.getPosition().x,
      CameraManager.instance.entity.getPosition().y - this.offset,
      this.entity.getPosition().z
    );
  });
var SetRigidbodyPosition = pc.createScript("setRigidbodyPosition");
(SetRigidbodyPosition.prototype.initialize = function () {}),
  (SetRigidbodyPosition.prototype.update = function (i) {
    this.entity.rigidbody.teleport(
      this.entity.getPosition().x,
      CameraManager.instance.entity.getPosition().y,
      this.entity.getPosition().z
    );
  });
var PokiManager = pc.createScript("pokiManager");
(PokiManager.prototype.initialize = function () {
  (PokiManager.instance = this),
    PokiSDK.gameLoadingFinished(),
    (this.rewardedOn = !1);
}),
  (PokiManager.prototype.interstitial = function (e) {
    PokiSDK.gameplayStop(),
      PokiSDK.commercialBreak().then(() => {
        e();
      });
  }),
  (PokiManager.prototype.removeBanner = function (e) {}),
  (PokiManager.prototype.displayBanner = function (e) {}),
  (PokiManager.prototype.rewarded = function (e, a) {
    if (window.com && window.com.zzz) {
      window.com.zzz.zz_mk_ad.playAd(
        () => { console.log("视频 OK"); e(true); },
        () => { console.log("视频 CNCL"); e(false); }
      ); // fix
      return;
    } else {
      console.warn("额呵呵");
    }
    this.rewardedOn
      ? e(!1)
      : (PokiSDK.gameplayStop(),
        (this.rewardedOn = !0),
        console.log(a), // xixi
        PokiSDK.rewardedBreak(a.category, a.details, a.placement)
          .then((a) => {
            console.log(a); // xixi
            (this.rewardedOn = !1),
              a || this.app.fire("shopAdblockPopup"),
              e(a);
          })
          .catch((e) => {
            (this.rewardedOn = !1), console.log("error", e); // xixi
          }));
  }),
  (PokiManager.prototype.update = function (e) {});
var OnBoarding = pc.createScript("onBoarding");
OnBoarding.attributes.add("handPosition", { type: "vec2", default: [0, 0] }),
  OnBoarding.attributes.add("sausagePosition", {
    type: "vec2",
    default: [0, 0],
  }),
  OnBoarding.attributes.add("leftKey", { type: "entity" }),
  OnBoarding.attributes.add("rightKey", { type: "entity" }),
  OnBoarding.attributes.add("controlText", { type: "entity" }),
  OnBoarding.attributes.add("block", { type: "entity" }),
  (OnBoarding.prototype.initialize = function () {
    (this.sausage = this.entity.findByName("Sausage")),
      (this.dash = this.entity.findByName("Dash")),
      (this.hand = this.entity.findByName("Hand")),
      (this.hand.startPos = this.hand.getPosition()),
      (this.sausage.startPos = this.sausage.getPosition()),
      (this.dash.startScale = this.dash.getLocalScale()),
      (this.rightKey.startScale = this.rightKey.getLocalScale()),
      (this.leftKey.startScale = this.leftKey.getLocalScale()),
      this.app.touch
        ? this.startMouseAnimation()
        : this.startKeyboardAnimation();
  }),
  (OnBoarding.prototype.startMouseAnimation = function (t) {
    this.controlText.element &&
      (this.app.touch
        ? (this.controlText.enabled = !1)
        : (this.controlText.element.text = "Mouse\nControls"),
      (this.leftKey.enabled = !1),
      (this.rightKey.enabled = !1),
      (this.hand.enabled = !0),
      this.hand.sprite &&
        ((this.dash.enabled = !0),
        (this.block.enabled = !0),
        (this.sausage.enabled = !0),
        this.dash.setLocalScale(
          0,
          this.dash.startScale.y,
          this.dash.startScale.z
        ),
        this.sausage.setPosition(
          this.sausage.startPos.x,
          this.sausage.startPos.y,
          this.sausage.startPos.z
        ),
        this.sausage.setLocalEulerAngles(0, 0, 0),
        this.hand.setPosition(
          this.hand.startPos.x,
          this.hand.startPos.y,
          this.hand.startPos.z
        ),
        (this.hand.sprite.opacity = 1),
        this.dash
          .tween(this.dash.getLocalScale())
          .to(new pc.Vec3(0.619, this.dash.startScale.y, 0), 1.5, pc.SineOut)
          .on("complete", () => {
            setTimeout(() => {
              (this.dash.enabled = !1),
                this.sausage
                  .tween(this.sausage.getLocalPosition())
                  .to(
                    new pc.Vec3(
                      this.sausagePosition.x,
                      this.sausagePosition.y,
                      0
                    ),
                    0.6,
                    pc.SineOut
                  )
                  .on("complete", () => {
                    setTimeout(() => {
                      (this.leftKey.enabled = !1),
                        (this.rightKey.enabled = !1),
                        (this.block.enabled = !1),
                        (this.sausage.enabled = !1),
                        (this.hand.enabled = !1),
                        (this.dash.enabled = !1),
                        this.app.touch
                          ? this.startMouseAnimation()
                          : this.startKeyboardAnimation();
                    }, 1e3);
                  })
                  .on("update", () => {
                    this.sausage &&
                      this.sausage.setLocalEulerAngles(
                        0,
                        0,
                        this.sausage.getLocalEulerAngles().z - 10
                      );
                  })
                  .start();
              let t = { x: 1 };
              this.hand
                .tween(t)
                .to({ x: 0 }, 0.6, pc.Linear)
                .on("update", () => {
                  this.hand.sprite && (this.hand.sprite.opacity = t.x);
                })
                .start(),
                this.hand
                  .tween(this.hand.getLocalPosition())
                  .to(
                    new pc.Vec3(
                      this.handPosition.x,
                      this.handPosition.y - 0.5,
                      0
                    ),
                    0.6,
                    pc.SineOut
                  )
                  .on("update", () => {})
                  .start();
            }, 250);
          })
          .start(),
        this.hand
          .tween(this.hand.getLocalPosition())
          .to(
            new pc.Vec3(this.handPosition.x, this.handPosition.y, 0),
            1.2,
            pc.SineOut
          )
          .on("update", () => {})
          .start()));
  }),
  (OnBoarding.prototype.startKeyboardAnimation = function (t) {
    this.controlText.element &&
      ((this.controlText.element.text = "KEYBOARD\nControls"),
      (this.leftKey.enabled = !0),
      (this.rightKey.enabled = !0),
      (this.block.enabled = !0),
      (this.sausage.enabled = !0),
      (this.hand.enabled = !1),
      (this.dash.enabled = !0),
      this.sausage.setPosition(
        this.sausage.startPos.x,
        this.sausage.startPos.y,
        this.sausage.startPos.z
      ),
      this.sausage.setLocalEulerAngles(0, 0, 0),
      this.dash.setLocalEulerAngles(new pc.Vec3(0, 0, -23.25)),
      this.leftKey
        .tween(this.leftKey.getLocalScale())
        .to(
          new pc.Vec3(
            this.leftKey.startScale.y / 2,
            this.leftKey.startScale.y / 2,
            0
          ),
          1,
          pc.SineOut
        )
        .on("loop", () => {
          (this.dash.enabled = !1),
            this.sausage
              .tween(this.sausage.getLocalPosition())
              .to(
                new pc.Vec3(this.sausagePosition.x, this.sausagePosition.y, 0),
                0.6,
                pc.SineOut
              )
              .on("complete", () => {
                setTimeout(() => {
                  this.startMouseAnimation();
                }, 1e3);
              })
              .on("update", () => {
                this.sausage &&
                  this.sausage.setLocalEulerAngles(
                    0,
                    0,
                    this.sausage.getLocalEulerAngles().z - 10
                  );
              })
              .start(),
            console.log("tete");
        })
        .on("update", () => {})
        .repeat(2)
        .yoyo(!0)
        .start(),
      this.dash
        .tween(this.dash.getLocalEulerAngles())
        .rotate(new pc.Vec3(0, 0, 23.25), 1, pc.Linear)
        .start());
  }),
  (OnBoarding.prototype.update = function (t) {});
var TweenText = pc.createScript("tweenText");
(TweenText.prototype.initialize = function () {
  this.entity
    .tween(this.entity.getLocalScale())
    .to(
      new pc.Vec3(
        1.15 * this.entity.getLocalScale().x,
        1.15 * this.entity.getLocalScale().y,
        1
      ),
      0.25,
      pc.CircularInOut
    )
    .on("complete", function () {})
    .loop(!0)
    .yoyo(!0)
    .start();
}),
  (TweenText.prototype.update = function (t) {});
var RotationTween = pc.createScript("rotationTween");
(RotationTween.prototype.initialize = function () {
  this.entity
    .tween(
      new pc.Vec3(
        this.entity.getLocalEulerAngles().x,
        this.entity.getLocalEulerAngles().y,
        -3
      )
    )
    .rotate(
      new pc.Vec3(
        this.entity.getLocalEulerAngles().x,
        this.entity.getLocalEulerAngles().y,
        3
      ),
      1,
      pc.CircularInOut
    )
    .on("complete", function () {})
    .loop(!0)
    .yoyo(!0)
    .start();
}),
  (RotationTween.prototype.update = function (t) {
    let e = CameraManager.instance.entity.getPosition();
    this.entity.setPosition(e.x, this.entity.getPosition().y, 0);
  });
var EquipButton = pc.createScript("equipButton");
(EquipButton.prototype.initialize = function () {}),
  (EquipButton.prototype.update = function (t) {});
var NewSkinPopup = pc.createScript("newSkinPopup");
NewSkinPopup.attributes.add("skinSpawnPoint", { type: "entity" }),
  NewSkinPopup.attributes.add("equipButton", { type: "entity" }),
  NewSkinPopup.attributes.add("quitButton", { type: "entity" }),
  NewSkinPopup.attributes.add("rewardedIcon", { type: "entity" }),
  NewSkinPopup.attributes.add("newSkinText", { type: "entity" }),
  NewSkinPopup.attributes.add("shine", { type: "entity" }),
  (NewSkinPopup.prototype.initialize = function () {
    this.equipButton.button.on("click", () => {
      console.warn("视频，解锁？");
      this.rewarded
        ? PokiManager.instance.rewarded(
            (t) => {
              if (t) {
                let t = SaveManager.instance.getItem("Skins");
                t.push(this.skin),
                  SaveManager.instance.setItem("Skins", t),
                  SaveManager.instance.setItem("CurrentSkin", this.skin),
                  GameManager.instance.startLevel();
              }
              SkinsButton.instance.showButton(),
                (GameManager.instance.playerController.activated = !0),
                this.entity.destroy();
            },
            { category: "cosmetic", detail: this.skin, placement: "popup" }
          )
        : (SaveManager.instance.setItem("CurrentSkin", this.skin),
          GameManager.instance.startLevel(),
          this.entity.destroy());
    }),
      this.quitButton.button.on("click", () => {
        SkinsButton.instance.showButton(),
          (GameManager.instance.playerController.activated = !0),
          this.entity.destroy();
      }),
      (this.displayedSkin = this.app.assets
        .find(this.skin, "template")
        .resource.instantiate()),
      doStuffToEntity(
        this.app.assets.find(this.skin, "template").resource._templateRoot,
        this.displayedSkin
      ),
      (this.displayedSkin.script.playerController.enabled = !1),
      (this.displayedSkin.script.skinDisplayer.enabled = !0);
    let t = this.skinSpawnPoint.getPosition();
    this.displayedSkin.setPosition(t.x, t.y, t.z),
      this.entity.addChild(this.displayedSkin),
      this.rewarded &&
        // ((this.equipButton.findByName("Text").element.text = "UNLOCK"),
        ((this.equipButton.findByName("Text").element.text = "? ?"),
        (this.equipButton.element.color = new pc.Color().fromString("F8B100")),
        (this.rewardedIcon.enabled = !0),
        (this.newSkinText.enabled = !1),
        (this.shine.enabled = !1));
  }),
  (NewSkinPopup.prototype.update = function (t) {});
var PointerLock = pc.createScript("pointerLock");
PointerLock.attributes.add("pointer", { type: "entity" }),
  PointerLock.attributes.add("sides", { type: "entity", array: !0 }),
  (PointerLock.prototype.initialize = function () {
    (PointerLock.instance = this),
      (this.locked = !1),
      (this.init = !1),
      this.app.mouse.disableContextMenu(),
      (document.body.style.cursor = "none"),
      document.addEventListener("mouseleave", (t) => {
        t.clientX <= 0 && this.blink(this.sides[3]),
          t.clientX >= window.innerWidth && this.blink(this.sides[1]),
          t.clientY <= 0 && this.blink(this.sides[0]),
          t.clientY >= window.innerWidth && this.blink(this.sides[2]);
      }),
      this.app.mouse.on(pc.EVENT_MOUSEMOVE, this.onMouseMove, this);
  }),
  (PointerLock.prototype.blink = function (t) {
    let i = { x: 0 },
      e = t.tween(i);
    e.to({ x: 0.5 }, 0.25, pc.Linear, 0, 1)
      .on("update", () => {
        e.manager.setTs(1 / this.app.timeScale), (t.element.opacity = i.x);
      })
      .yoyo(!0)
      .repeat(2)
      .start();
  }),
  (PointerLock.prototype.enablePointerLock = function (t) {
    this.app.mouse.enablePointerLock(),
      this.app.mouse.disableContextMenu(),
      (this.locked = !0);
  }),
  (PointerLock.prototype.disablePointerLock = function (t) {
    this.app.mouse.disablePointerLock(), (this.locked = !1);
  }),
  (PointerLock.prototype.onMouseDown = function (t) {}),
  (PointerLock.prototype.onMouseMove = function (t) {
    this.init || ((this.init = !0), (this.x = t.x), (this.y = t.y)),
      pc.Mouse.isPointerLocked() &&
        ((this.x += t.event.movementX),
        (this.y += t.event.movementY),
        this.x < 0 && (this.x = 0),
        this.y < 0 && (this.y = 0)),
      (this.x = t.x),
      (this.y = t.y),
      (this.point = CameraManager.instance.entity.camera.screenToWorld(
        this.x,
        this.y,
        0
      )),
      this.pointer.setPosition(this.point.x, this.point.y, 0);
  }),
  (PointerLock.prototype.update = function (t) {}); // PlayerLock.js
/* var PlayerController = pc.createScript('playerController');


PlayerController.attributes.add('head', { type: 'entity' })
PlayerController.attributes.add('tail', { type: 'entity' })
PlayerController.attributes.add('segmentMass', { type: 'number', default: 1 })
PlayerController.attributes.add('coolDownTime', { type: 'number' })
PlayerController.attributes.add('sausageSegments', { type: 'entity', array: true });
PlayerController.attributes.add('dashMeter', { type: 'entity' });
PlayerController.attributes.add('eyes', { type: 'entity', array: true });
PlayerController.attributes.add('force', {
    type: 'number',
    default: 9
});
PlayerController.attributes.add('forceThresholds', {
    title: 'Force Thresholds',
    type: 'vec2',
    default: [0.5, 2.5]
});
PlayerController.attributes.add('springMultiplier', {
    type: 'number',
    default: 1
});
PlayerController.attributes.add('springs', { type: 'entity', array: true });
PlayerController.attributes.add('landParticles', { type: 'asset', assetType: 'template' });

PlayerController.attributes.add('debugColor', {
    title: 'Debug Color',
    description: 'The color of the debug rendering of the constraint.',
    type: 'rgb',
    default: [1, 0, 0]
});

PlayerController.attributes.add('activated', { type: 'boolean', default: true })
PlayerController.attributes.add('debug', { type: 'boolean', default: true })



// initialize code called once per entity
PlayerController.prototype.initialize = function () {

    PlayerController.instance = this;
    this.headOrTail = this.sausageSegments[0]
    console.log(this.entity)
    this.screenToWorldRatio = GameManager.instance.camera.camera.orthoHeight * 2 / window.innerHeight;
    this.startPos = new pc.Vec2();
    this.attached = false;
    this.loaded = false;
    this.segment = this.head;
    this.stuckSegment = this.segment;
    this.offset = new pc.Vec2();
    this.coolDown = false;

    var touch = this.app.touch;
    if (touch) {
        touch.on(pc.EVENT_TOUCHSTART, this.onMouseDown, this);
        touch.on(pc.EVENT_TOUCHMOVE, this.onMouseMove, this);
        touch.on(pc.EVENT_TOUCHEND, this.onMouseUp, this);

    }
    else {
        this.app.mouse.on(pc.EVENT_MOUSEMOVE, this.onMouseMove, this);
        this.app.mouse.on(pc.EVENT_MOUSEDOWN, this.onMouseDown, this);
        this.app.mouse.on(pc.EVENT_MOUSEUP, this.onMouseUp, this);
    }


    this.on('destroy', function () {
        PlayerController.instance = undefined;
        if (this.app.mouse) {
            this.app.mouse.off(pc.EVENT_MOUSEDOWN, this.onMouseDown, this);
            this.app.mouse.off(pc.EVENT_MOUSEUP, this.onMouseUp, this);
        }

        if (this.app.touch) {
            this.app.touch.off(pc.EVENT_TOUCHSTART, this.onMouseDown, this);
            this.app.touch.off(pc.EVENT_TOUCHEND, this.onMouseUp, this);
            this.app.touch.off(pc.EVENT_TOUCHMOVE, this.onMouseMove, this);
        }
    }, this);

    this.sausageSegments.forEach((segment) => {
        segment.collision.on('collisionstart', (event) => {
            this.onCollisionEnter(event, segment);
        }, this);

    });






    setTimeout(() => {
        this.springs.forEach((spring) => {
            spring.script.springJoint.stifness = spring.script.springJoint.initialStifness * this.springMultiplier;
            spring.script.springJoint.damping = spring.script.springJoint.initialStifness * this.springMultiplier;

            spring.script.springJoint.createConstraint();
        });
    }, 100);


    this.on('attr', function (name, value, prev) {
        // If any constraint properties change, recreate the constraint
        if (name === 'springMultiplier') {
            this.changeSpringStifness(value);
        }
    });

    setTimeout(() => {

    }, 500)


};

PlayerController.prototype.changeSpringStifness = function (stifness) {
    this.springs.forEach((spring) => {
        spring.script.springJoint.stifness = spring.script.springJoint.initialStifness * this.springMultiplier * stifness;
        spring.script.springJoint.damping = spring.script.springJoint.initialStifness * this.springMultiplier * stifness;

        spring.script.springJoint.createConstraint();
    });

}
PlayerController.prototype.hingeLimits = function (stiff) {
    this.sausageSegments.forEach((segment, index) => {

        if (index > 0) {
            if (stiff) {
                segment.script.hingeConstraint.limits = new pc.Vec2(-7, 7)
            }
            else {
                segment.script.hingeConstraint.limits = new pc.Vec2(-45, 45)
            }
        }

    });
}

PlayerController.prototype.onMouseDown = function (event) {
    let x, y;

    if (!PointerLock.instance.locked && this.activated) {
        PointerLock.instance.enablePointerLock()
    }


    if (this.activated) {

        if (event.hasOwnProperty("touches")) {

            x = event.touches[0].touch.clientX;
            y = event.touches[0].touch.clientY;

        }
        else {

            x = PointerLock.instance.x;
            y = PointerLock.instance.y;
        }

        if ((this.loaded || this.debug) && !GameManager.instance.finished && !this.paused) {
            this.mouseDown = true;
            this.dashMeter.setLocalScale(0, 0, 0)
            this.dashMeter.enabled = true;
            this.app.timeScale = 0.1;
            this.startPos = new pc.Vec2(x, y);
        }

    }

    event.event.preventDefault();

};
const clamp = (num, min, max) => Math.min(Math.max(num, min), max);

PlayerController.prototype.onMouseMove = function (event) {
    let x, y;

    if (event.hasOwnProperty("touches")) {

        x = event.touches[0].touch.clientX;
        y = event.touches[0].touch.clientY;
    }
    else {

        x = PointerLock.instance.x;
        y = PointerLock.instance.y;
    }

    if (this.activated) {
        if (this.mouseDown) {
            this.mouseMoved = true;
            this.mousePos = new pc.Vec2(x, y);
            this.delta = this.mousePos.sub(this.startPos).mulScalar(this.screenToWorldRatio);
            this.offset = new pc.Vec2().copy(this.delta).normalize().mulScalar(clamp(this.delta.length(), this.forceThresholds.x, this.forceThresholds.y))
            this.dashMeter.setLocalScale(this.offset.length() / 5, this.offset.length() / 5, 0);

        }
    }

    event.event.preventDefault();

};

PlayerController.prototype.onMouseUp = function (event) {
    if (PointerLock.instance.locked && this.activated) {
        PointerLock.instance.disablePointerLock()
    }
    if ((this.loaded || this.debug && !this.paused)) {
        this.unstick();

        if (this.delta != undefined && this.delta != pc.Vec3.ZERO)
            this.dash(this.processForce(this.delta));


        this.mouseDown = false;
        this.app.timeScale = 1;
        this.paused = false;


        this.dashMeter.enabled = false;
        this.mouseMoved = false;
    }



    event.event.preventDefault();


};

PlayerController.prototype.processForce = function (inputDelta) {
    let direction = inputDelta.clone().normalize();
    let magnitude = clamp(inputDelta.length(), 0.5, 2.5);
    return direction.mulScalar(magnitude * this.force);
};

PlayerController.prototype.dash = function (force) {


    let velocity = new Ammo.btVector3();
    this.sausageSegments.forEach((segment, index) => {
        segment.rigidbody.body.setActivationState(1); // 1 - active
        segment.rigidbody.body.activate();
        velocity.setValue(0, 0, 0);
        segment.rigidbody.body.setLinearVelocity(velocity);
        velocity.setValue(0, 0, 0);
        this.sausageSegments[2].rigidbody.body.setAngularVelocity(velocity);
    });

    this.eyes.forEach((eye, index) => {
        if (this.eye) {

            velocity.setValue(0, 0, 0);
            eye.rigidbody.body.setLinearVelocity(velocity);
        }

    });


    this.coolDown = true;
    setTimeout(() => {
        this.coolDown = false;
    }, this.coolDownTime);


    this.headOrTail.rigidbody.body.applyImpulse(new Ammo.btVector3(force.x, -force.y, 0));

};


PlayerController.prototype.unstick = function (event) {

    if (this.activated && this.attached && !this.paused) {
        let hinge = this.stuckSegment.children[0].script.hinge;
        hinge.enabled = false;
        this.hingeLimits(true)
        this.changeSpringStifness(8);

        this.loaded = false;
        this.attached = false;
    }

}

PlayerController.prototype.stick = function (segment, entity) {
    this.attached = true;
    this.loaded = true;
    let hinge = segment.children[0].script.hinge;
    this.stuckSegment = segment;

    let distance = 0;

    this.sausageSegments.forEach((seg) => {
        //   if (seg == this.head || seg == this.tail && seg != segment) {
        let thisDistance = this.stuckSegment.getPosition().sub(seg.getPosition()).length();// Vector2.Distance(stuckSausageSegment.transform.position, segment.transform.position);
        if (thisDistance <= distance) return;

        distance = thisDistance;
        this.headOrTail = seg;
        //  }
    });

    this.hingeLimits(false)
    this.changeSpringStifness(3);

    hinge.entityB = entity.other;
    hinge.pivotA = entity.contacts[0].localPoint;
    hinge.pivotB = entity.contacts[0].localPointOther;
    hinge.createConstraint();
    hinge.enabled = true;
    let particles = this.landParticles.resource.instantiate();
    segment.addChild(particles)
    particles.particlesystem.play()
    particles.setLocalPosition(0, 0, 0)
    CameraManager.instance.flashOutline(.5)
};


PlayerController.prototype.onCollisionEnter = function (entity, segment) {


    if (!this.attached && (!this.coolDown) && !entity.other.tags._list.includes("NotSticky")) {
        this.oldEntity = entity;

        if (!entity.other.tags._list.includes("Segment")) {
            this.stick(segment, entity)
            this.mouseDown = false;
            this.app.timeScale = 1;


        }
    }

};


// update code called every frame
PlayerController.prototype.update = function (dt) {

    if (this.head.getPosition().y < -20 && !GameManager.instance.finished && this.activated) {
        this.activated = false;
        GameManager.instance.startLevel()


    }
    if (this.dashMeter.enabled) {

        this.dashMeter.setPosition(this.headOrTail.getPosition());
        this.dashMeter.setRotation(new pc.Quat().setFromEulerAngles(0, 0, 180 * Math.atan2(-this.offset.normalize().y, this.offset.normalize().x) / Math.PI));
    }


};

// swap method called for script hot-reloading
// inherit your script state here
// PlayerController.prototype.swap = function(old) { };

// to learn more about script anatomy, please read:
// https://developer.playcanvas.com/en/user-manual/scripting/ */

var SkipLevel = pc.createScript("skipLevel");
SkipLevel.attributes.add("nextLevel", { type: "entity" }),
  (SkipLevel.prototype.initialize = function () {
    (SkipLevel.instance = this),
      GameManager.instance.respawned && this.showButton();
  }),
  (SkipLevel.prototype.showButton = function (e) {
    this.nextLevel.enabled = !0;
    let t = this.nextLevel.tween(this.nextLevel.getLocalPosition());
    t.to(
      new pc.Vec3(10, this.nextLevel.getLocalPosition().y, 0),
      0.5,
      pc.ElasticOut
    )
      .on("complete", () => {})
      .on("update", () => {
        t.manager.setTs(1 / this.app.timeScale);
      })
      .start();
  }),
  (SkipLevel.prototype.hideButton = function (e) {
    e || (e = 0.5);
    let t = this.nextLevel.tween(this.nextLevel.getLocalPosition());
    t.to(
      new pc.Vec3(80, this.nextLevel.getLocalPosition().y, 0),
      e,
      pc.ElasticIn
    )
      .on("complete", () => {
        this.nextLevel.enabled = !1;
      })
      .on("update", () => {
        t.manager.setTs(1 / this.app.timeScale);
      })
      .start();
  });
var SkinsButton = pc.createScript("skinsButton");
SkinsButton.attributes.add("skinsButton", { type: "entity" }),
  (SkinsButton.prototype.initialize = function () {
    (SkinsButton.instance = this), (this.hidden = !0);
  }),
  (SkinsButton.prototype.showButton = function (t) {
    if (!this.skinsButton.enabled) {
      this.skinsButton.enabled = !0;
      let t = this.skinsButton.tween(this.skinsButton.getLocalPosition());
      t.to(
        new pc.Vec3(30, this.skinsButton.getLocalPosition().y, 0),
        0.5,
        pc.ElasticOut
      )
        .on("complete", () => {})
        .on("update", () => {
          t.manager.setTs(1 / this.app.timeScale);
        })
        .start();
    }
  }),
  (SkinsButton.prototype.hideButton = function (t) {
    if ((t || (t = 0.5), this.skinsButton.enabled)) {
      this.hidden = !0;
      let n = this.skinsButton.tween(this.skinsButton.getLocalPosition());
      n.to(
        new pc.Vec3(140, this.skinsButton.getLocalPosition().y, 0),
        t,
        pc.ElasticIn
      )
        .on("complete", () => {
          this.skinsButton.enabled = !1;
        })
        .on("update", () => {
          n.manager.setTs(1 / this.app.timeScale);
        })
        .start();
    }
  });
var DisableAd = pc.createScript("disableAd");
(DisableAd.prototype.initialize = function () {
  this.app.on("shopAdblockPopup", () => {
    this.showPopup();
  });
}),
  (DisableAd.prototype.showPopup = function () {
    let t = { x: 0 },
      e = this.entity.tween(t);
    e.to({ x: 1 }, 1, pc.ExponentialOut, 0, 1)
      .on("update", () => {
        e.manager.setTs(1 / this.app.timeScale),
          (this.entity.element.opacity = t.x);
      })
      .yoyo(!0)
      .repeat(2)
      .start();
  }),
  (DisableAd.prototype.update = function (t) {});
var SausageTween = pc.createScript("sausageTween");
(SausageTween.prototype.initialize = function () {}),
  (SausageTween.prototype.update = function (e) {});
var TransitionScreen = pc.createScript("transitionScreen");
TransitionScreen.attributes.add("sausage", { type: "entity" }),
  TransitionScreen.attributes.add("newSkin", { type: "entity" }),
  TransitionScreen.attributes.add("shine", { type: "entity" }),
  TransitionScreen.attributes.add("toTheme", { type: "entity", array: !0 }),
  TransitionScreen.attributes.add("levelTexts", { type: "entity", array: !0 }),
  (TransitionScreen.prototype.initialize = function () {
    (this.levelTexts[0].element.text = GameManager.instance.levelId - 1),
      (this.levelTexts[1].element.text = GameManager.instance.levelId),
      (this.levelTexts[2].element.text = GameManager.instance.levelId + 1),
      this.toTheme.forEach((e) => {
        e.sprite.color = new pc.Color().fromString(
          ThemeManager.instance.theme.backgroundColor
        );
      }),
      this.levelTexts.forEach((e) => {
        e.element.color = new pc.Color().fromString(
          ThemeManager.instance.theme.backgroundColor
        );
      }),
      (this.entity.findByName("Background").sprite.color =
        new pc.Color().fromString(
          ThemeManager.instance.theme.backgroundElementsColor
        )),
      Object.values(GameManager.instance.skinList.resources[0]).find(
        (e) => e.level == GameManager.instance.levelId + 1
      ) && (this.newSkin.enabled = !0),
      this.entity.setLocalScale(0, 0, 0),
      this.entity
        .tween(this.entity.getLocalScale())
        .to(new pc.Vec3(1.25, 1.25, 1.25), 0.5, pc.ElasticOut)
        .on("complete", () => {
          this.sausage
            .tween(this.sausage.getLocalPosition())
            .to(
              new pc.Vec3(
                -0.35,
                this.sausage.getLocalPosition().y,
                this.sausage.getLocalPosition().z
              ),
              0.5,
              pc.Linear
            )
            .on("complete", () => {
              this.shine
                .tween(this.shine.getLocalScale())
                .to(new pc.Vec3(1, 1, 1), 2, pc.ElasticOut)
                .start(),
                setTimeout(() => {
                  this.entity
                    .tween(this.entity.getLocalScale())
                    .to(new pc.Vec3(0, 0, 0), 0.5, pc.ElasticIn)
                    .on("complete", () => {
                      GameManager.instance.startLevel(),
                        this.entity.parent.destroy();
                    })
                    .start();
                }, 500);
            })
            .start();
        })
        .start();
  }),
  (TransitionScreen.prototype.update = function (e) {});
var PlayerController = pc.createScript("playerController");
PlayerController.attributes.add("head", { type: "entity" }),
  PlayerController.attributes.add("tail", { type: "entity" }),
  PlayerController.attributes.add("segmentMass", {
    type: "number",
    default: 1,
  }),
  PlayerController.attributes.add("coolDownTime", { type: "number" }),
  PlayerController.attributes.add("sausageSegments", {
    type: "entity",
    array: !0,
  }),
  PlayerController.attributes.add("dashMeter", { type: "entity" }),
  PlayerController.attributes.add("eyes", { type: "entity", array: !0 }),
  PlayerController.attributes.add("force", { type: "number", default: 9 }),
  PlayerController.attributes.add("forceThresholds", {
    title: "Force Thresholds",
    type: "vec2",
    default: [0.5, 2.5],
  }),
  PlayerController.attributes.add("springMultiplier", {
    type: "number",
    default: 1,
  }),
  PlayerController.attributes.add("springs", { type: "entity", array: !0 }),
  PlayerController.attributes.add("landParticles", {
    type: "asset",
    assetType: "template",
  }),
  PlayerController.attributes.add("debugColor", {
    title: "Debug Color",
    description: "The color of the debug rendering of the constraint.",
    type: "rgb",
    default: [1, 0, 0],
  }),
  PlayerController.attributes.add("activated", {
    type: "boolean",
    default: !0,
  }),
  PlayerController.attributes.add("debug", { type: "boolean", default: !0 }),
  (PlayerController.prototype.initialize = function () {
    if (
      ((PlayerController.instance = this),
      (this.headOrTail = this.sausageSegments[0]),
      (this.screenToWorldRatio =
        (2 * GameManager.instance.camera.camera.orthoHeight) /
        window.innerHeight),
      (this.startPos = new pc.Vec2()),
      (this.attached = !1),
      (this.loaded = !1),
      (this.segment = this.head),
      (this.stuckSegment = this.segment),
      (this.offset = new pc.Vec2()),
      (this.coolDown = !1),
      (this.firstMouseUp = !0),
      (this.aiming = !1),
      (this.power = 0),
      (this.maxPower = 1.6),
      !GameManager.instance.keyboard)
    ) {
      var t = this.app.touch;
      t
        ? (t.on(pc.EVENT_TOUCHSTART, this.onMouseDown, this),
          t.on(pc.EVENT_TOUCHMOVE, this.onMouseMove, this),
          t.on(pc.EVENT_TOUCHEND, this.onMouseUp, this))
        : (this.app.mouse.on(pc.EVENT_MOUSEMOVE, this.onMouseMove, this),
          this.app.mouse.on(pc.EVENT_MOUSEDOWN, this.onMouseDown, this),
          this.app.mouse.on(pc.EVENT_MOUSEUP, this.onMouseUp, this)),
        this.on(
          "destroy",
          function () {
            (PlayerController.instance = void 0),
              this.app.mouse &&
                (this.app.mouse.off(pc.EVENT_MOUSEDOWN, this.onMouseDown, this),
                this.app.mouse.off(pc.EVENT_MOUSEUP, this.onMouseUp, this)),
              this.app.touch &&
                (this.app.touch.off(
                  pc.EVENT_TOUCHSTART,
                  this.onMouseDown,
                  this
                ),
                this.app.touch.off(pc.EVENT_TOUCHEND, this.onMouseUp, this),
                this.app.touch.off(pc.EVENT_TOUCHMOVE, this.onMouseMove, this));
          },
          this
        );
    }
    this.app.keyboard.on(pc.EVENT_KEYDOWN, this.pressedKey, this),
      this.app.keyboard.on(pc.EVENT_KEYUP, this.releasedKey, this),
      this.sausageSegments.forEach((t) => {
        t.collision.on(
          "collisionstart",
          (e) => {
            this.onCollisionEnter(e, t);
          },
          this
        );
      }),
      setTimeout(() => {
        this.springs.forEach((t) => {
          (t.script.springJoint.stifness =
            t.script.springJoint.initialStifness * this.springMultiplier),
            (t.script.springJoint.damping =
              t.script.springJoint.initialStifness * this.springMultiplier),
            t.script.springJoint.createConstraint();
        });
      }, 100),
      this.on("attr", function (t, e, s) {
        "springMultiplier" === t && this.changeSpringStifness(e);
      });
  }),
  (PlayerController.prototype.changeSpringStifness = function (t) {
    this.springs.forEach((e) => {
      (e.script.springJoint.stifness =
        e.script.springJoint.initialStifness * this.springMultiplier * t),
        (e.script.springJoint.damping =
          e.script.springJoint.initialStifness * this.springMultiplier * t),
        e.script.springJoint.createConstraint();
    });
  }),
  (PlayerController.prototype.hingeLimits = function (t) {
    this.sausageSegments.forEach((e, s) => {
      s > 0 &&
        (e.script.hingeConstraint.limits = t
          ? new pc.Vec2(-7, 7)
          : new pc.Vec2(-45, 45));
    });
  }),
  (PlayerController.prototype.onMouseDown = function (t) {
    let e, s;
    this.activated &&
      (t.hasOwnProperty("touches")
        ? ((e = t.touches[0].touch.clientX), (s = t.touches[0].touch.clientY))
        : ((e = t.x), (s = t.y)),
      (!this.loaded && !this.debug) ||
        GameManager.instance.finished ||
        this.paused ||
        ((this.mouseDown = !0),
        this.dashMeter.setLocalScale(0, 0, 0),
        (this.dashMeter.enabled = !0),
        (this.app.timeScale = 0.1),
        (this.startPos = new pc.Vec2(e, s)))),
      t.event.preventDefault();
  }),
  (PlayerController.prototype.onMouseMove = function (t) {
    let e, s;
    t.hasOwnProperty("touches")
      ? ((e = t.touches[0].touch.clientX), (s = t.touches[0].touch.clientY))
      : (!t.buttons[0] && this.mouseDown && (this.mouseDown = !1),
        (e = t.x),
        (s = t.y)),
      this.activated &&
        !this.paused &&
        this.mouseDown &&
        ((this.mouseMoved = !0),
        (this.mousePos = new pc.Vec2(e, s)),
        (this.delta = this.mousePos
          .sub(this.startPos)
          .mulScalar(this.screenToWorldRatio)),
        (this.offset = new pc.Vec2()
          .copy(this.delta)
          .normalize()
          .mulScalar(
            clamp(
              this.delta.length(),
              1.25 * this.forceThresholds.x,
              1.25 * this.forceThresholds.y
            )
          )),
        this.dashMeter.setLocalScale(
          this.offset.length() / 5,
          this.offset.length() / 5,
          0
        )),
      t.event.preventDefault();
  }),
  (PlayerController.prototype.onMouseUp = function (t) {
    (this.loaded || this.debug) &&
      !this.paused &&
      this.mouseDown &&
      (null != this.delta &&
        this.delta != pc.Vec3.ZERO &&
        this.delta.length() >= 1 &&
        (this.unstick(), this.dash(this.processForce(this.delta))),
      (this.app.timeScale = 1),
      (this.mouseDown = !1),
      (this.dashMeter.enabled = !1),
      (this.mouseMoved = !1)),
      this.unPause &&
        ((this.paused = !1), (this.unPause = !1), (this.spaceDown = !1)),
      t && t.event.preventDefault();
  }),
  (PlayerController.prototype.keyboardControl = function () {}),
  (PlayerController.prototype.pressedKey = function (t) {
    "ArrowLeft" == t.event.code && (this.rightKeyDown = !0),
      "ArrowRight" == t.event.code && (this.leftKeyDown = !0),
      "Space" == t.event.code && (this.spaceDown = !0),
      (this.spaceDown || this.leftKeyDown || this.rightKeyDown) &&
        !this.aiming &&
        this.loaded &&
        this.attached &&
        ((this.degree = -45), (this.delta = new pc.Vec2()), (this.aiming = !0));
  }),
  (PlayerController.prototype.releasedKey = function (t) {
    (this.loaded || this.debug) && this.paused,
      this.unPause &&
        ((this.app.timeScale = 1),
        (this.paused = !1),
        (this.unPause = !1),
        (this.spaceDown = !1),
        (this.aiming = !1),
        (this.rightKeyDown = !1),
        (this.leftKeyDown = !1)),
      "ArrowLeft" == t.event.code && (this.rightKeyDown = !1),
      "ArrowRight" == t.event.code && (this.leftKeyDown = !1),
      this.spaceDown ||
        this.leftKeyDown ||
        this.rightKeyDown ||
        (!this.loaded && !this.debug) ||
        GameManager.instance.finished ||
        !this.activated ||
        this.paused ||
        setTimeout(() => {
          this.spaceDown ||
            this.leftKeyDown ||
            this.rightKeyDown ||
            (!this.loaded && !this.debug) ||
            GameManager.instance.finished ||
            !this.activated ||
            this.paused ||
            !this.attached ||
            ((this.aiming = !1),
            (this.app.timeScale = 1),
            (this.spaceDown = !1),
            this.unstick(),
            (this.dashMeter.enabled = !1),
            this.dash(this.processForce(this.delta)));
        }, 250),
      t && t.event.preventDefault(),
      t.event.preventDefault();
  }),
  (PlayerController.prototype.processForce = function (t) {
    let e = t.clone().normalize(),
      s = clamp(t.length(), 0.5, 2.5);
    return e.mulScalar(s * this.force);
  }),
  (PlayerController.prototype.dash = function (t) {
    let e = new Ammo.btVector3();
    this.sausageSegments.forEach((t, s) => {
      t.rigidbody.body.setActivationState(1),
        t.rigidbody.body.activate(),
        e.setValue(0, 0, 0),
        t.rigidbody.body.setLinearVelocity(e),
        e.setValue(0, 0, 0),
        this.sausageSegments[2].rigidbody.body.setAngularVelocity(e);
    }),
      this.eyes.forEach((t, s) => {
        this.eye &&
          (e.setValue(0, 0, 0), t.rigidbody.body.setLinearVelocity(e));
      }),
      (this.coolDown = !0),
      setTimeout(() => {
        this.coolDown = !1;
      }, this.coolDownTime),
      console.log("force", t.length()),
      this.headOrTail.rigidbody.body.applyImpulse(
        new Ammo.btVector3(t.x, -t.y, 0)
      );
  }),
  (PlayerController.prototype.unstick = function (t) {
    if (
      this.activated &&
      this.attached &&
      !this.paused &&
      this.stuckSegment.children[0]
    ) {
      console.log("Segment 2", this.stuckSegment),
        (this.stuckSegment.children[0].script.hinge.enabled = !1),
        this.hingeLimits(!0),
        this.changeSpringStifness(8),
        (this.loaded = !1),
        (this.attached = !1);
    }
  }),
  (PlayerController.prototype.stick = function (t, e) {
    (this.attached = !0), (this.loaded = !0);
    let s = t.children[0].script.hinge;
    (this.stuckSegment = t), console.log("Segment", this.stuckSegment);
    let i = 0;
    this.sausageSegments.forEach((t) => {
      let e = this.stuckSegment.getPosition().sub(t.getPosition()).length();
      e <= i || ((i = e), (this.headOrTail = t));
    }),
      this.hingeLimits(!1),
      this.changeSpringStifness(4),
      (s.entityB = e.other),
      (s.pivotA = e.contacts[0].localPoint),
      (s.pivotB = e.contacts[0].localPointOther),
      s.createConstraint(),
      (s.enabled = !0);
    let o = this.landParticles.resource.instantiate();
    t.addChild(o),
      o.particlesystem.play(),
      o.setLocalPosition(0, 0, 0),
      CameraManager.instance.flashOutline(0.5);
  }),
  (PlayerController.prototype.onCollisionEnter = function (t, e) {
    this.attached ||
      this.coolDown ||
      t.other.tags._list.includes("NotSticky") ||
      ((this.oldEntity = t),
      t.other.tags._list.includes("Segment") ||
        (this.stick(e, t), (this.mouseDown = !1)));
  }),
  (PlayerController.prototype.death = function () {
    PokiManager.instance.interstitial(() => {
      (this.activated = !1),
        (GameManager.instance.respawned = !0),
        GameManager.instance.startLevel(!0);
    });
  }),
  (PlayerController.prototype.update = function (t) {
    if (
      (this.app.keyboard.isPressed(pc.KEY_SPACE) && this.keyboardControl(),
      this.app.keyboard.isPressed(pc.KEY_RIGHT) && (this.degree += 5),
      this.app.keyboard.wasPressed(pc.KEY_RIGHT) && (this.degree -= 5),
      this.app.keyboard.isPressed(pc.KEY_LEFT) && (this.degree -= 5),
      this.aiming)
    ) {
      console.log("aiming"),
        this.dashMeter.setLocalScale(0, 0, 0),
        (this.dashMeter.enabled = !0),
        (this.app.timeScale = 0.1);
      this.forceThresholds.x;
      var e = (this.degree * Math.PI) / 180;
      (this.delta.x = Math.cos(e)),
        (this.delta.y = Math.sin(e)),
        this.delta.mulScalar(100),
        (this.offset = new pc.Vec2()
          .copy(this.delta)
          .normalize()
          .mulScalar(
            clamp(
              this.delta.length(),
              1.25 * this.forceThresholds.x,
              1.25 * this.forceThresholds.y
            )
          )),
        this.dashMeter.setLocalScale(
          this.offset.length() / 5,
          this.offset.length() / 5,
          0
        );
    }
    this.head.getPosition().y < -30 &&
      !GameManager.instance.finished &&
      this.activated &&
      this.death(),
      this.dashMeter.enabled &&
        (this.dashMeter.setPosition(this.headOrTail.getPosition()),
        this.dashMeter.setRotation(
          new pc.Quat().setFromEulerAngles(
            0,
            0,
            (180 *
              Math.atan2(
                -this.offset.normalize().y,
                this.offset.normalize().x
              )) /
              Math.PI
          )
        ));
  });
var OnBoardingKeyboard = pc.createScript("onBoardingKeyboard");
(OnBoardingKeyboard.prototype.initialize = function () {}),
  (OnBoardingKeyboard.prototype.update = function (o) {});

//# sourceMappingURL=__game-scripts.js.map
